% Script name: actu2.m
%
% Author: David Schonberger
% EECS Dept., University of Kansas
%
% Created: June 26, 2000.
%
% Last modified: July 10, 2000
%
%**************************************************
%*************************************************** 
% Models servovalve and hydraulic cylinder function
% in a hydraulic actuator.
%
% Model works in three main steps:
%
% 1) Specifiy viscous damping coeff, coloumb friction, 
% stiction, load force, and piston area. Sweep values of 
% y_dot (piston velocity) and solve for differential
% pressure, P.
%
% Note: This assumes y_dot_dot (accel) is negligible
% due to fast rise time. Hence no dependence upon
% piston and load mass. 
%
%
% 2) For a given differential pressure P, found in 1), calculate change
% in flow induced by a small change in this 
% pressure P. This is done via the linearized equation
% delta(Q) = k_q*delta(U) - k_c*delta(P).
% The coefficients k_q and k_c are estimated
% as delta(Q)/delta(U), and delta(Q)/delta(P),
% respectively. 
% 
% Yields a matrix of (P/P_s, U/U_max)
% pairs, each of which gives a change in flow, delta(Q).
% We plot this surface.
%
%
% 3) Next, do power calculations, as follows.
% First, piston output power is F_o * (pist. velocity).
% F_o, output force, is given by F_o = A_P*P - opposing forces.
% Thus given an area and a (velocity, pressure) pair, the piston
% output force may be calculated. To find power needed from engine,
% work backwards through a series of assumed efficiencies:
%
% a) 90% efficiency in converting engine power to
%	torque/angular velocity for running pump
%	torque motor.
%
% b) 60% efficiency in converting torque/angular velocity
%	into hydraulic power, P_s*Q_s.
%
% c) 90-95% efficiency in the servovalve.
%
% d) 99% efficiency in piston. Opposing forces
% 	are alrady taken into account, so leakage is the
%	only other quantity considered here.
%
% There are other power losses possible:  loss due to movement 
% of the entire cylinder housing, for instance. They are 
% ignored here.
%**********************************************************
%**********************************************************
% BEGIN SCRIPT

clear

%-----------------------------------------------------
% STEP 1--sweep dy/dt, find P
%-----------------------------------------------------

% Parameters
A_p = 3;	% a typical piston area (in^2)
ApLow = 2;	% lower bound
ApHigh = 4;	% upper bound
F_col = 50;	% coloumb friction (lbs)
F_stk = 75;	% stiction (lbs)
F_L = 6000;	% load force (lbs)
Beta_v = 20;	% viscous damping coefficient (lbs-sec/in)
P_s = 4000;	% system pressure, assumed to be maxed (lbs/in^2)
Q_max = 125;	% max flow rate (in^3/sec)
U_max = 0.5;	% max valve opening (in)


%**********
% efficiencies 
eff_eng = 0.9;
eff_pump = 0.6;
eff_valve = 0.9;
eff_piston = 0.99;
%**********

%**********
% determination of max power available to an actuator follows:
maxEngPow = 24000000;	% max available engine power (ft-lbs/sec)
maxHydPowPercent = 0.005; % percent of engine power available 
                          % for hydraulic systems

maxHydPow = maxEngPow * maxHydPowPercent;  % max power available for
                                           % hydraulic systems

maxActuPowPercent = 0.125;   % percent of hydraulic power available to
                             % an actuator

maxActuPow = maxHydPow * maxActuPowPercent;  % max power available to an
                                             % actuator
%**********

% incremental percentage changes in P and U for finding
% delta Q in linearized eqn.
dp_percent = 0.03;
du_percent = 0.03;

%*****range of velocities*****
ydLow = 0;
ydInc = 1;
ydHigh = 40;
%*****

%*****velocity sweep*****

y_dot = ydLow : ydInc : ydHigh;
signYd = sign(y_dot);

numInt = floor((ydHigh - ydLow)/ydInc) + 1;


% Eqn from cyl_piston facet, used to sweep y_dot 
% to get P vals & from those,  P/P_s vals
P = (y_dot.*Beta_v + signYd.*F_col + F_L)/A_p;
P_Ps = P./P_s;
%*****

% High and low vals for U/U_max
uumLow = -1;
uumHigh = 1;

% set increment size for U/U_max
inc = (uumHigh - uumLow)/ numInt * 0.99;

% init U/U_max...
U_Umax = zeros(1,numInt);

% fill U/U_max with vals...
for i = 1 : numInt
   U_Umax(1,i) = uumLow + (i-1)*inc;
end

% replace last val with uumHigh val
U_Umax(1,numInt) = uumHigh;

% Replace zero entry in U_Umax (if any) with nonzero val
% to avoid division by zero
for i = 1 : numInt
    if U_Umax(1,i) == 0
        U_Umax(1,i) = inc/2;
   end
end

% Vector of signs for U/U_max
sgn_U = sign(U_Umax);

% matrix init
k_c_dp = zeros(numInt, numInt);
k_q_du = zeros(numInt, numInt);
Q_QM_low = zeros(numInt, numInt);
Q_QM_high = zeros(numInt, numInt);
U_UM_low = zeros(numInt, numInt);
U_UM_high = zeros(numInt, numInt);

%---------------------------------------------------
% STEP 2--calculate delta(Q). Done in three substeps.
%---------------------------------------------------

%*****
% STEP 2a)--Find k_c, the flow pressure coefficient, given by dq/dp
%*****

%**********
% Basic steady stae eqn. from servo facet:
% Q_Qmax = U_Umax.*sqrt(1 - P_Ps.*sgn_U);
%**********

% change in P/P_s
P_PS_inc = 0.005;	% 1/2 percent change either direction

%*****Estimate k_c & k_c*delta(P)*****
for i = 1 : numInt
   for j = 1 : numInt
	Q_QM_low(i,j) = U_Umax(1,j)*sqrt(1 - P_Ps(1,i)*sgn_U(1,j)*(1 - P_PS_inc));
	Q_QM_high(i,j) = U_Umax(1,j)*sqrt(1 - P_Ps(1,i)*sgn_U(1,j)*(1 + P_PS_inc));
	k_c_dp(i,j) = -Q_max/P_s*(Q_QM_high(i,j) - Q_QM_low(i,j))/...
	(2*P_PS_inc*P_Ps(1,i))*(dp_percent*P(1,i));
   end
end
%**********

%*****
% STEP 2b)--Find k_q, flow gain, given by dq/du
%*****

% change in U_Umax
U_UM_inc = 0.005;	% 1/2 percent

%*****Estimate k_q & k_q*delta(U)*****
for i = 1 : numInt
   for j = 1 : numInt
	Q_QM_low(i,j) = U_Umax(1,i)*(1 - U_UM_inc)*sqrt(1 - P_Ps(1,j)*sgn_U(1,i));
	Q_QM_high(i,j) = U_Umax(1,i)*(1 + U_UM_inc)*sqrt(1 - P_Ps(1,j)*sgn_U(1,i));
	k_q_du(i,j) = Q_max/U_max*(Q_QM_high(i,j) - Q_QM_low(i,j))/...
	(2*U_UM_inc*U_Umax(1,i))*(du_percent*U_Umax(1,i)*U_max);
   end
end
%**********

%*****
% STEP 2c) Subtract matrices from 2a) and 2b), plot change in flow.
%*****

deltaQ = k_c_dp - k_q_du; 

% surface plot of deltaQ with P_Ps and U_Umax on x- & y-axes, resp.
%subplot(2,2,1)
surf(P_Ps, U_Umax,deltaQ);
view([-0.4 0.2 0.5])
xlabel('P/Ps');
ylabel('U/Umax');
zlabel('DeltaQ (in^3/sec)');
title('Change in flow vs pressure and servo spool opening, A_p = 3 in^2');
pause;

%---------------------------
% STEP 3--Power Calculations
%---------------------------

% calculate increment size for area sweep
ApInc = (ApHigh - ApLow)/numInt*0.99;

% init piston area vector...
A_pVec = zeros(1,numInt);

% fill area vector
for i = 1 : numInt
   A_pVec(1,i) = ApLow + (i - 1)*ApInc ; 
end

% replace last val with high A_P val
A_pVec(1, numInt) = ApHigh;

%*****Fixed upper bound on power to actuator*****
maxExtraPow = zeros(numInt,numInt);
for i = 1 : numInt;
   for j = 1 : numInt
	maxAPow(i,j) = maxActuPow;	% Hypothetical  max available 
					% extra engine power. 
   end
end

% dummy vector for y axis in plot
% low val => low (v,p) pair
% high val => high (v,p) pair
dummyVec = zeros(1, numInt);
for i = 1 : numInt
  dummyVec(1,i) = i/numInt;
end
%*****

%***** Generate vectors for "spikes" showing
%operating points

%parameters
highpow = 60000;
lowVP = 0.2;
lowArea = 2.5;
highVP = 0.7;
highArea = 2.75;

%vector init
vlVec = zeros(1, numInt);
opPtOneXVec = zeros(1, numInt);
opPtOneYVec = zeros(1,numInt);
opPtTwoXVec = zeros(1, numInt);
opPtTwoYVec = zeros(1,numInt);
opPtThreeXVec = zeros(1, numInt);
opPtThreeYVec = zeros(1,numInt);

%vector given values
for i = 1 : numInt
   vlVec(1,i) = highpow/numInt*(i-1)*0.99;
   opPtOneXVec(1,i) = lowArea;
   opPtOneYVec(1,i) = lowVP;
   opPtTwoXVec(1,i) = highArea;
   opPtTwoYVec(1,i) = lowVP;
   opPtThreeXVec(1,i) = highArea;
   opPtThreeYVec(1,i) = highVP;
end
%*****

% Calculate piston output power. Each row in the 
% resulting matrix is for a fixed area. The area range
% is set above. Each column is for a fixed (v,p) pair.
pistonPOut = zeros(numInt, numInt);
for i = 1 : numInt
   for j = 1 : numInt
	pistonPOut(i,j) = y_dot(1,j)*(A_pVec(1,i)*P(1,j) - F_col - ...
	-F_stk - Beta_v*y_dot(1,j));
   end
end

% Calculate draw on engine power as piston output power
% divided by various efficiencies. Factor of 12 is to convert
% from in-lbs/sec to ft-lbs/sec.
PEngine = pistonPOut./(eff_eng*eff_pump*eff_valve*eff_piston*12);

%Plot--Power constraint constant surface
surf(A_pVec, dummyVec, maxAPow);
xlabel('Piston area (in^2)');
ylabel('(yDot, diff. pressure) pairs');
zlabel('Engine power consumed (ft-lbs.sec)');
title('Power Constraint');
pause;

%Plot--Power consumption vs area and (v,p) pairs
%subplot(2,2,3);
surf(A_pVec, dummyVec, PEngine);
xlabel('Piston area (in^2)');
ylabel('(yDot, diff. pressure) pairs');
zlabel('Power (ft-lbs.sec)');
title('Power consumed by actuator');
pause;

%Plot--Pow consumption plus two spikes, no
%pow constraint surf
surf(A_pVec, dummyVec,PEngine);
hold on
plot3(opPtOneXVec, opPtOneYVec, vlVec,'k');
plot3(opPtTwoXVec, opPtTwoYVec, vlVec,'k');
xlabel('Piston area (in^2)');
ylabel('(yDot, diff. pressure) pairs');
zlabel('Power (ft-lbs.sec)');
title('Two operating points, area increased, no violation');
hold off
pause;

%*****Show me the power violation!*****
%subplot(2,2,4)
surf(A_pVec, dummyVec, maxAPow);
hold on
surf(A_pVec, dummyVec, PEngine);
plot3(opPtOneXVec, opPtOneYVec, vlVec,'k');
plot3(opPtTwoXVec, opPtTwoYVec, vlVec,'k');
plot3(opPtThreeXVec, opPtThreeYVec, vlVec,'k');
xlabel('Piston area (in^2)');
ylabel('(yDot, diff. pressure) pairs');
zlabel('Engine power consumed (ft-lbs.sec)');
title('Power Violation!');
hold off


