//*********************************************
//filename: actuator.sld
//
// The actuator is an instance of the servo valve and the cylinder piston
// interconnected via hydraulic fluid flow.  Quantities of interest incude:
//
//    Q_S - hydraulic souce flow rate
//    P_S - hydralic source pressure
//    U - servo valve spool displacement
//    Y - piston displacement
//    D_c - cylinder diameter
//    A_P - piston area
//
// Power is observed using the quantity actPower, an internal variable
// within the actuator model.
//
//Authors: Prof. Perry Alexander, David Schonberger
//Last modified: June 30, 2000
//*********************************************

use realTypes, servovalve, cyl_piston;

facet actuator (
    U::real; //spool displacement (in).
    Q_S::posReal; //source volume flow rate (in^3/sec)
    P_S::posReal; //source pressure (lbs/in^2)

    b::posReal; // land width (in)
    rho::posReal; // hydraulic fluid density (lbs/in^3)
    c_d::posReal; // discharge coefficient at each port

    Y :: real; //piston shaft linear displacement (in)
    D_c :: posReal; // cylinder internal diameter (in)
    A_p :: posReal; // piston area (in^2)
    V_1_0, V_2_0 :: posReal; // exaust & inlet side cylinder volumes for
                            // Y = 0
    Beta_v :: real; //viscous damping of piston (lbs-sec/in)
    Beta :: posReal; //bulk modulus of hydraulic fluid (lbs/in^2)
    F_stk :: real; // "sticktion" force (lbs)
    M_p,M_L :: posReal; // Masses of piston and load
    F_L :: real; // Load force (lbs)
    F_0 :: real // Output force (lbs)
) is


    Q_1,Q_2 :: posReal; //exhaust & inlet volume flow rate (in^3/sec)
    P_1,P_2 :: posReal; //exhaust & inlet pressure (lbs/in^2)
    actPower :: posReal; // actuator power consumption

begin logic

  valve: servovalve(Q_1,Q_2,P_1,P_2,U,Q_S,P_S,b,rho,c_d);
  piston: cyl_piston(Q_1,Q_2,P_1,P_2,Y,D_c,A_p,V_1_0,V_2_0,Beta_v,
                     Beta,F_stk,M_p,M_L,F_L,F_0);
  p: actPower = valve.power + piston.power;

end actuator;
