//************************************************
// Rosetta model of the hydraulic cylinder/piston
// portion of an electrohydraulic actuator.
// Composed of three facets:
// 1) cyl_piston_fcn--describes the mathematical
//	relationships that govern the hydraulic cylinder's
//	functionality.
// 2) cyl_piston_power--states a conservation of power equation.
// 3) cyl_piston--defines a cyl_piston as the conjunction
//	of the first two facets.
// Authors: Dr. Perry Alexander and David Schonberger
// Last modified: June 28, 2000
//**************************************************

use realTypes;

package cyl_piston is

begin logic 
facet cyl_piston_fcn(
    Q_1,Q_2 :: real; //exhaust & inlet volume flow rate (in^3/sec)
    P_1,P_2 :: real; //exhaust & inlet pressure (lbs/in^2)
    Y :: real; //piston shaft linear displacement (in)
    D_c :: posReal; // cylinder internal diameter (in)
    A_p :: posReal; // piston area (in^2)
    V_1_0, V_2_0 :: posReal; // exaust & inlet side cylinder volumes for
                            // Y = 0 (in^3)
    Beta_v :: real; //viscous damping of piston (lbs-sec/in)
    F_col :: real;  // coloumb friction
    F_stk :: real; // "sticktion" force (lbs)
    M_p,M_L :: posReal; // Masses of piston and load (slugs)
    F_L :: real; // Load force (lbs)
    F_0 :: real // Output force (lbs)
  ) is

//internal vars

  A_c :: posReal; 	//cylinder internal cross-sectional area (in^2)
  V_1 :: nonNegReal;	//exhaust-side cylinder volume (in^3)
  V_2 :: nonNegReal;	//Inlet-side cylinder volume (in^3)
  V_p :: real is deriv(Y,t);          //Piston velocity (in/sec)
  Q :: real is (Q_1+Q_2)/2;	//average flow rate (in^3/sec) - delete this
  P :: real is P_2 - P_1; 	//differential pressure
  eff_piston :: real is 0.99;  	//piston efficiency, assumed to be 99%

begin continuous

  F1: A_c = pi*(D_c/2)^2;

//Calculation of continuous exhaust and inlet side
//volumes from initial value and displacement (Y)
  F2: V_2 = V_2_0 + A_p * Y;
  F3: V_1 = V_1_0 - A_p * Y;

//Total volume of the piston is constant over time
  C2: V_1 + V_2 = V_2_0 + V_1_0;

//Calculate the output force is ideal force minus stiction
//and friction
  F4: F_0 = A_p*P - F_stk - F_col - Beta_v*V_p;

//Newton says MA=F
  F5: (M_p + M_L)*deriv(V_p,t) = F_0 - F_L;

//Assuming fast rise time, piston acceleration becomes negligible
//and we consider a steady state equation relating velocity, area,
//pressure and forces opposing piston motion. This formula is obtainable
//from F4 & F5 by assuming deriv(V_p,t) = 0. It is used in the Matlab 
//model actu2.m, along with the nonlinear servovalve eqn, to determine
//flow changes as a function of changes in pressure & servo spool motion.
//It is also used by itself to calculate piston output power.
  F6: P = (Beta_v*V_p +  F_col + F_stk + F_L)/A_p;

//Calculate the inlet and exhaust flow rates with the assumption
//that the hydraulic fluid is not compressible. Not currently
//used in Matlab model actu2.m. Would be useful if a study of dynamic
//piston motion were undertaken.
  F6: Q_2 = A_p*deriv(Y,t);
  F7: Q_1 = A_p*deriv(Y,t);

end cyl_piston_fcn;


facet cyl_piston_power(
    Q_1,Q_2 :: real; //exhaust & inlet volume flow rate (in^3/sec)
    P_1,P_2 :: real; //exhaust & inlet pressure (lbs/in^2)
    power :: posReal; //Cylinder power (ft-lbs/sec)
    PowOut_piston :: posReal; //piston output power (ft-lbs/sec)
    V_p :: real;	//piston velocity (in/sec)
    A_p :: posReal;	//piston effective area (in^2)
    F_stk :: real;	//stiction force (lbs)
    F_col :: real;	//coloumb friction (lbs)
    F_L :: real		//load force (lbs)
  ) is

use realTypes;

begin continuous
  F1: power = Q_2*P_2 - Q_1*P_1;
  F2: PowOut_piston = V_p*(A_p*P - F_stk - F_col - F_L)*eff_piston;
end cyl_piston_power;


// Define the piston as the conjunction of the piston functional model and the
// piston power model.

cyl_piston :: facet is cyl_piston_fcn and cyl_piston_power;

end cyl_piston;
