//*************************************************************
// Filename: servovalve.sld
//
// Rosetta model of the servovalve portion of an
// electrohydraulic actuator. 
// Composed of three facets: 
// 1) servovalve_fcn--describes the mathematical relationships
//	that govern the servovalve's functionality.
// 2) servovalve_power--states conservation of power equation
// 3) servovalve--defines a valve as consisting the conjunction
//	of the first two facets. 
//
// Authors: Dr. Perry Alexander and David Schonberger
// Last modified: June 30, 2000
//**************************************************************

use realTypes;

package servovalve is

begin logic

facet servovalve_fcn(
    U::real; //Spool displacement (in).  U positive indicates 
             //Q_2 is flow out of valve to hydraulic cylinder,
	     //and Q_1 is flow from cylinder to valve.
	     //U negative indicates the reverse is true.
 
    P_S::posReal; //source pressure (lbs/in^2)
    Q_1 :: real;  //flow rate too/from servo (in^3/sec)
    Q_2 :: real;
    U_max::posReal; //maximum spool displacement (in)
    b::posReal; // land width (in)
    rho::posReal; // hydraulic fluid density (lbs/in^3)
    c_d::posReal // discharge coefficient at each port (units???)
) is

  P :: real is P_2-P_1;	    //differential pressure (lbs/in^2)
  Q :: real is (Q_1+Q_2)/2; //average flow rate (in^3/sec)
  Q_max :: real is U_max * b * c_d * (P_s/rho)^0.5; //maximum flow
                                                    //into servo valve
						    // (in^3/sec)
 //Rosetta has no built-in signum function
 //for computing the sign of a real number.
 //The following defines this function.
  sgn(x::real)::real is 
	if x /= 0 then x/abs(x)
	else 0 endif;

begin continous

  //Nonlinear steady state valve equation.
  F1: Q/Q_max = U/U_max * ((1 - P/P_S) * sgn(U/U_max))^0.5;
  
  //Flow, spool disp, diff pressure cannot exceed max
  C1: abs(Q) =< abs(Q_max);
  C2: abs(U) =< abs(U_max);
  C3: abs(P) =< abs(P_S);
end servovalve_fcn;


facet servovalve_power(
    Q_1,Q_2:: real; //flow rate in and out of servovalve (in^3/sec)
    P_1,P_2:: real; //presure in and out of servovalve (lbs/in^2)
    Q_S::posReal; //source volume flow rate (in^3/sec)
    P_S::posReal; //source pressure (lbs/in^2)
    power::real //power consumed by the servovalve (in-lbs/sec)
		 //Should be set to ft-lbs/sec when used in actual
		 //calculations.
) is

use realTypes;

begin continous
  F1: power=P_S*Q_S - P_2*Q_2 + P_1*Q_1;
end servovalve_power;


servovalve :: facet is servovalve_fcn and servovalve_power;

end servovalve;
