package springAll is 

begin logic

  facet spring ( d :: real;  	// wire diameter
	       OD :: real;	// outside diameter of spring
	       ID :: real;	// inside diameter of spring
	       G :: real;	// Modulus of Rigidity of wire
	       N :: real;	// total number of coils
	       p :: real	// pitch
	     ) is


   pi :: real is 3.1415926452645;

   D :: real is (OD + ID) / 2.0;	// mean diameter of spring
   C :: real is D / d;		// spring constant

   n :: real;			// number of active coils
   H :: real;			// fully compressed height

   R :: real;			// Force/length (lb/inch)
   P :: real;			// force to fully deflect spring

   l :: real;			// maximum spring displacement
   Lm :: real;			// minimum free length
   L :: real;			// desired free length

   Tu :: real;			// ultimate tensile strength of wire
   Ty :: real;			// tensile yield strength
   Sy :: real;			// shear yield strength
   S :: real;			// uncorrected shear stress
   k :: real;			// correction factor
   Sk :: real;			// corrected shear factor

   export R, L;

begin logic

   C1: (OD - ID) / 2.0 = d;	// dependencies between diameter parameters

   C2: C >= 4.0 and C =< 20.0;	// empirical constraints on spring constant

   F1: n = (G * d^4.0) / (8.0 * R * D^3.0);
   C3: n >= 3.0;			// empirical constraint on n

   F2: N = n + 2.0;		// for closed and ground spring
   
   F3: H = d * N;

   F4: Lm = H + l;
   F5: L = H + (l / 0.8);

   F6: p = (L - 2.0*d) / n;
   C4: p =< D;			// pitch can't exceed coil diameter

   F7: Tu = 200000.0 * D^(-0.14);
   F8: Ty = 0.75 * Tu;
   F9: Sy = Ty * 0.577;

   F10: P = R * l;

   F11: S = (8.0 * P * D) / (pi * d^3.0);
   F12: k = (4.0*C - 1.0) / (4.0*C - 4.0) + 0.615 / C;
   F13: Sk = S * k;

   C5: Sk < Sy;			// shear force < wire shear yield strength  

end spring;


facet spring_system is

   outer_d :: real;
   outer_OD :: real;
   outer_ID :: real;
   outer_G :: real is 11.5E6;	// music wire
   outer_N :: real;
   outer_p :: real;
   inner_d :: real;
   inner_OD :: real;
   inner_ID :: real;
   inner_G :: real is 11.5E6;	// music wire
   inner_N :: real;
   inner_p :: real;

   system_L :: real;		// system free length

   system_R :: real;		// system rate
   inner_R :: real;
   outer_R :: real;

   system_deflection :: real;
   inner_deflection :: real;
   outer_deflection :: real;

   force :: real;

begin logic

   outer_spring: spring( outer_d, outer_OD, outer_ID,
			 outer_G, outer_N, outer_p);

   inner_spring: spring( inner_d, inner_OD, inner_ID,
			 inner_G, inner_N, inner_p);

   C1: outer_OD < 2.5;		// outer spring must fit inside 2.5" tube
   C2: outer_ID > inner_OD;	// outer spring must fit over inner spring
   C3: inner_ID > 1.0;		// inner spring must fit over 1.0" tube

   F0: system_L = (outer_spring.L max inner_spring.L);
   C4: system_L < 12.0;

   C5: system_deflection >= 0.0 and system_deflection =< 6.0;

   F1: inner_deflection =
          if system_deflection < (system_L - inner_spring.L) then 0.0
	    else system_deflection - (system_L - inner_spring.L)
        endif;
   F2: outer_deflection =
          if system_deflection < (system_L - outer_spring.L) then 0.0
	  else system_deflection - (system_L - outer_spring.L)
      endif;

   F3: inner_R =
          if inner_deflection = 0.0 then 0.0
	  else inner_spring.R
      endif;
   F4: outer_R =
          if outer_deflection = 0.0 then 0.0
	  else outer_spring.R
      endif;

   F5: system_R = outer_R + inner_R;

   C6: system_R =
	  if system_deflection =< 4.0 then 50.0
	  else 75.0
      endif;

   F6: force = system_R * system_deflection;
//
//  add parenthesis to the boolean expression.
//  Zhongjun, Murthy, Garrin Oct. 26, 2001
//
   A1: ((system_deflection = 0.0) => (force = 0.0))
       and ((system_deflection = 4.0) => (force = 200.0))
       and ( (system_deflection = 6.0) => (force = 350.0));

end spring_system;

end springAll;
