//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Garrin kimmell@ittc.ukans.edu
//               Joseph josephk@cs.adelaide.edu.au
//
//  $Id: Application.java,v 1.4 2002/11/18 18:00:59 kimmell Exp $
//
//  Revision History:
//  ----------------
//  $Log: Application.java,v $
//  Revision 1.4  2002/11/18 18:00:59  kimmell
//  1. Added a method "fixOp" to remove underscores from around operators
//     before printing.
//  -kimmell
//
//  Revision 1.3  2002/07/16 07:15:47  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 20:59:13  kimmell
//  1. Removed instantiate and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.6  2002/05/22 19:39:00  kimmell
//  Altered the constructor for Application so that it properly adds the function reference and parameters to the child list.
//  -kimmell
//
//  Revision 1.1.2.5  2002/04/17 19:55:04  cvs
//  Modified ROM structural elements, such as Item and Label, to inherit from Expression. This allows the meta-functions to operate along the same lines as any other function.
//  -kimmell
//
//  Revision 1.1.2.4  2002/03/19 17:30:36  cvs
//  Fixed a bug in the instantiate() method. The duplicate() method still needs to be fixed.
//  -kimmell
//
//  Revision 1.1.2.3  2002/03/12 22:49:55  cvs
//  Generalized Application to allow application of ANY function. This makes the Formula class deprecated.
//  -kimmell
//
//  Revision 1.1.2.2  2002/01/23 15:57:20  cvs
//  Moved the org.sldl.lang.primitive package to org.sldl.parser.primitive and made the appropriate changes in the source tree.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:14  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.1.2.2  2002/01/04 12:30:46  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.1.2.1  2001/11/24 06:44:44  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Iterator;
import java.util.Map;


/** This represents the <B>Application</B> object. An instance of this
 *  class indicates that the Expression consists of two arguments that
 *  inherintly operate on each other without the need for an infix
 *  operator. For example, a parameter list operates on a function reference.
 * @author <I> Garrin Kimmell and Joseph Kuehn </I>
 * (kimmell@ittc.ukans.edu, josephk@cs.adelaide.edu.au)
 */

public class Application extends Expression {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "Function", "Parameters", "ApplicationType" };


    // The function to be applied
    private Expression function;
    // The list of parameters
    private ExpressionList parameters;
    
    // Used for correct pretty-printing of applications of functions
    // and operators

    public static final int NORMAL = 0;
    public static final int BINARY = 1;
    public static final int PREFIX = 2;
    public static final int POSTFIX = 3;
    private int applicationType = NORMAL; 

    
    
    public Application(Expression argFunction, ExpressionList argParameters)
    {
      this(argFunction, argParameters, NORMAL);
    }
    
    public Application(Expression argFunction, 
		       ExpressionList argParameters, 
		       int argApplicationType) {
      function = argFunction;
      parameters = argParameters;
      applicationType = argApplicationType;
      // System.out.println("Created an application:\n" + this);

      addChildObject(function);
      addChildList(parameters);
    }
    
    
    /** Secondary constructor.
     */
    public Application() {
      //      this(null, null);
    }
   

  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public Application(Map settings)
    {
    super(settings);

    function = (Expression)getSetting(Application.keySet[0]);
    parameters = (ExpressionList)getSetting(Application.keySet[1]);
    if (getSetting(Application.keySet[2]) == null)
      { applicationType = NORMAL; }
    else
      { applicationType = ((Integer)getSetting(Application.keySet[2])).intValue(); }

    addChildObject(function);
    addChildList(parameters);
    }






  public String toString() {
    // KIMMELL: New toString() code;      
    //System.out.println("Beginning Application::toString()");
    String theString = "";
    
    theString = (getParens() ? "(" : "");
    switch (applicationType) {
      
    case NORMAL:
      // System.out.println("New set");
      if (parameters != null) {
	
	theString = theString + function.toString() + "(";
	boolean first = true;
	for (Iterator paramsIter = parameters.iterator();
	     paramsIter.hasNext();){
	  
	  // Add in a comma, if it is not the first parameter
	  theString = first ? theString : theString + ", ";
	  
	  theString = theString + ((Expression)
				   paramsIter.next()).toString();
	  first = false;
	  
	} // end of for (Iterator paramsIter = parameters.iterator();
	  // paramsIter.hasNext();)
      }
      theString = theString + ")";
      break;
      
    case BINARY:
      theString = theString + ((Expression) parameters.get(0)).toString() + " " +
	fixOp(function.toString()) + " " + ((Expression) parameters.get(1)).toString();
      
      break;
      
    case PREFIX:
      theString = theString + fixOp(function.toString())  + ((Expression) parameters.get(0)).toString();
      
      break;
      
    case POSTFIX:
      theString = theString + ((Expression) parameters.get(0)).toString() 
	+ fixOp(function.toString());
      
      break;
      
    } // end of switch (applicationType)
    
      theString = theString + (getParens() ? ")" : "");
      // System.out.println("End of Application::toString()");
      return theString;
      
  }
  

  /* The <code>fixOp</code> removes the underscores from before and
     after an operator's string representation when printing out.
  */
  private String fixOp(String s) {
    int beginning = 0;
    int end = s.length();
    if (s.startsWith("__")) {
      beginning = 2;
    } 
    
    if (s.endsWith("__")) {
      end -= 2;
    } // end of if (s.endsWith("__"))
    
    return s.substring(beginning, end);
  }
    


    public String publishROM(String indent) {
      return  toString();

      
    }


    public int getApplicationType()
      { return applicationType; }

    public void setApplicationType(int type) {
      applicationType = type;
    }



    public Expression getFunction() {
      return function;
    }

    public void setFunction(Expression f) {
      removeChildObject(function);
      function = f;
      addChildObject(function);
    }


    public ExpressionList getParameters() {
      return parameters;
    }

    public void setParameters(ExpressionList l) {
      removeChildList(parameters);
      parameters = l;
      addChildList(parameters);
    }



}




