//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: BaseObject.java,v 1.4 2002/07/16 07:15:47 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: BaseObject.java,v $
//  Revision 1.4  2002/07/16 07:15:47  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.3  2002/07/09 15:12:45  kimmell
//  Added decoration interface.
//  1. decorations is a Map containing all decorations for the object.
//  2. addDecoration adds a key/value pair to the decorations.
//  3. getDecoration returns the decoration associated with a key.
//  4. removeDecoration removes the decoration associated with a key.
//
//  -kimmell
//
//  Revision 1.2  2002/07/08 21:01:58  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  3. Removed "history" utility methods used with matches method.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.3  2002/06/05 16:29:47  kimmell
//  Added a setParent method for manually setting the parent of an object.
//  -kimmell
//
//  Revision 1.1.2.2  2002/03/12 22:51:52  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:15  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.7.2.4  2002/01/04 12:30:46  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.7.2.3  2001/11/24 06:44:44  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//  Revision 1.7.2.2  2001/11/14 19:17:17  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//  Revision 1.7.2.1  2001/11/07 22:41:02  cvs
//  Added treenode functionality to the ROM.
//  -Joseph
//
//  Revision 1.7  2001/03/12 23:56:46  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.6  2000/12/04 21:52:54  cvs
//  BaseObject class now implements Serializable.
//  -- Cindy
//
//  Revision 1.5  2000/12/03 23:59:45  cvs
//  Added the new types of array, set and sequence. The methods
//  getType  and alreadyDeclared are also added to get a type that has
//  been previously declared. 
//  --srinivas
//
//  Revision 1.4  2000/11/17 23:14:13  cvs
//  Added the method labelLookup in some classes and changed the same
//  method in others 
//  --srinivas
//
//  Revision 1.3  2000/08/22 18:40:17  cvs
//  First Commit by Steve Parker, Averstar.  Basic Structure of
//  classes setup for ROM. 
//
//  Revision 1.2  2000/08/09 04:29:51  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//  Revision 1.1  2000/08/08 22:50:55  cvs
//  The BaseObject.java the super class of all objects in the object
//  model.
//  -- Cindy
//
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.io.Serializable;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Map;
import java.util.HashMap;
import javax.swing.tree.TreeNode;


/**
 * This represents the <B>BaseObject</B> object. All ROM objects that
 *  represent constructs in the Rosetta language must inherit from this
 *  object so that design cohesion can be maintained.
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 * @version 1.0
 */
public abstract class BaseObject implements Serializable, TreeNode {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "Decorations" };

  private BaseObject parent;
  private Vector children;
  private SourceReference mySource;

  
  /**
   * The <code>decorations</code> variable is a map that contains all
   * of the decorations for a particluar ROM object. A particular
   * decoration is a key/value pair.
   *
   */
  private Map decorations;

  /** The <code>mySettings</code> variable is a map that contains the
   *  initial values used to construct this BaseObject.
   */
  private Map mySettings;



  /** The primary Constructor.
   */
  public BaseObject() {
    parent = null;
    children = new Vector();
    mySource = null;
    decorations = new HashMap();
  }


  /** Secondary Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public BaseObject(Map settings)
    {
    if (settings != null)
      { mySettings = settings; }
    else
      { mySettings = new HashMap(); }

    parent = null;
    children = new Vector();
    mySource = null;
    decorations = (Map)getSetting(BaseObject.keySet[0]);
    if (decorations == null)
      { decorations = new HashMap(); }
    }


  /** Method to fetch an initial setting from a given key.
   * @param key The key to use to fetch the value.
   * @return Object The value related to that key, or null if none.
   */
  protected Object getSetting(Object key)
    { return mySettings.get(key); }



  /** SourceReference: Method for tracking error details back to the source code it came from.
   * @return SourceReference The reference information of this BaseObject.
   */
  public SourceReference getSourceReference()
  { return mySource; }

  /**
   * SourceReference: Method for tracking error details back to the source code it came from.
   * This method is most appropriately used for atomic syntax productions or
   * productions with 'tails' (ie non-item tokens like 'facet').
   * @param source The new reference information of this BaseObject.
   */
  public void setSourceReference(SourceReference source)
  { mySource = source; }


  /**
   * SourceReference: Method for tracking error details back to the source code it came from.
   *  This method re-evaluates the SourceReferences of all BaseObjects below
   *  this item based upon the SourceReferences below them, etc... It then
   *  updates its own values in a non-decreasing union with its children.
   *  The ordering of the join is important as our own SourceReference
   *  could still be null.
   */
  public void resetSourceReferences()
  {
//      int pos;

//      System.out.println("resetSourceReferences(" + getClass().getName()
//  		       + ")");
//      if (isLeaf())
//        { 
//  	System.out.println("in a leaf");
//  	return; }

//      for (pos = 0; pos < getChildCount(); pos++)
//        {
//  	getChildObjectAt(pos).resetSourceReferences();
//  	if (getChildObjectAt(pos).getSourceReference() != null)
//  	  System.out.println("Source reference: " + mySource);
//  	  System.out.println("child at " + pos + " has a source reference");
//  	  { mySource = getChildObjectAt(pos).getSourceReference().join(mySource); }
//        }
  }


  /**
   * SourceReference: Method for recursively setting all the filenames of
   *  the references below and including this one.
   * @param filename The new filename to use for the SourceReferences.
   */
  public void setSourceReferencesFilename(String filename)
  {
    int pos;

    if (getSourceReference() != null)
      { getSourceReference().setFilename(filename); }

    for (pos = 0; pos < getChildCount(); pos++)
      { getChildObjectAt(pos).setSourceReferencesFilename(filename); }
  }







  /** TreeNode: Returns the children of the reciever as an Enumeration.
   * @return Enumeration An iterator for the children.
   */
  public Enumeration children()
  { return children.elements(); }


  /** TreeNode: Returns true if the receiver allows children.
   * @return boolean Could this node have children?
   */
  public boolean getAllowsChildren()
  { return true; }


  /** TreeNode: Returns the child TreeNode at index childIndex.
   * @param childIndex The position of the child wanted.
   * @return TreeNode This will actually be another BaseObject
   */
  public TreeNode getChildAt(int childIndex)
  {
    if ((childIndex < 0) || (childIndex > children.size()))
      { return null; }
    else
      { return (TreeNode)(children.get(childIndex)); }
  }


  /** TreeNode: Returns the number of children TreeNodes the receiver contains.
   * @return int Number of child objects.
   */
  public int getChildCount()
  { return children.size(); }


  /** TreeNode: Returns the index of node in the receivers children.
   * @param node The child for which the index is wanted.
   * @return int First incidence of node tested with ".equals()".
   */
  public int getIndex(TreeNode node)
  { return children.indexOf(node); }


  /** TreeNode: Returns the parent TreeNode of the receiver.
   * @return TreeNode The parent BaseObject.
   */
  public TreeNode getParent()
  { return parent; }

  /**
   * Describe <code>setParent</code> method here.
   *
   * @param argParent a <code>BaseObject</code> value
   */
  public void setParent(BaseObject argParent) {
    parent = argParent;
  }

  /** TreeNode: Returns true if the receiver is a leaf.
   * @return boolean Does this node have _no_ children?
   */
  public boolean isLeaf()
  { return (children.size() == 0); }







  /** TreeNode Extension: Check to see if an object is a child using "(a == b)".
   * @param child The BaseObject to look for.
   * @return int The position of the object, -1 if not found.
   */
  protected int findChild(BaseObject child)
  {
    int pos;

    for (pos = 0; pos < children.size(); pos++)
      {
	if (child == children.get(pos))
	  { return pos; }
      }

    return -1;
  }


  /** TreeNode Extension: Check to see if an object is a child (uses findChild method).
   * @param child The BaseObject to look for.
   * @return boolean Is the Object a child?
   */
  protected boolean isChild(BaseObject child)
  { return (findChild(child) != -1); }


  /**
   * TreeNode Extension: Register a new child with the tree in a specific position.
   *  Objects with ordered data (facets, functions) can use this when changing a middle
   *  value. If the position is out of range the child is added to the
   * end of the list.
   * @param child The new BaseObject to add to the tree.
   * @param pos The position to add the child to.
   */
  protected void addChildObject(BaseObject child, int pos)
  {
    if ((child != null) && !isChild(child))
      {
	child.parent = this;
	if ((pos < 0) || (pos > children.size()))
	  { children.add(child); }
	else
	  { children.add(pos, child); }
      }
  }


  /**
   * TreeNode Extension: Register a new child with the tree.
   * @param child The new BaseObject to add to the tree.
   */
  protected void addChildObject(BaseObject child)
  {
    if ((child != null) && !isChild(child))
      {
	child.parent = this;
	children.add(child);
      }
  }


  /**
   * TreeNode Extension: De-register a child from the tree.
   * @param child The child to remove.
   */
  protected void removeChildObject(BaseObject child)
  {
    if (child != null)
      {
	int pos;
	pos = findChild(child);
	if (pos > -1)
	  {
	    child.parent = null;
	    children.remove(pos);
	  }
      }
  }


  /** TreeNode Extension: De-register a position-specified child from the tree.
   * @param pos The position of the child being removed.
   * @return BaseObject The child that was removed from the tree (null if pos out of bounds).
   */
  protected BaseObject removeChildObject(int pos)
  {
    if ((pos < 0) || (pos > children.size()))
      { return null; }
    else
      {
	BaseObject child;
	child = (BaseObject)(children.remove(pos));
	child.parent = null;
	return child;
      }
  }


  /**
   * TreeNode Extension: Sugar for registering lists of things at a time.
   * @param listIn List of BaseObject items to be registered
   */
  protected void addChildList(BaseObjectList listIn)
  {
    int pos;
    BaseObject nextObject;

    if (listIn == null)
      { return; }

    for (pos = 0; pos < listIn.size(); pos++)
      {
	nextObject = listIn.getObjectAt(pos);
	addChildObject(nextObject);
      }
  }


  /**
   * TreeNode Extension: Sugar for de-registering lists of things at a
   * time.
   * @param listIn List to be de-registered
   */
  protected void removeChildList(BaseObjectList listIn)
  {
    int pos;
    BaseObject nextObject;

    if (listIn == null)
      { return; }

    for (pos = 0; pos < listIn.size(); pos++)
      {
	nextObject = listIn.getObjectAt(pos);
	removeChildObject(nextObject);
      }
  }


  /** TreeNode Extension: To retrieve the parent
   * @return BaseObject representing the parent of this object
   */
  public BaseObject getParentObject() {
    return parent;
  }


  /** TreeNode Extension: Returns the child BaseObject at index childIndex.
   * @param childIndex The position of the child wanted.
   * @return BaseObject the BaseObject at that position.
   */
  public BaseObject getChildObjectAt(int childIndex)
  { return (BaseObject)(children.get(childIndex)); }


  /** TreeNode Extension: Returns the children in an BaseObject array.
   * @return BaseObject[] All the children in order.
   */
  public BaseObject[] getAllChildObjects()
  {
    int pos;
    BaseObject[] childArray;

    childArray = new BaseObject[getChildCount()];
    for (pos = 0; pos < getChildCount(); pos++)
      { childArray[pos] = getChildObjectAt(pos); }
    return childArray;
  }


  /** Create a dot-notation string for a BaseObject.
   * @return String The combined dot-notation string including parents.
   */
  public String getDotString()
  {
    if (parent == null)
      { return this.toString(); }
    else
      { return (parent.getDotString() + "." + this); }
  }



  /** To get the string representation of the object. Note that this
   *  representation is only concerned with the node itself, so recursion
   *  to collect children strings may not always be necessary. Note that
   *  this method should still return something useful, for example
   *  Formulas will call their children.toStrings() since they have no
   *  internal members to represent by themselves (they merely serve
   *  as a container for collecting other Expressions together).
   * @return String representing the string of this object
   */
  public abstract String toString();


  /** This method is used to provide regression testing. Basically
   *  calling this method should try and return you syntactically
   *  correct and semantically equivilant Rosetta code for this object.  
   *  Note that this differs from "toString()" as that is supposed
   *  to return a String that encapsulate only the contents of that node.
   *  In particular this call is by necessity recursive, whereas the
   *  "toString()" method need not be.
   * @return String Rosetta source code that could produce this expression.
   */
  public final String publishROM()
  { return publishROM(defaultPublishROMIndent); }


  /** This method is used to provide regression testing. Basically
   *  calling this method should try and return you syntactically
   *  correct and semantically equivilant Rosetta code for this object.  
   *  Note that this differs from "toString()" as that is supposed
   *  to return a String that encapsulate only the contents of that node.
   *  In particular this call is by necessity recursive, whereas the
   *  "toString()" method need not be.
   * @param indent The current string to use as a logical block indentor.
   * @return String Rosetta source code that could produce this expression.
   */
  public abstract String publishROM(String indent);

  /**
   * Describe variable <code>defaultPublishROMIndent</code> here.
   *
   */
  public final String defaultPublishROMIndent = "  ";


  
  /**
   * The <code>addDecoration</code> method decorates the object with a
   * new value, tagged with a key. The method will overwrite the
   * current value for the key, if it already exists in the object's
   * decorations.
   *
   * @param key an <code>Object</code> that tags the decoration.
   * @param value an <code>Object</code> that represents the decoration.
   */
  public void addDecoration(Object key, Object value) {
    decorations.put(key, value);
  }

  /**
   * The <code>getDecoration</code> method returns the value
   * associated with the key in a ROM object's decorations. The method
   * will return null if the key does not exist in the object's decorations.
   *
   * @param key an <code>Object</code> which identifies a value in the
   * decorations. 
   * @return an <code>Object</code> value.
   */
  public Object getDecoration(Object key) {
    return decorations.get(key);
  }

  /**
   * The <code>removeDecoration</code> method removes the decoration
   * associated with the key.
   *
   * @param key an <code>Object</code> value
   */
  public void removeDecoration(Object key) {
    decorations.remove(key);
  }

  /**
   * The <code>getDecorations</code> method returns a Map containing
   * the current decorations of this BaseObject.
   *
   * @return Map A Map containing the current decorations of this BaseObject.
   */
  public Map getDecorations() {
    return decorations;
  }


}

