//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Joseph josephk@cs.adelaide.edu.au
//
//  $Id: BaseObjectList.java,v 1.2 2002/07/16 07:15:47 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: BaseObjectList.java,v $
//  Revision 1.2  2002/07/16 07:15:47  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.3  2002/04/17 19:55:04  cvs
//  Modified ROM structural elements, such as Item and Label, to inherit from Expression. This allows the meta-functions to operate along the same lines as any other function.
//  -kimmell
//
//  Revision 1.1.2.2  2002/01/23 15:57:20  cvs
//  Moved the org.sldl.lang.primitive package to org.sldl.parser.primitive and made the appropriate changes in the source tree.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:15  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.1.2.2  2001/11/24 06:44:44  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//  Revision 1.1.2.1  2001/11/14 19:17:18  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Map;
import java.util.HashMap;


/** This represents the <B>BaseObjectList</B> object. Extensions
 *  of this list should be restricted to a single type of BaseObject.
 *  Note that there are no public methods for adding objects to
 *  guarentee that the extended classes can remain consistant.
 *  In general, any function call that has a return type will not
 *  be public, nor will any function which alters the contents of
 *  the underlying list.
 * @author <I> Joseph Kuehn </I>
 * (josephk@cs.adelaide.edu.au)
 */
public abstract class BaseObjectList implements Serializable {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "HeadEntry" };



  /** Simple class for representing a list as a linked list.
   */
  public static class ListEntry implements Serializable
    {
    /** The <code>keySet</code> array lists a collection of keys
     *  correspond to a description of the internal state of an
     *  instance of this class when considered in concert with all
     *  other <code>keySet</code> arrays in its superclasses.
     */
    public static final String[] keySet = { "EntryValue", "NextEntry" };


    private final BaseObject value;
    private final ListEntry next;

    /** Primary Constructor.
     * @param settings The Map containing the default settings of this object.
     */
    public ListEntry(Map settings)
      {
      if (settings == null)
        { value = null; next = null; }
      else
        {
        value = (BaseObject)settings.get(ListEntry.keySet[0]);
        next = (ListEntry)settings.get(ListEntry.keySet[1]);
        }
      }

    /** Method to fetch the value at this position in the list.
     * @return BaseObject The value attached to this node.
     */
    public BaseObject getEntryValue()
      { return value; }

    /** Method to fetch the next node in the list.
     * @return ListEntry The next node in this list.
     */
    public ListEntry getNextEntry()
      { return next; }
    }



  private Vector myList;

  /** The primary Constructor.
   */
  public BaseObjectList()
    { myList = new Vector(); }


  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  protected BaseObjectList(Map settings)
    {
    myList = new Vector();
    if (settings != null)
      { addListEntries((ListEntry)(settings.get(BaseObjectList.keySet[0]))); }
    }


  /** Method for representing the BaseObjectList as a linked list.
   * @return ListEntry The head of the linked list.
   */
  public ListEntry getHeadEntry()
    {
    int pos;
    ListEntry result;
    Map nextContents;

    result = null;
    for (pos = size() - 1; pos >= 0; pos--)
      {
      nextContents = new HashMap();
      nextContents.put(ListEntry.keySet[0], getObjectAt(pos));
      nextContents.put(ListEntry.keySet[1], result);
      result = new ListEntry(nextContents);
      }
    return result;
    }

  /** Method for adding entries from a linked list to this list.
   * @param head The head of the linked list to add.
   * @return void
   */
  protected void addListEntries(ListEntry head)
    {
    if (head != null)
      {
      addObject(head.getEntryValue());
      addListEntries(head.getNextEntry());
      }
    }


  /** Method for a subclass to add an element to the list.
   * @param element The BaseObject to add.
   * @return void
   */
  protected void addObject(BaseObject element)
    { myList.add(element); }


  /** Method for a subclass to add an element to the list by position.
   * @param int The position to add the BaseObject to.
   * @param element The BaseObject to add.
   * @return void
   */
  protected void addObject(int pos, BaseObject element)
    { myList.add(pos, element); }


  /** Method to remove an element from the list.
   * @param element The BaseObject to remove.
   * @return void
   */
  public void remove(BaseObject element)
    { myList.remove(element); }


  /** Method for a subclass to remove an element from the list by position.
   * @param pos The position of the BaseObject to remove.
   * @return BaseObject The element removed.
   */
  protected BaseObject removeObject(int pos)
    { return (BaseObject)(myList.remove(pos)); }


  /** To check if list contains a specific element
   * @param element The BaseObject to search for.
   * @return boolean, true if element present, false otherwise.
   */
  public boolean contains(BaseObject element)
    { return myList.contains(element); }


  /** The one exception to the masking clause. Tools that wish to
   *  iterate over a list of BaseObjects for generic purposes can
   *  do so with this method.
   * @param pos The position to retreive the BaseObject from.
   * @return BaseObject The BaseObject found at that position.
   */
  public BaseObject getObjectAt(int pos)
    { return (BaseObject)(myList.get(pos)); }


  /** Method to create an enumeration of the elements in this list.
   * @return Enumeration An enumeration for this list.
   */
  public Enumeration elements()
    { return myList.elements(); }


  /** Method to find the number of elements in this list.
   * @return int The size of this list.
   */
  public int size()
    { return myList.size(); }


  /** Method to check whether the list is empty.
   * @return boolean Is the list empty?
   */
  public boolean isEmpty()
    { return myList.isEmpty(); }


  /** Method to find the index of an element in the list.
   * @param element The BaseObject to look for.
   * @return int The position of the BaseObject in the list.
   */
  public int indexOf(BaseObject element)
    { return myList.indexOf(element); }


  /** Method to find the index of an element in the list from an offset.
   * @param element The BaseObject to look for.
   * @param offset The position to starting looking at.
   * @return int The position of the BaseObject in the list.
   */
  public int indexOf(BaseObject element, int offset)
    { return myList.indexOf(element, offset); }


  /** Method for a subclass to retreive all elements from the list.
   * @return BaseObject[] The elements by position.
   */
  protected BaseObject[] toObjectArray()
    {
      int pos;
      BaseObject[] contents;
      
      contents = new BaseObject[size()];
      for (pos = 0; pos < size(); pos++)
	{ contents[pos] = getObjectAt(pos); }
      return contents;
    }
    

  /** Method to create an iterator of the elements in this list.
   * @return Iterator An iterator for this list.
   */
  public Iterator iterator()
    { return myList.iterator(); }


  /** Method to generate a string representation of this list using a given seperator.
   * @param seperator The string to use inbetween each element of this list.
   * @return String representing the string of this object.
   */
  public String toString(String seperator)
    {
    String str;
    Iterator listIterator;

    if (isEmpty())
      { return ""; }

    listIterator = iterator();
    str = "" + listIterator.next();
    while (listIterator.hasNext())
      { str = str + seperator + listIterator.next(); }
    return str;
    }


  }  
