//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Garrin kimmell@ittc.ukans.edu
//               Joseph josephk@cs.adelaide.edu.au
//
//  $Id: CaseExpression.java,v 1.3 2002/07/16 07:15:47 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: CaseExpression.java,v $
//  Revision 1.3  2002/07/16 07:15:47  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:02:07  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.4  2002/06/05 16:30:42  kimmell
//  Modified the publishROM method to simply return the result of the toString method.
//  -kimmell
//
//  Revision 1.1.2.3  2002/05/28 21:25:14  kimmell
//  Added a slot for a default value. The parser doesn't currently fill this slot correctly.
//  -kimmell
//
//  Revision 1.1.2.2  2002/03/12 22:51:53  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:15  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.1.2.3  2002/01/04 12:30:47  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.1.2.2  2001/11/24 06:44:44  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//  Revision 1.1.2.1  2001/11/14 19:17:18  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents the <B>CaseExpression</B> object. An instance of this
 *  class indicates that a 'case' statement has been used. Because the
 *  case statement has a unique structure not duplicated by any other
 *  function it is necessary to include it here.
 * @author <I>Garrin Kimmell and Joseph Kuehn</I>
 *  (kimmell@ittc.ukans.edu, josephk@cs.adelaide.edu.au)
 */
public class CaseExpression extends Expression	{

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "TestExpression", "TestValues", "Results" };


  private Expression test;
  private Expression defaultResult;
  private ExpressionList values, results;

  /** The primary Constructor
   */
  public CaseExpression()
  {
    test = null;
    values = new ExpressionList();
    results = new ExpressionList();
  }


  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public CaseExpression(Map settings)
    {
    super(settings);

    test = (Expression)getSetting(CaseExpression.keySet[0]);
    values = (ExpressionList)getSetting(CaseExpression.keySet[1]);
    results = (ExpressionList)getSetting(CaseExpression.keySet[2]);
    defaultResult = null;

    addChildObject(test);
    addChildList(values);
    addChildList(results);
    }



  /** The function to find the number of (value,result) pairs in the case statement.
   * @return int The number of pairs currently in the case statement.
   */
  public int optionCount()
  { return values.size(); }


  /** The function to add a (value,result) pair to the case statement.
   * @param value The new case to test against.
   * @param result The new result of that case.
   * @return void
   */
  public void addOption(Expression value, Expression result)
  {
    values.add(value);
    results.add(result);
    addChildObject(value);
    addChildObject(result);
  }


  /** The function to remove a (value,result) pair from the case statement.
   * @param pos The position to find the testValue & result at.
   * @return void
   */
  public void removeOption(int pos)
  {
    removeChildObject(values.remove(pos));
    removeChildObject(results.remove(pos));
  }
    
    
  /** The function to get a test value from the case statement.
   * @param pos The position to find the test at.
   * @return Expression The test at that position.
   */
  public Expression getTestValue(int pos) {
    return values.get(pos);
  }
    
  /** The function to get a test result from the case statement.
   * @param pos The position to find the result at.
   * @return Expression The result at that position.
   */
  public Expression getResult(int pos) {
    return results.get(pos);
  }

  /**
   * The <code>getOptions</code> method returns the possible options
   * for this case expression.
   *
   * @return an <code>ExpressionList</code> value
   */
  public ExpressionList getOptions() {
    return values;
  }



  /** The function to get the different Expression tests.
   * @return ExpressionList The test value expressions of the Case statement.
   */
  public ExpressionList getTestValues() {
    return values;
  }

  /** The function to set the different Expression tests.
   * @param valuesIn The new test value expressions of the Case statement.
   * @return void
   */
  public void setTestValues(ExpressionList valuesIn){
    removeChildList(values);
    values = valuesIn;
    addChildList(values);
  }
    



  /**
   * The <code>getResults</code> method returns the results possible
   * for this case expression. These are respective to the options
   * returned by getOptions();
   *
   * @return an <code>ExpressionList</code> value
   */
  public ExpressionList getResults() {
    return results;
  }

  /**
   * The <code>setResults</code> method sets the possible results
   * for this case expression. These are respective to the options
   * returned by getOptions();
   *
   * @param resultsIn The new set of possible results.
   */
  public void setResults(ExpressionList resultsIn) {
    removeChildList(results);
    results = resultsIn;
    addChildList(results);
  }
    


    
    
  /** The function to set the Expression to test.
   * @param exprIn The new test expression of the Case statement.
   * @return void
   */
  public void setTestExpression(Expression exprIn){
    removeChildObject(test);
    test = exprIn;
    addChildObject(test);
  }
    
  /** The function to get the Expression to test.
   * @return Expression The test of the Case statement.
   */
  public Expression getTestExpression() {
    return test;
  }
    


    
  public Expression getDefaultResult() {
    return defaultResult;
  }

  public void setDefaultResult(Expression result) {
    defaultResult = result;
  }
    
    
 









  /** To get the string representation of the object
   * @return String representing the string of this object
   */
  public String toString(){
    String answer;

    answer = "case (" + test + ")";
    //return answer;
    return publishROM(defaultPublishROMIndent);

  }


  /** To get the string declaration of the object
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the declaration of this object
   */
  public String publishROM(String indent) {
    int pos;
    String answer;

    answer = "case ";
    if (test != null)
      { answer = answer + test.publishROM(indent + defaultPublishROMIndent); }
    else
      { answer = answer + "NULL"; }
    answer = answer + " is\n" + indent;
    if (!values.isEmpty()) {


	answer = answer + values.get(0).publishROM(indent +
						   defaultPublishROMIndent) + " -> " 
	  + results.get(0).publishROM(indent +
				      defaultPublishROMIndent) + "\n"
	  + indent; 
    }
    
    


    for (pos = 1; pos < values.size(); pos++)
      { answer = answer + values.get(pos).publishROM(indent +
						     defaultPublishROMIndent) + " -> " 
	  + results.get(pos).publishROM(indent + defaultPublishROMIndent) + "\n" + indent; }
    

    answer = answer + "end case";
    return answer;
  }



}

