//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Joseph josephk@cs.adelaide.edu.au
//           David schond@ittc.ukans.edu
//
//  $Id: CharacterLiteral.java,v 1.4 2002/09/12 09:24:32 tarp Exp $
//
//  Revision History:
//  ----------------
//  $Log: CharacterLiteral.java,v $
//  Revision 1.4  2002/09/12 09:24:32  tarp
//  Updated CharacterLiteral to use back-quotes instead of the forward-quotes.
//
//  Revision 1.3  2002/07/16 07:15:47  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:02:24  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.2  2002/03/12 22:51:54  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:16  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.1.2.4  2002/01/04 12:30:47  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.1.2.3  2001/11/24 06:44:45  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//  Revision 1.1.2.2  2001/11/14 19:17:18  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//  Revision 1.1.2.1  2001/11/07 22:41:02  cvs
//  Added treenode functionality to the ROM.
//  -Joseph
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents the <B>CharacterLiteral</B> object. An instance of
 *  this class represents a wrapper for an actual character in the ROM.
 * @author <I> Joseph Kuehn </I>
 * (josephk@cs.adelaide.edu.au)
 */
public class CharacterLiteral extends Literal {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "Data" };


  private Character charData;


  /** The primary constructor
   * @param charIn The Character value
   */
  public CharacterLiteral(Character charIn)
    {
    charData = charIn;
    }


  /** The secondary constructor. May be passed a single character, a
   *  character in quotes or a Rosetta unicode string.
   * @param stringIn The string to interpret
   * @throws InvalidUnicodeFormatException If the string is mal-formed.
   */
  public CharacterLiteral(String stringIn)
    {
    charData = new Character(' ');
    setData(stringIn);
    }

  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public CharacterLiteral(Map settings)
    {
    super(settings);

    charData = (Character)getSetting(CharacterLiteral.keySet[0]);
    }



  /** To get the Character data from this Literal.
   * @return Character The value of this Literal.
   */
  public Character getData()
    { return charData; }

  /** To set the Character data for this Literal.
   * @param charIn The new value of this Literal.
   * @return void
   */
  public void setData(Character charIn)
    { charData = charIn; }

  /** To set the Character data for this Literal using a string.
   *  May be passed a single character, a character in quotes
   *  or a Rosetta unicode string.
   * @param stringIn The string to interpret
   * @return void
   * @throws InvalidUnicodeFormatException If the string is mal-formed.
   */
  public void setData(String stringIn)
    {
    String unicodeString;

    if (stringIn == null)
      { throw new InvalidUnicodeFormatException("The given character string is null."); }

    if (stringIn.length() == 1)
      { charData = new Character(stringIn.charAt(0)); }
    else if (stringIn.length() == 3)
      { charData = new Character(stringIn.charAt(1)); }
    else
      {
      unicodeString = convertRosettaToJava(stringIn); 
      charData = new Character(convertJavaToCharacter(unicodeString));
      }
    }

 

  /** To get the string representation of the object
   * @return String representing the string of this object
   */
  public String toString()
    { return "'" + getData() + "'"; }


  /** To get the string declaration of this object
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the declaration of this object
   */
  public String publishROM(String indent)
    {
    if (getData() == null)
      { return "'NULL'"; }
    else
      { return "'" + convertJavaToRosetta(convertCharacterToJava(getData().charValue())) + "'"; }
    }




  /** Exception to represent when a string is not valid Rosetta Unicode = \'U+XXXX\'.
   */
  public static class InvalidUnicodeFormatException extends RuntimeException
    {
    /** primary constructor.
     */
    public InvalidUnicodeFormatException(String message)
      { super(message); }
    }


  /** Method for converting a Rosetta unicode string into a Java unicode String.
   * @param rosettaString The String containing the Unicode representation.
   * @return String The converted string.
   * @throws InvalidUnicodeFormatException If the string is mal-formed.
   */
  public static String convertRosettaToJava(String rosettaString)
    {
    String myCode;

    if ((rosettaString == null) || (rosettaString.length() != 8)
       || (!rosettaString.substring(0,3).equalsIgnoreCase("'U+")) || (rosettaString.charAt(7) != '\''))
      { throw new InvalidUnicodeFormatException("The given string [" + rosettaString + "] does not follow the 'U+XXXX' Rosetta Unicode format."); }
    myCode = "'\\u" + rosettaString.substring(3,7) + "'";
    return myCode;
    }


  /** Method for converting a Java unicode string into a Rosetta unicode String.
   * @param javaString The String containing the Unicode representation.
   * @return String The converted string.
   * @throws InvalidUnicodeFormatException If the string is mal-formed.
   */
  public static String convertJavaToRosetta(String javaString)
    {
    String myCode;

    if ((javaString == null) || (javaString.length() != 8)
       || (!javaString.substring(0,3).equalsIgnoreCase("'\\u")) || (!javaString.substring(7,8).equals("'")))
      { throw new InvalidUnicodeFormatException("The given string [" + javaString + "] does not follow the '\\uXXXX' Java Unicode format."); }
    myCode = "'U+" + javaString.substring(3,7) + "'";
    return myCode;
    }


  /** Method for converting a Java unicode string into a Java unicode char. Ugly.
   * @param javaString The String containing the Unicode representation.
   * @return char The converted character.
   * @throws InvalidUnicodeFormatException If the string is mal-formed.
   */
  public static char convertJavaToCharacter(String javaString)
    {
    String myCode;

    if ((javaString == null) || (javaString.length() != 8)
       || (!javaString.substring(0,3).equalsIgnoreCase("'\\u")) || (!javaString.substring(7,8).equals("'")))
      { throw new InvalidUnicodeFormatException("The given string [" + javaString + "] does not follow the '\\uXXXX' Java Unicode format."); }
    myCode = javaString.substring(3,7);

    return (char)convertHexString(myCode);
    }

  /** Method for converting character into a Java unicode string. Ugly.
   * @param charIn The character to convert to Unicode representation.
   * @return String The unicode string.
   */
  public static String convertCharacterToJava(char charIn)
    {
    return "'\\u" + convertToHexString((int)charIn) + "'";
    }


  /** Method for converting an unsigned hexidecimal string into an int.
   * @param hexString The String containing the unsigned hexidecimal number.
   * @return int The value represented by that string, -1 if an error occurred.
   */
  public static int convertHexString(String hexString)
    {
    int pos, base, digit, value;

    if (hexString == null)
      { return -1; }

    base = 1;
    value = 0;
    for (pos = hexString.length() - 1; pos >= 0; pos--)
      {
      switch (hexString.charAt(pos))
        {
        case '0' : digit = 0; break;
        case '1' : digit = 1; break;
        case '2' : digit = 2; break;
        case '3' : digit = 3; break;
        case '4' : digit = 4; break;
        case '5' : digit = 5; break;
        case '6' : digit = 6; break;
        case '7' : digit = 7; break;
        case '8' : digit = 8; break;
        case '9' : digit = 9; break;
        case 'a' | 'A' : digit = 10; break;
        case 'b' | 'B' : digit = 11; break;
        case 'c' | 'C' : digit = 12; break;
        case 'd' | 'D' : digit = 13; break;
        case 'e' | 'E' : digit = 14; break;
        case 'f' | 'F' : digit = 15; break;
        default: return -1;
        }
      value = value + (base * digit);
      base = base * 16;
      }
    return value;
    }

  /** Method for converting the absolute value of an integer into an
   *  unsigned hexidecimal string of at least 4 digits.
   * @param value The value to be converted into a string.
   * @return String The String containing the unsigned hexidecimal number.
   */
  public static String convertToHexString(int value)
    {
    String result;
    int remainder, digit;

    result = "";
    remainder = Math.abs(value);
    while ((remainder > 0) || (result.length() < 4))
      {
      digit = remainder % 16;
      remainder = remainder / 16;

      switch (digit)
        {
        case 0:  case 1:  case 2:  case 3:  case 4:
        case 5:  case 6:  case 7:  case 8:  case 9:
          result = digit + result; break;
        case 10:
          result = "a" + result; break;
        case 11:
          result = "b" + result; break;
        case 12:
          result = "c" + result; break;
        case 13:
          result = "d" + result; break;
        case 14:
          result = "e" + result; break;
        case 15:
          result = "f" + result; break;
        default: break;
        }
      }
    return result;
    }

}


