//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: sparker@averstar.com
//
//  $Id: Component.java,v 1.3 2002/07/16 07:15:47 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Component.java,v $
//  Revision 1.3  2002/07/16 07:15:47  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:02:36  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.2  2002/03/12 22:51:54  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:16  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.5.2.4  2002/01/04 12:30:47  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.5.2.3  2001/11/24 06:44:45  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//  Revision 1.5.2.2  2001/11/14 19:17:18  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//  Revision 1.5.2.1  2001/11/07 22:41:02  cvs
//  Added treenode functionality to the ROM.
//  -Joseph
//
//  Revision 1.5  2001/03/12 23:56:46  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.4  2000/12/04 20:12:16  cvs
//  Assumption, definition and verification blocks are labelled items.
//  They are also put in the declared item list (instead of list of terms
//  as in previous version).
//  -- Cindy
//
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/**
 * <b>Component</b>:  Represtents the type of facet know as a component.
 *
 * @author <i>Steve Parker</i> 
 * (<a href="mailto:sparker@averstar.com">sparker@averstar.com</a>).
 */
public class Component extends Facet {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "Assumption", "Definition", "Verification" };


    private Expression assumption, definition, verification;

    /**
     * Primary constructor.
     *
     * Assumed that first three terms of this Facet are the Assumption, 
     * Definition, and Verification Facets.
     *
     * @param assumptionIn The ASSUMPTION Facet.
     * @param definitionIn The DEFINITION Facet.
     * @param verificationIn The VERIFICATION Facet.
     */
    public Component(Expression assumptionIn, Expression definitionIn,
					 Expression verificationIn) {
		setTermItems(null);
		assumption = assumptionIn;
		definition = definitionIn;
		verification = verificationIn;
		addChildObject(assumption);
		addChildObject(definition);
		addChildObject(verification);
    }

	/**
	 * Another constructor.
	 * Do not need to initialize the assumption, definition and
	 * verification facets in the constructor
	 */
	public Component() {
		this(null, null, null);
	}


  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public Component(Map settings)
    {
    super(settings);

    setTermItems(null);

    assumption = (Expression)getSetting(Component.keySet[0]);
    definition = (Expression)getSetting(Component.keySet[1]);
    verification = (Expression)getSetting(Component.keySet[2]);

    addChildObject(assumption);
    addChildObject(definition);
    addChildObject(verification);
    }


	/**
	 * To set the assumption facet
	 * @param assumptionIn The ASSUMPTION Facet.
	 * @return void
	 */
	public void setAssumption(Expression assumptionIn) {
		removeChildObject(assumption);
		assumption = assumptionIn;
		addChildObject(assumption);
	}
	
	/**
     * Return the ASSUMPTION Facet.
     */
    public Expression getAssumption()
	{ return assumption; }

    /**
	 * To set the definition facet
	 * @param definitionIn The DEFINITION Facet.
	 * @return void
	 */
	public void setDefinition(Expression definitionIn) {
		removeChildObject(definition);
		definition = definitionIn;
		addChildObject(definition);
	}
	
	/**
     * Return the DEFINITION Facet.
     */
    public Expression getDefinition()
 	{ return definition; }

    /**
	 * To set the verification facet
	 * @param verificationIn The VERIFICATION Facet.
	 * @return void
	 */
	public void setVerification(Expression verificationIn) {
		removeChildObject(verification);
		verification = verificationIn;
		addChildObject(verification);
	}
	
	/**
     * Return the VERIFICATION Facet.
     */
    public Expression getVerification()
	{ return verification; }





	
  /** To get the string representation of the object
   * @return String representing the string of this object
   */
  public String toString()
    { return "component(A" + assumption + ",D" + definition + ",V" + verification + ") " + partialString(); }



  /** Method to make it easier to pretty-print.
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the declaration of this object.
   */
  protected String partialPublish(String indent)
    {
	int pos, deIndent;
	String answer;

    if (indent.length() < 2)
      { deIndent = 0; }
    else
      { deIndent = 2; }

	answer = publishInterface(indent) + " is\n" + indent;
        answer = answer + publishDeclarations(indent, "\n") + publishExports(indent) + "\n" + indent;

	if (assumption != null)
          {
          if (assumption instanceof Facet)
            { answer = answer + "assumption " + ((Facet)assumption).partialPublish(indent + defaultPublishROMIndent) + " assumption;\n\n" + indent; }
          else
            { answer = answer + "assumption is " + assumption.publishROM(indent + defaultPublishROMIndent) + ";\n\n" + indent; }
          }
	if (definition != null)
          {
          if (definition instanceof Facet)
            { answer = answer + "definition " + ((Facet)definition).partialPublish(indent + defaultPublishROMIndent) + " definition;\n\n" + indent; }
          else
            { answer = answer + "definition is " + definition.publishROM(indent + defaultPublishROMIndent) + ";\n\n" + indent; }
          }
	if (verification != null)
          {
          if (verification instanceof Facet)
            { answer = answer + "verification " + ((Facet)verification).partialPublish(indent + defaultPublishROMIndent) + " verification;\n\n" + indent; }
          else
            { answer = answer + "verification is " + verification.publishROM(indent + defaultPublishROMIndent) + ";\n\n" + indent; }
          }
	answer = answer.substring(0, answer.length()-deIndent) + "end";
	return answer;
    }


  /** Method to get the infix string declaration of the object
   * @param myLabel The name of this syntactical shorthand object.
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the declaration of this object.
   */
  public String publishROM(Label myLabel, String indent)
    { return publishUses(indent) + "component " + myLabel + partialPublish(indent) + " component " + myLabel; }


  /** To get the string declaration of the object
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the declaration of this object
   */
  public String publishROM(String indent)
    { return "component" + partialPublish(indent) + " component"; }


}
