//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Joseph josephk@cs.adelaide.edu.au
//
//  $Id: CompositeValue.java,v 1.3 2002/07/16 07:15:47 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: CompositeValue.java,v $
//  Revision 1.3  2002/07/16 07:15:47  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:02:48  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.3  2002/04/17 19:55:04  cvs
//  Modified ROM structural elements, such as Item and Label, to inherit from Expression. This allows the meta-functions to operate along the same lines as any other function.
//  -kimmell
//
//  Revision 1.1.2.2  2002/03/12 22:51:54  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:16  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.1.2.3  2002/01/04 12:30:47  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.1.2.2  2001/11/24 06:44:45  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//  Revision 1.1.2.1  2001/11/14 19:17:18  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents the <B>CompositeValue</B> object. A sub-class of this
 *  abstract class represents an implementation of one of Rosetta's
 *  composite (ie compound) values. That is, this is an expression for
 *  encapsulation of collections of other expressions.
 * @author <I> Joseph Kuehn </I>
 * (josephk@cs.adelaide.edu.au)
 */
public abstract class CompositeValue extends Expression {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "Elements" };


  private ExpressionList elements;

  /** The primary constructor.
   */
  public CompositeValue()
    { elements = new ExpressionList(); }


  /** The secondary constructor
   * @param elementsIn The list of elements in set
   */
  public CompositeValue(ExpressionList elementsIn)
    {
    elements = elementsIn;
    addChildList(elements);
    }

  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public CompositeValue(Map settings)
    {
    super(settings);

    elements = (ExpressionList)getSetting(CompositeValue.keySet[0]);

    addChildList(elements);
    }





  /** To set the elements of this value.
   * @param exprList The new contents, all the old contents are removed.
   * @ return void
   */
  public void setElements(ExpressionList exprList)
    {
    removeChildList(elements);
    elements = exprList;
    addChildList(elements);
    }


  /** Method to add an Expression to the value.
   * @param value The Expression to add to the Composite Value.
   * @return void
   */
  public void addElement(Expression value)
    {
    if (elements == null)
      { elements = new ExpressionList(); }
    elements.add(value);
    addChildObject(value);
    }



  /** To return all the elements of the Composite value.
   * @return ExpressionList The list representing the contents of this value.
   */
  public ExpressionList getElements()
    { return elements; }


  /** Nice way of counting the number the elements of the Composite value.
   * @return int The size of the list representing the contents of this value, -1 if null.
   */
  public int size()
    {
    if (getElements() == null)
      { return -1; }
    else
      { return getElements().size(); }
    }




  /** To return a specific element of the Composite value. Note that for some
   *  Composite Values this method does not imply an ordering of the elements,
   *  it is simply provided here for implementation convienience.
   * @param pos The list position of the element wanted.
   * @return Expression The element in list at that position.
   */
  public Expression getElementAt(int pos)
    {
    if (elements == null)
      { return null; }
    else
      { return elements.get(pos); }
    }



  /** Method to get the index of an Expression by (exp == val) pointer test.
   * @param expr The Expression to search for.
   * @return int The position it was at, -1 if not found.
   */
  public int getIndexOf(Expression expr)
    {
    int pos;

    if (elements == null)
      { return -1; }

    for (pos = 0; pos < elements.size(); pos++)
      {
      if (elements.get(pos) == expr)
        { return pos; }
      }
    return -1;
    }


  /** Method to assist in pretty-printing. Does not add spacing by default.
   * @param separator The string to use to separate elements.
   * @return String The representation of the contents using the separator.
   */
  protected String contentString(String separator)
    {
    int pos;
    String result;

    if ((elements == null) || elements.isEmpty())
      { return ""; }
    result = "" + elements.get(0);
    for (pos = 1; pos < elements.size(); pos++)
      { result = result + separator + elements.get(pos); }
    return result;
    }


  /** Method to assist in pretty-printing. Does not add spacing by default.
   * @param separator The string to use to separate elements.
   * @param indent The current string to use as a logical block indentor.
   * @return String The representation of the contents using the separator.
   */
  protected String contentPublish(String separator, String indent)
    {
    int pos;
    String result;

    if ((elements == null) || elements.isEmpty())
      { return ""; }
    result = elements.get(0).publishROM(indent);
    for (pos = 1; pos < elements.size(); pos++)
      { result = result + separator + elements.get(pos).publishROM(indent); }
    return result;
    }

}



