//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Garrin kimmell@ittc.ku.edu
//               Joseph josephk@cs.adelaide.edu.au
//
//  $Id: ConstructedType.java,v 1.3 2002/07/16 07:15:47 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: ConstructedType.java,v $
//  Revision 1.3  2002/07/16 07:15:47  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:03:01  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.3  2002/04/17 19:55:04  cvs
//  Modified ROM structural elements, such as Item and Label, to inherit from Expression. This allows the meta-functions to operate along the same lines as any other function.
//  -kimmell
//
//  Revision 1.1.2.2  2002/03/12 22:51:55  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:16  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.1.2.3  2002/01/04 13:05:08  cvs
//  Updated the old 'AggregateReferences' to 'QuantifiedReferences'.
//
//  Revision 1.1.2.2  2002/01/04 12:30:47  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.1.2.1  2001/11/24 06:44:45  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents the <B>ConstructedType</B> object. An instance of this
 *  class represents a definition of a constructed type definition. This contains
 *  the definitions of the constructors, and the recognisers and observers for
 *  those constructors.
 * @author <I> Garrin Kimmell, Joseph Kuehn </I>
 * (kimmell@ittc.ku.edu, josephk@cs.adelaide.edu.au)
 */
public class ConstructedType extends Expression  {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "Type", "Constructors" };


  private ItemReference thisType;
  private ItemList constructorList;

  /** Primary constructor.
   * @param type The parent item of this value. If anonymous - must be the 'constructed' univ item.
   */
  public ConstructedType(ItemReference type)
  {
    constructorList = new ItemList();
    thisType = type;

    addChildObject(thisType);
  }


  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public ConstructedType(Map settings)
    {
    super(settings);

    thisType = (ItemReference)getSetting(ConstructedType.keySet[0]);
    constructorList = (ItemList)getSetting(ConstructedType.keySet[1]);

    addChildObject(thisType);
    addChildList(constructorList);
    }



  /** This represents the <B>ConstructedTypeConstructor</B> object. An instance
   *  of this class represents a specific constructor in the ConstructedType.
   *  In particular this is the value of that constructor, rather than the item.
   */
  public static class ConstructedTypeConstructor extends Expression  {

    /** The <code>keySet</code> array lists a collection of keys
     *  correspond to a description of the internal state of an
     *  instance of this class when considered in concert with all
     *  other <code>keySet</code> arrays in its superclasses.
     */
    public static final String[] keySet = { "Constructor", "Observers", "Recognizer" };


    private ItemReference myConstructor;
    private Function myFunction;
    private Item myRecognizer;
    private ItemList observerList;

    /** Primary Constructor.
     * @param constructor A link to the item of this constructor - needed for observers.
     */
    public ConstructedTypeConstructor(ItemReference constructor)
    { this(constructor, new ItemList(), null); }

    /** Secondary constructor.
     * @param constructor A link to the item of this constructor - needed for observers.
     * @param params The interface listing for this constructor.
     */
    public ConstructedTypeConstructor(ItemReference constructor, ItemList params)
    { this(constructor, params, null); }


    /** Secondary constructor.
     * @param constructor A link to the item of this constructor - needed for observers.
     * @param params The interface listing for this constructor.
     * @param recogLabel The label to hold for the recognizer for now.
     */
    public ConstructedTypeConstructor(ItemReference constructor, ItemList params, Label recogLabel)
    {
      ConstructedValue result;

      myConstructor = constructor;
      // FIXME: Need to make a copy of myConstructor?
      result = new ConstructedValue(myConstructor);
//      result.setElements(params.createReferenceList());
      // FIXME: Need to make a copy of myConstructor?
      myFunction = new Function(params, new ConstructedValue(myConstructor), result);
      addChildObject(myFunction);
      observerList = new ItemList();
      if (recogLabel != null)
        { myRecognizer = new Item(recogLabel, null, null); }
      else
        { myRecognizer = null; }
      addChildObject(myRecognizer);
    }


    /** Map Constructor.
     * @param settings The Map containing the default settings of this object.
     */
    public ConstructedTypeConstructor(Map settings)
      {
      super(settings);

      myConstructor = (ItemReference)getSetting(ConstructedTypeConstructor.keySet[0]);
      observerList = (ItemList)getSetting(ConstructedTypeConstructor.keySet[1]);
      myRecognizer = (Item)getSetting(ConstructedTypeConstructor.keySet[2]);

      addChildObject(myConstructor);
      addChildList(observerList);
      addChildObject(myRecognizer);
      }


    /** Method to access the Item set as this constructors label.
     * @return ItemReference The reference to this constructors item.
     */
    public ItemReference getConstructorReference()
    { return myConstructor; }


    /** Method to access the constructor function.
     * @return Function The function value of the constructor.
     */
    public Function getConstructorFunction()
    { return myFunction; }



    /** Method to intelligently generate the recognizer function for this constructor.
     *  Removes the previous recognizer regardless of success or failure.
     * @param name The name of the recognizer function.
     * @return void
     */
    public void generateRecognizer(Label name)
    {
      Item parameter;
      ItemList paramList;
      Function recogFunction;
      LabelList nextName;
      Application testMembership;

      removeChildObject(myRecognizer);
      // FIXME: Need to make a copy of thisType?
//      parameter = new Item(new Label("x"), thisType, null);
      parameter = new Item(new Label("x"), null, null);
      paramList = new ItemList();
      paramList.add(parameter);

      ExpressionList membershipParams = new ExpressionList();
      membershipParams.add(new ItemReference(parameter));
      // FIXME: Need to make a copy of myConstructor?
      membershipParams.add(new ConstructedValue(myConstructor));
      nextName = new LabelList();
      nextName.add(new Label("in"));
      testMembership = new Application(new ItemReference(nextName),
				       membershipParams);

      nextName = new LabelList();
      nextName.add(new Label("boolean"));
      recogFunction = new Function(paramList, new ItemReference(nextName), testMembership);
      myRecognizer = new Item(name, null, recogFunction);
      addChildObject(myRecognizer);
    }


    /** Method to specify the recognizer for this constructor.
     * @param recognizer The recognizer for this constructor.
     * @return void
     */
    public void setRecognizer(Item recognizer)
    {
      removeChildObject(myRecognizer);
      myRecognizer = recognizer;
      removeChildObject(myRecognizer);
    }


    /** Method to retreive the recognizer for this constructor.
     * @return Item The recognizer for this constructor.
     */
    public Item getRecognizer()
    { return myRecognizer; }



    /** Method to intelligently generate all the observers for this constructor base on params.
     *  Removes all previous observers regardless of success or failure.
     * @return void
     */
    public void generateObservers()
    {
      int pos;
      Item nextParam, nextObserver, parameter;
      Function nextFunction;
      ItemList paramList;
      // ArrayValue arrayIndex;
      ExpressionList arrayIndex;

      removeChildList(observerList);
      observerList = new ItemList();
      for (pos = 0; pos < myFunction.getInterfaceItems().size(); pos++)
        {
	  nextParam = myFunction.getInterfaceItems().get(pos);
	  // FIXME: Need to make a copy of myConstructor?
	  parameter = new Item(new Label("x"), new ConstructedValue(myConstructor), null);
	  paramList = new ItemList();
	  paramList.add(parameter);
	  arrayIndex = new ExpressionList();
	  arrayIndex.add(new NumberLiteral(new Integer(pos)));
	  // FIXME: Need to make a copy of nextParam.getType() ?
	  nextFunction = new Function(paramList, nextParam.getType(),
				      new Application(new
					ItemReference(parameter),
						      arrayIndex));
	  // FIXME: Need to make a copy of nextParam.getLabel()?
	  nextObserver = new Item(nextParam.getLabel(), null, nextFunction);
	  observerList.add(nextObserver);
        }
      addChildList(observerList);
    }


    /** Method to add an observer for this constructor.
     * @param observer The Item which represents an observer for this constructor.
     * @return void
     */
    public void addObserver(Item observer)
    {
      if (observerList == null)
        { observerList = new ItemList(); }
      observerList.add(observer);
      addChildObject(observer);
    }


    /** Method to retreive an observer for this constructor by name.
     * @param name The name matching the desired observer.
     * @return Item The Item which represents the named observer.
     */
    public Item getObserver(String name)
    {
      int pos;
      Item nextObserver;

      if (observerList == null)
        { return null; }

      for (pos = 0; pos < observerList.size(); pos++)
        {
	  nextObserver = observerList.get(pos);
	  if (nextObserver.getLabel().toString().equalsIgnoreCase(name))
	    { return nextObserver; }
        }
      return null;
    }
      

    /** Method to retreive all the observers for this constructor.
     * @return ItemList The observers for this constructor.
     */
    public ItemList getObservers()
    { return observerList; }


    /** Method to set all the observers for this constructor.
     * @param observers The new observers for this constructor.
     * @return void
     */
    public void setObservers(ItemList observers)
    {
      removeChildList(observerList);
      observerList = observers;
      addChildList(observerList);
    }




    /** To get the string representation of the object
     * @return String representing the string of this object   
     */
    public String toString()
    { return myConstructor + "(" + myFunction + ") : " + myRecognizer; }


    /** To get the string declaration of the object
     * @param indent The current string to use as a logical block indentor.
     * @return String representing the declaration of this object   
     */
    public String publishROM(String indent)
    {
      int pos;
      String result;
      result = "" + myConstructor;
      if (!myFunction.getInterfaceItems().isEmpty())
        {
	  result = result + "(" +  myFunction.getInterfaceItems().get(0).publishROM(indent + defaultPublishROMIndent);
	  for (pos = 1; pos < myFunction.getInterfaceItems().size(); pos++)
	    { result = result + ", " + myFunction.getInterfaceItems().get(pos).publishROM(indent + defaultPublishROMIndent); }
	  result = result + ")";
        }
      result = result + " :: " + myRecognizer;
      return result;
    }


  }







  /** Method to retreive the super-type of this constructed type.
   * @return ItemReference The specified type of this constructor.
   */
  public ItemReference getTypeReference()
  { return thisType; }

  /** Method to set the super-type of this constructed type.
   * @param reference The newly specified type of this constructor.
   * @return void
   */
  public void setTypeReference(ItemReference reference) {
    removeChildObject(thisType);
    thisType = reference;
    addChildObject(thisType);
  }




  /** Method to create a new constructor value for this type.
   * @param constructor A link to the item of this constructor - needed for observers.
   * @return ConstructedTypeConstructor The value of this constructor.
   */
  public ConstructedTypeConstructor createConstructor(ItemReference constructor)
  { return new ConstructedTypeConstructor(constructor); }


  /** Method to create a new constructor value for this type.
   * @param constructor A link to the item of this constructor - needed for observers.
   * @param params The parameter list for this constructor.
   * @return ConstructedTypeConstructor The value of this constructor.
   */
  public ConstructedTypeConstructor createConstructor(ItemReference constructor, ItemList params)
  { return new ConstructedTypeConstructor(constructor, params); }


  /** Method to create a new constructor value for this type.
   * @param constructor A link to the item of this constructor - needed for observers.
   * @param params The parameter list for this constructor.
   * @param recogLabel The label to hold for the recognizer for now.
   * @return ConstructedTypeConstructor The value of this constructor.
   */
  public ConstructedTypeConstructor createConstructor(ItemReference constructor, ItemList params, Label recogLabel)
  { return new ConstructedTypeConstructor(constructor, params, recogLabel); }




  /** Method to add a new constructor item to the definition.
   * @param constructor The next constructor Item.
   * @return void
   */
  public void addConstructor(Item constructor)
  {
    if (constructorList == null)
      { constructorList = new ItemList(); }
    constructorList.add(constructor);
    addChildObject(constructor);
  }


  /** Method to set all the constructors at once. Removes all the old constructors.
   * @param constructors The new constructor Items.
   * @return void
   */
  public void setConstructors(ItemList constructors)
  {
    removeChildList(constructorList);
    constructorList = constructors;
    addChildList(constructorList);
  }


  /** Method to retreive all the constructors.
   * @return ItemList All the constructors for this Type.
   */
  public ItemList getConstructors()
  { return constructorList; }



  /** Method to attempt to automatically generate all the observers and
   *  recognizers for this construction. Assumes that the recognizer items
   *  have been set with dummy items containing the labels to use.
   * @return void
   */
  public void generateConstructorData()
  {
    int pos;
    ConstructedTypeConstructor nextConstructor;

    if (constructorList == null)
      { constructorList = new ItemList(); }
    for (pos = 0; pos < constructorList.size(); pos++)
      {
	nextConstructor = (ConstructedTypeConstructor)(constructorList.get(pos).getValue());
	nextConstructor.generateObservers();
	nextConstructor.generateRecognizer((Label)(nextConstructor.getRecognizer().getLabel()));
      }
  }




 




  /** To get the string representation of the object
   * @return String representing the string of this object
   */
  public String toString()
  {
    int pos;
    String result;

    result = "data ";
    if ((constructorList == null) || constructorList.isEmpty())
      { return result + "NULL"; }
    result = result + constructorList.get(0);
    for (pos = 1; pos < constructorList.size(); pos++)
      { result = result + " | " + constructorList.get(pos); }
    return result;
  }


  /** To get the string required to define this Constructed Type.
   * @param indent The current string to use as a logical block indentor.
   * @return String The declaration of this Constructed Type.
   */
  public String publishROM(String indent)
  {
    int pos;
    String result;

    result = "data ";
    if ((constructorList == null) || constructorList.isEmpty())
      { return result + "NULL"; }
    result = result + constructorList.get(0).getValue().publishROM(indent + defaultPublishROMIndent);
    for (pos = 1; pos < constructorList.size(); pos++)
      { result = result + " | " + constructorList.get(pos).getValue().publishROM(indent + defaultPublishROMIndent); }
    return result;
  }
  

}



