//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Garrin kimmell@ittc.ku.edu
//               Joseph josephk@cs.adelaide.edu.au
//
//  $Id: ConstructedValue.java,v 1.3 2002/07/16 07:15:48 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: ConstructedValue.java,v $
//  Revision 1.3  2002/07/16 07:15:48  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:03:39  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.2  2002/03/12 22:51:55  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:16  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.1.2.2  2002/01/04 12:30:47  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.1.2.1  2001/11/24 06:44:45  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents the <B>ConstructedValue</B> object. An instance of this
 *  class represents a value of a Constructed Type in Rosetta. This is a
 *  value that has been formed using a constructor function defined for a
 *  specific constructed type.
 * @author <I> Garrin Kimmell, Joseph Kuehn </I>
 * (kimmell@ittc.ku.edu, josephk@cs.adelaide.edu.au)
 */
public class ConstructedValue extends CompositeValue {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "Constructor" };


  private ItemReference myConstructor;


  /** Primary constructor.
   * @param constructor The constructed type function used to build this value.
   */
  public ConstructedValue(ItemReference constructor)
  {
    myConstructor = constructor;
    addChildObject(myConstructor);
  }


  /** Secondary constructor.
   * @param constructor The constructed type function used to build this value.
   * @param contents The value of the contents of the constructed value.
   */
  public ConstructedValue(ItemReference constructor, ExpressionList contents)
  {
    super(contents);
    myConstructor = constructor;
    addChildObject(myConstructor, 0);
  }


  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public ConstructedValue(Map settings)
    {
    super(settings);

    myConstructor = (ItemReference)getSetting(ConstructedValue.keySet[0]);

    addChildObject(myConstructor, 0);
    }




  /** Method to retreive the constructor used to generate this constructed value.
   * @return ItemReference The constructor reference.
   */
  public ItemReference getConstructor()
  { return myConstructor; }

  /** Method to set the constructor used to generate this constructed value.
   * @param constructor The constructor reference.
   */
  public void setConstructor(ItemReference constructor)
    {
    removeChildObject(myConstructor);
    myConstructor = constructor;
    addChildObject(myConstructor, 0);
    }


 


  /** To get the string representation of the object
   * @return String representing the string of this object
   */
  public String toString()
  { return myConstructor + "(" + contentString(", ") + ")"; }


  /** To get the string required to define this Constructed Value.
   * @param indent The current string to use as a logical block indentor.
   * @return String The declaration of this Constructed Value.
   */
  public String publishROM(String indent) {
    return myConstructor + "(" + contentPublish(", ", indent) + ")"; 
  }
    
}



