//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: 
//
//  $Id: Function.java,v 1.4 2002/07/16 16:54:03 kimmell Exp $
//
//  Revision History:
//  ----------------
//  $Log: Function.java,v $
//  Revision 1.4  2002/07/16 16:54:03  kimmell
//  1. Added a constructor with a single parameter - and ExpressionList,
//  which builds a new function. This is a convenience mehtod used when
//  the Function is only used to represent a function type. It creates a
//  ItemList of parameters, with meaningless parameter names, and sets the
//  range.
//
//  2. Added a method getFunctionType() which returns an ExpressionList
//  combining the types of parameters and the range.
//
//  Revision 1.3  2002/07/16 07:15:48  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:05:14  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.3  2002/04/17 19:55:05  cvs
//  Modified ROM structural elements, such as Item and Label, to inherit from Expression. This allows the meta-functions to operate along the same lines as any other function.
//  -kimmell
//
//  Revision 1.1.2.2  2002/03/12 22:51:57  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:18  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.7.2.4  2002/01/04 12:30:48  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.7.2.3  2001/11/24 06:44:46  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//  Revision 1.7.2.2  2001/11/14 19:17:19  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//  Revision 1.7.2.1  2001/11/07 22:41:04  cvs
//  Added treenode functionality to the ROM.
//  -Joseph
//
//  Revision 1.7  2001/03/12 23:56:47  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;
import java.util.Iterator;


/**
 * <b>Function</b>:  Represents a Rosetta Function.
 *
 * @author <i>Steve Parker</i> 
 * (<a href="mailto:sparker@averstar.com">sparker@averstar.com</a>).
 */
public class Function extends Expression {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "InterfaceItems", "Range", "Expression" };


  private ItemList interfaceList;
  private Expression myRange, myExpression;

  /** Primary constructor.
   */
  public Function()
  { this(new ItemList(), null, null); }


  /** Secondary constructor.
   * @param params The interface listing for this function.
   * @param range The return type for this function. Note that if the
   *             function is named than the item should have a map type.
   * @param value The Expression the function creates.
   */
  public Function(ItemList params, Expression range, Expression value) {
    interfaceList = params;
    myRange = range;
    myExpression = value;
    addChildList(interfaceList);
    addChildObject(myRange);
    addChildObject(myExpression);
  }


  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public Function(Map settings)
    {
    super(settings);

    interfaceList = (ItemList)getSetting(Function.keySet[0]);
    myRange = (Expression)getSetting(Function.keySet[1]);
    myExpression = (Expression)getSetting(Function.keySet[2]);

    addChildList(interfaceList);
    addChildObject(myRange);
    addChildObject(myExpression);
    }


  
  /**
   * Creates a new <code>Function</code> instance.
   * This constructor is used when the Function is to be used only as
   * a type. Because of this, we don't care about parameter names, so
   * the constructor automatically creates a parameter list, and sets
   * the range correctly.
   *
   * @param type an <code>ExpressionList</code> value
   */
  public Function(ExpressionList type) {
    String paramPrefix = "x";
    int paramSuffix = 0;
    Expression current;
    ItemList params = new ItemList();

    Iterator typeIter = type.iterator();
    while (true) {
      current = (Expression) typeIter.next();
      if (typeIter.hasNext()) {
	Label paramName = new Label(paramPrefix + paramSuffix);
	Item param = new Item(paramName, current, null);
	params.add(param);
	paramSuffix = paramSuffix + 1;
      } else {
	myRange = current;
	myExpression = null;
	break;
      } 
      
    } // end of while (true)
    
    interfaceList = params;
  }


  /** To add another parameter to the interface.
   * @param interfaceItem The parameter to be added.
   * @return void
   */
  public void addInterfaceItem(Item interfaceItem) {
    if (interfaceList == null)
      { interfaceList = new ItemList(); }
    interfaceList.add(interfaceItem);
    addChildObject(interfaceItem);
  }


  /** To set the interface list outright. Removes old Items.
   * @param listIn The new parameter list of the function.
   * @return void
   */
  public void setInterfaceItems(ItemList listIn) {
    removeChildList(interfaceList);
    interfaceList = listIn;
    addChildList(interfaceList);
  }
	
  /** To get the interface list for this function.
   * @return ItemList The parameters of the function.
   */
  public ItemList getInterfaceItems() {
    return interfaceList;
  }

  /** To set the Expression value of the function.
   * @param expr The new lambda-expression the function will sub into.
   * @return void
   */
  public void setExpression(Expression expr){
    removeChildObject(myExpression);
    myExpression = expr;
    addChildObject(myExpression);
  }
	
  /**To get the expression of the function
   * @return Expression The lambda-expression the function subs into.
   */
  public Expression getExpression(){
    return myExpression;
  }



  /** To set the range of the function. This is important since the
   *  type of the named object should be a mapping.
   * @param range The Expression of the range of the function.
   * @return void
   */
  public void setRange(Expression range){
    removeChildObject(myRange);
    myRange = range;
    addChildObject(myRange);
  }
	
  /**To get the range of the function.
   * @return Expression The declared range of the function.
   */
  public Expression getRange(){
    return myRange;
  }



  public ExpressionList getFunctionType() {
    ExpressionList type = new ExpressionList();
    for (Iterator paramIter = interfaceList.iterator(); paramIter.hasNext();){
      type.add(((Item)paramIter.next()).getType());
    } // end of for (Iterator paramIter = interfaceList.iterator(); paramIter.hasNext();)
    type.add(myRange);
    return type;
  }



  /** To get the string representation of the object
   * @return String representing the string of this object   
   */
  public String toString() {
    int pos;
    String answer;

    answer = "<*(";
    if ((interfaceList != null) && (!interfaceList.isEmpty()))
      {
	answer = answer + interfaceList.get(0);
	for (pos = 1; pos < interfaceList.size(); pos++)
	  { answer = answer + "; " + interfaceList.get(pos); }
      }
    answer = answer + ") :: " + getRange() + " is " + getExpression() + "*>";
    return answer;
  }


  /** Method to get the infix string declaration of the object
   * @param myLabel The name of this syntactical shorthand object.
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the declaration of this object.
   */
  public String publishROM(Label myLabel, String indent)
  {
    int pos;
    String answer;

    answer = myLabel + "(";
    if ((interfaceList != null) && (!interfaceList.isEmpty()))
      {
	answer = answer + interfaceList.get(0).publishROM(indent + defaultPublishROMIndent);
	for (pos = 1; pos < interfaceList.size(); pos++)
	  { answer = answer + "; " + interfaceList.get(pos).publishROM(indent + defaultPublishROMIndent); }
      }
    answer = answer + ") :: ";
    if (getRange() != null)
      { answer = answer + getRange().publishROM(indent + defaultPublishROMIndent); }
    else
      { answer = answer + "NULL"; }
    if (getExpression() != null)
      { answer = answer + " is " + getExpression().publishROM(indent + defaultPublishROMIndent); }
    return answer;
  }



  /** To get the string declaration of the object
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the declaration of this object   
   */
  public String publishROM(String indent) {
    int pos;
    String answer;

    answer = "<*(";
    if ((interfaceList != null) && (!interfaceList.isEmpty()))
      {
	answer = answer + interfaceList.get(0).publishROM(indent + defaultPublishROMIndent);
	for (pos = 1; pos < interfaceList.size(); pos++)
	  { answer = answer + "; " + interfaceList.get(pos).publishROM(indent + defaultPublishROMIndent); }
      }
    answer = answer + ") :: ";
    if (getRange() != null)
      { answer = answer + getRange().publishROM(indent + defaultPublishROMIndent); }
    else
      { answer = answer + "NULL"; }
    if (getExpression() != null)
      { answer = answer + " is " + getExpression().publishROM(indent + defaultPublishROMIndent); }
    answer = answer + "*>";
    return answer;
  }


}
