//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Garrin kimmell@ittc.ukans.edu
//               Joseph josephk@cs.adelaide.edu.au
//
//  $Id: IfExpression.java,v 1.3 2002/07/16 07:15:48 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: IfExpression.java,v $
//  Revision 1.3  2002/07/16 07:15:48  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:05:29  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.2  2002/03/12 22:51:57  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:18  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.1.2.2  2002/01/04 12:30:48  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.1.2.1  2001/11/24 06:44:46  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents the <B>IfExpression</B> object. An instance of this
 *  class indicates that an 'if' statement has been used. Because the
 *  if statement has a unique structure not duplicated by any other
 *  function it is necessary to include it here.
 * @author <I>Garrin Kimmell and Joseph Kuehn</I>
 *  (kimmell@ittc.ukans.edu, josephk@cs.adelaide.edu.au)
 */
public class IfExpression extends Expression {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "IfValue", "ThenValue", "ElseValue" };


  private Expression ifValue, thenValue, elseValue;

  /** The primary Constructor
   */
  public IfExpression()
    { this(null, null, null); }


  /** A secondary Constructor
   */
  public IfExpression(Expression ifExpr, Expression thenExpr, Expression elseExpr)
    {
    ifValue = ifExpr;
    thenValue = thenExpr;
    elseValue = elseExpr;

    addChildObject(ifValue);
    addChildObject(thenValue);
    addChildObject(elseValue);
    }

  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public IfExpression(Map settings)
    {
    super(settings);

    ifValue = (Expression)getSetting(IfExpression.keySet[0]);
    thenValue = (Expression)getSetting(IfExpression.keySet[1]);
    elseValue = (Expression)getSetting(IfExpression.keySet[2]);

    addChildObject(ifValue);
    addChildObject(thenValue);
    addChildObject(elseValue);
    }


	
  /** The function to get the 'if' test-expression of this if statement.
   * @return Expression The 'if' test-expression.
   */
  public Expression getIfValue()
    { return ifValue; }

  /** The function to set the 'if' test-expression of this if statement.
   * @param ifExpr The new 'if' test-expression.
   * @return void
   */
  public void setIfValue(Expression ifExpr)
    {
    removeChildObject(ifValue);
    ifValue = ifExpr;
    addChildObject(ifValue);
    }




  /** The function to get the 'then'  result expression of this if statement.
   * @return Expression The expression result if the 'if' expression is true.
   */
  public Expression getThenValue()
    { return thenValue; }

  /** The function to set the 'then'  result expression of this if statement.
   * @param thenValue The new expression result if the 'if' expression is true.
   * @return void
   */
  public void setThenValue(Expression thenExpr)
    {
    removeChildObject(thenValue);
    thenValue = thenExpr;
    addChildObject(thenValue);
    }



  /** The function to get the 'else' Expression.
   * @return Expression The 'else' of the if statement.
   */
  public Expression getElseValue()
    { return elseValue; }

  /** The function to set the 'else' Expression.
   * @param elseExpr The new else expression of the if statement.
   * @return void
   */
  public void setElseValue(Expression elseExpr)
    {
    removeChildObject(elseValue);
    elseValue = elseExpr;
    addChildObject(elseValue);
    }





  /** To get the string representation of the object
   * @return String representing the string of this object
   */
  public String toString()
    {
    String answer;

    answer = "(" + ifValue + " ? " + thenValue + " | " + elseValue + ")";
    return answer;
    }


  /** To get the string declaration of the object
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the declaration of this object
   */
  public String publishROM(String indent)
    {
    String answer;

    answer = "if " + ifValue + " then " + thenValue;
    if (elseValue != null)
      { answer = answer + " else " + elseValue; }
    answer = answer + " end if";
    return answer;
    }


}

