//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: Item.java,v 1.7 2002/11/20 17:15:31 kimmell Exp $
//
//  Revision History:
//  ----------------
//  $Log: Item.java,v $
//  Revision 1.7  2002/11/20 17:15:31  kimmell
//  The key for the parameter mode decoration is now "__#". Made the
//  required change in publishROM.
//
//  Revision 1.6  2002/11/19 16:35:58  kimmell
//  1. Fixed the formatting of whitespace when publishing.
//
//  Revision 1.5  2002/11/19 16:24:19  kimmell
//  1. Modified printing of the mode. The publishROM routine expects the
//     mode - if it exists - be a decoration of the Item. The decoration
//     key is "mode", and the value is an instance of ItemReference.
//
//  -kimmell
//
//  Revision 1.4  2002/09/05 06:13:13  tarp
//  Fixed a lower-case typo in the keySet of Item.
//
//  Revision 1.3  2002/07/16 07:15:48  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:08:37  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.3  2002/04/17 19:55:05  cvs
//  Modified ROM structural elements, such as Item and Label, to inherit from Expression. This allows the meta-functions to operate along the same lines as any other function.
//  -kimmell
//
//  Revision 1.1.2.2  2002/03/12 22:51:58  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:18  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.8.2.4  2002/01/04 12:30:49  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.8.2.3  2001/11/24 06:44:46  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//  Revision 1.8.2.2  2001/11/14 19:17:19  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//  Revision 1.8.2.1  2001/11/07 22:41:04  cvs
//  Added treenode functionality to the ROM.
//  -Joseph
//
//  Revision 1.8  2001/05/30 16:31:58  cvs
//  Added some accessor methods to the classes. The methods are used by the Moml to Rosetta converter.
//
//  --Srinivas
//
//  Revision 1.7  2000/12/04 19:39:38  cvs
//  ROM is serialized and stored in packageName_ROM.ser
//  Use clause can now be used for importing packages in same directory.
//  Class Item is now an abstract class.
//  -- Cindy
//
//  Revision 1.6  2000/11/17 23:14:13  cvs
//  Added the method labelLookup in some classes and changed the same
//  method in others 
//  --srinivas
//
//  Revision 1.5  2000/10/25 20:50:07  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//  Revision 1.4  2000/09/16 19:48:16  cvs
//  Revamped ROM class hierarchy.  Committed partially skelatal
//  classes that successfully compile. 
//
//  Revision 1.3  2000/08/22 18:40:17  cvs
//  First Commit by Steve Parker, Averstar.  Basic Structure of
//  classes setup for ROM. 
//
//  Revision 1.3  2000/08/09 04:29:53  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//  Revision 1.2  2000/08/08 23:40:25  cvs
//  Revised version of Item class. -David
//
//  Revision 1.1  2000/08/08 22:50:55  cvs
//  This is the Item class. -David
// 
// 
//
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents the <B>Item</B> object. This is the fundamental
 *  building block of the ROM, and provides the concept of a 3-tuple
 *  consisting of (label, type, value). Every definition in Rosetta
 *  is conceptually is an Item, with compound types containing
 *  collections of references to other Items in a meaningful expression.
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public class Item extends BaseObject {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "Label", "Type", "Value" };


  private Label label;
  private Expression type;
  private Expression value;


  /** The primary constructor
   * @param labelIn Label representing label of item
   * @param typeIn Type representing the type
   * @param valueIn Value representing value of the item
   */
  public Item(Label labelIn, Expression typeIn, Expression valueIn)
    {
    label = labelIn;
    type = typeIn;
    value = valueIn;

    addChildObject(label);
    addChildObject(type);
    addChildObject(value);
    }


  /** Secondary Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public Item(Map settings)
    {
    super(settings);

    label = (Label)getSetting(Item.keySet[0]);
    type = (Expression)getSetting(Item.keySet[1]);
    value = (Expression)getSetting(Item.keySet[2]);

    addChildObject(label);
    addChildObject(type);
    addChildObject(value);
    }



  /** Old constructor
   * @param labelIn Label representing label of item
   * @param typeIn Type representing the type
   * @param valueIn Value representing value of the item
   * @param stringIn String representation of item
   * @deprecated Stop using the string hook. It's misleading.
   */
  public Item(Label labelIn, Expression typeIn, Expression valueIn, String stringIn)
    { this(labelIn, typeIn, valueIn); }

    

  /** To get the Label of this Item.
   * @return Label The Label of this Item.
   */
  public Label getLabel() {
    return label;
  }

  /** To set the Label of this Item.
   * @param labelIn The new Label of this Item.
   * @return void
   */
  public void setLabel(Label labelIn) {
    removeChildObject(label);
    label = labelIn;
    addChildObject(label);
  }



  /** To retrieve the type of this Item.
   * @return Expression The type of this object.
   */
  public Expression getType()
    {  return type; }

  /** To set the type of this Item.
   * @param typeIn The new type of this Item.
   * @return void
   */
  public void setType(Expression typeIn) {
    removeChildObject(type);
    type = typeIn;
    addChildObject(type);
  }


  /** To retrieve the value of this Item.
   * @return Expression The value of this Item.
   */
  public Expression getValue() {
    return value;
  }

  /** To set the value of this Item.
   * @param exprIn The new value associated with this Item.
   * @return void
   */
  public void setValue(Expression exprIn) {
    removeChildObject(value);
    value = exprIn;
    addChildObject(value);
  }



  /** To get the name (string of the label) of this Item.
   * @return String The String corresponding to the Label of this Item.
   */
  public String toString() {
    return "" + getLabel();
  }


  /** To get the full string required to declare this Item.
   * @param indent The current string to use as a logical block indentor.
   * @return String The String corresponding to the declaration of this Item.
   */
  public String publishROM(String indent) {
    String answer;

    if ((getValue() != null) && (getValue() instanceof Facet))
      { return ((Facet)getValue()).publishROM(getLabel(), indent); }

    if ((getValue() != null) && (getValue() instanceof Function))
      { return ((Function)getValue()).publishROM(getLabel(), indent); }

    answer = getLabel() + "::";
    // Print out the mode
    // This is currently an ItemReference.
    ItemReference mode = (ItemReference) getDecoration("__#");
    if (mode != null) {
      answer = answer + mode + " ";
    }

    if (getType() != null)
      { answer = answer + getType().publishROM(indent + defaultPublishROMIndent); }
    else
      { answer = answer + "NULL"; }
    if (getValue() != null)
      { answer = answer + " is " + getValue().publishROM(indent + defaultPublishROMIndent); }
    return answer;
  }

  /** To get the full string required to declare this Item as a term.
   * @param indent The current string to use as a logical block indentor.
   * @return String The String corresponding to the declaration of this Item as a term.
   */
  public String publishTerm(String indent) {
    String answer;

    answer = getLabel() + ": ";
    if (getValue() != null)
      { answer = answer + getValue().publishROM(indent + defaultPublishROMIndent); }
    else
      { answer = answer + "NULL"; }
    return answer;
  }


}    
