//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: ItemReference.java,v 1.5 2002/07/17 20:26:34 jstan Exp $
//
//  Revision History:
//  ----------------
//  $Log: ItemReference.java,v $
//  Revision 1.5  2002/07/17 20:26:34  jstan
//  *** empty log message ***
//
//  Revision 1.3  2002/07/16 07:15:48  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 20:58:39  kimmell
//  1. Removed instantiate and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.3  2002/04/17 19:55:06  cvs
//  Modified ROM structural elements, such as Item and Label, to inherit from Expression. This allows the meta-functions to operate along the same lines as any other function.
//  -kimmell
//
//  Revision 1.1.2.2  2002/03/12 22:51:58  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:19  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.9.2.4  2002/01/04 12:30:49  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.9.2.3  2001/11/24 06:44:47  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//  Revision 1.9.2.2  2001/11/14 19:17:20  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//  Revision 1.9.2.1  2001/11/07 22:41:04  cvs
//  Added treenode functionality to the ROM.
//  -Joseph
//
//  Revision 1.9  2001/03/12 23:56:47  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.8  2000/12/04 19:39:38  cvs
//  ROM is serialized and stored in packageName_ROM.ser
//  Use clause can now be used for importing packages in same directory.
//  Class Item is now an abstract class.
//  -- Cindy
//
//  Revision 1.7  2000/11/27 23:46:52  cvs
//  Changed the publishROM method in  all the classes and added some
//  predefined operators to the Universe. 
//  --srinivas
//
//  Revision 1.6  2000/11/17 23:14:13  cvs
//  Added the method labelLookup in some classes and changed the same
//  method in others 
//  --srinivas
//
//  Revision 1.5  2000/11/13 21:49:11  cvs
//  Added the function buildROM to all the classes.
//  --srinivas
//
//  Revision 1.4  2000/10/30 22:34:30  cvs
//  Added secondary constructors to some of the classes and some
//  accessor functions are also added 
//  --srinivas
//
//  Revision 1.3  2000/08/30 20:39:11  cvs
//  Changed Expression class to inherit from Value and amended all
//  other classes affected by this change. -David 
//
//  Revision 1.2  2000/08/10 00:49:48  cvs
//  Implemented the new hierarchy for ItemReference and added some
//  accessor methods.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 23:03:09  cvs
//  Dropped the ROM_ prefix on all files; dumped getElt() method from
//  List.java, made Universe a subclass of BaseObject. --David 
//
//  Revision 1.2  2000/08/09 04:29:53  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 00:51:15  cvs
//  Skeletons for several added classes. -David
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents the <B>ItemReference</B> object. This indiciates
 *  an item is being referred to from inside an Expression, that is
 *  that this is an _occurance_ of an name, not a _definition_.
 *  At creation an item reference is considered 'local', that is its
 *  target can only be determined by beginning at the place of incidence
 *  and tracking through the various scopes looking for the item it refers
 *  to. However, once that item is found a 'global reference' might be
 *  determinable, that is a reference beginning from a fixed root of the
 *  the name space. The advantage of this method is that it is much faster
 *  to resolve as only basic item lookup features are required, and the
 *  more advanced scoping rules have already been determined. Lastly, a
 *  direct link can be used when the item has no global name - that is it
 *  occurs in the interface of an anonymous function.
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public class ItemReference extends Expression {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "ItemReferred", "RelativeName", "AbsoluteName", "ReferenceName" };


  private Item itemReferred;
  private LabelList relativeName, absoluteName;
  private String referenceName;


  /** Primary constructor
   * @param itemIn The item being referred to.
   */
  public ItemReference(Item itemIn) {
    itemReferred = itemIn;
    relativeName = null;
    absoluteName = null;
    referenceName = null;
  }


  /** Secondary constructor - used during the initial syntax tokenisation.
   * @param localRef The local reference of the item being referred to.
   */
  public ItemReference(LabelList localRef) {
    itemReferred = null;
    relativeName = localRef;
    absoluteName = null;
    referenceName = null;
    addChildList(relativeName);
  }

  public ItemReference(String argReferenceName) {
    itemReferred = null;
    relativeName = null;
    absoluteName = null;
    referenceName = argReferenceName;
    
 }


  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public ItemReference(Map settings)
    {
    super(settings);

    itemReferred = (Item)getSetting(ItemReference.keySet[0]);
    relativeName = (LabelList)getSetting(ItemReference.keySet[1]);
    absoluteName = (LabelList)getSetting(ItemReference.keySet[2]);
    referenceName = (String)getSetting(ItemReference.keySet[3]);

    addChildList(relativeName);
    addChildList(absoluteName);
    }






  /** Method to get the item directly linked to.
   * @return Item The direct link represented by this reference.
   */
  public Item getItemReferred() {
    return itemReferred;
  }

  /** To set the item being referred
   * @param itemIn The item to be referred to 
   * @return void
   */
  public void setItemReferred(Item itemIn) {
    itemReferred = itemIn;
  }



  public String getReferenceName() {
    return referenceName;
  }

  public void setReferenceName(String argReferenceName) {
    referenceName = argReferenceName;
  }


  /** Method to retreive the reference to the item.
   * @return LabelList The qualified item name - could still be null or local though.
   */
  public LabelList getRelativeName()
  { return relativeName; }

  /** Method to set the reference path to the item.
   * @param refPath The qualified item name - could still be null or local though.
   * @return void
   */
  public void setRelativeName(LabelList refPath) {
    removeChildList(relativeName);
    relativeName = refPath;
    addChildList(relativeName);
  }



  /** Method to retreive the reference to the item.
   * @return LabelList The qualified item name - could still be null or local though.
   */
  public LabelList getAbsoluteName()
  { return absoluteName; }

  /** Method to set the reference path to the item.
   * @param refPath The qualified item name - could still be null or local though.
   * @return void
   */
  public void setAbsoluteName(LabelList refPath) {
    removeChildList(absoluteName);
    absoluteName = refPath;
    addChildList(absoluteName);
  }





  /** To get the string representation of the object
   * @return String representing the string of this object
   */
  public String toString() {
    if (relativeName != null) {
      return relativeName.toString(".");
    } else {
      return referenceName;
    } // end of else
      
  }
  
  /** To get the string declaration of the object
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the declaration of this object
   */
  public String publishROM(String indent) {
    return toString();
  }

}


