//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: Label.java,v 1.3 2002/07/16 07:15:48 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Label.java,v $
//  Revision 1.3  2002/07/16 07:15:48  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:08:57  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.2  2002/03/12 22:51:58  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:19  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.7.2.3  2002/01/04 12:30:49  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.7.2.2  2001/11/24 06:44:47  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//  Revision 1.7.2.1  2001/11/14 19:17:20  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//  Revision 1.7  2001/05/25 21:05:10  cvs
//  some changes made so that the dvtg works fine. It should not affect other things
//
//  --srinivas
//
//  Revision 1.6  2001/03/12 23:56:47  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.5  2000/11/27 23:46:52  cvs
//  Changed the publishROM method in  all the classes and added some
//  predefined operators to the Universe. 
//  --srinivas
//
//  Revision 1.4  2000/11/17 23:14:13  cvs
//  Added the method labelLookup in some classes and changed the same
//  method in others 
//  --srinivas
//
//  Revision 1.3  2000/10/25 20:50:07  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//  Revision 1.2  2000/08/22 18:40:17  cvs
//  First Commit by Steve Parker, Averstar.  Basic Structure of
//  classes setup for ROM. 
//
//  Revision 1.1  2000/08/09 04:29:54  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents the <B>Label</B> object. An instance of this
 *  class represents the name of an Item, either in its declaration
 *  or as a high-order reference to be passed to a Meta-function.
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public class Label extends Literal {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "Data" };


  private String labelName;

  /** The primary Constructor
   * @param nameIn The string representing name of the label.
   */
  public Label(String nameIn) {
    labelName = nameIn;
  }

  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public Label(Map settings)
    {
    super(settings);

    labelName = (String)getSetting(Label.keySet[0]);
    }
	

  /** To check if this label equals another label
   * @param labelIn The label to be compared to
   * @return boolean, true if the strings of both labels are equal,
   * false otherwise
   */
  public boolean equals(Label labelIn) {
    if ((labelIn == null) || ((labelName == null) && (labelIn.labelName != null)))
      { return false; }
    return (labelName != null) && labelName.equalsIgnoreCase(labelIn.labelName);
  }


  /** To get name attached to this label.
   * @return String The name attached to this label.
   */
  public String getData() {
    return labelName;
  }

  /** To set name attached to this label.
   * @param name The new name attached to this label.
   * @return void
   */
  public void setData(String name) {
    labelName = name;
  }


  /** To get the string representation of the object
   * @return String representing the string of this object
   */
  public String toString() {
    return labelName;
  }


  /** To get the string representation of the object
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the string of this object
   */
  public String publishROM(String indent) {
    return labelName;
  }

}



