//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: LetFunction.java,v 1.3 2002/07/16 07:15:48 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: LetFunction.java,v $
//  Revision 1.3  2002/07/16 07:15:48  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:09:41  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.3  2002/04/17 19:55:06  cvs
//  Modified ROM structural elements, such as Item and Label, to inherit from Expression. This allows the meta-functions to operate along the same lines as any other function.
//  -kimmell
//
//  Revision 1.1.2.2  2002/03/12 22:51:59  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:19  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.3.2.3  2002/01/04 12:30:49  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.3.2.2  2001/11/24 06:44:47  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//  Revision 1.3.2.1  2001/11/14 19:17:20  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents the <B>LetFunction</B> object. An instance of this
 *  class indicates that the lambda-calculus expression abstraction
 *  construct we call "let" in Rosetta has been used. This is _NOT_
 *  simply an aggregate function reference because let has a very
 *  special syntax/semantic property beyond an ordinary function call.
 * @author <I>Srinivas Akkipeddi</I>
 * <a href="mailto:akki@ittc.ukans.edu"></a>
 */
public class LetFunction extends Expression {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "Item", "Expression" };

	
  private Item item;
  private Expression expr;

  /** The primary Constructor
   */
  public LetFunction() {
    this(null, null);
  }

  /** The secondary constructor
   */
  public LetFunction(Item lambdaItem, Expression lambdaExpr)
  {
    item = lambdaItem;
    expr = lambdaExpr;
    addChildObject(item);
    addChildObject(expr);
  }


  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public LetFunction(Map settings)
    {
    super(settings);

    item = (Item)getSetting(LetFunction.keySet[0]);
    expr = (Expression)getSetting(LetFunction.keySet[1]);

    addChildObject(item);
    addChildObject(expr);
    }




  /** The function to set the Item of the Let statement.
   * @param lambdaItem The new Item value.
   * @return void
   */
  public void setItem(Item lambdaItem){
    removeChildObject(item);
    item = lambdaItem;
    addChildObject(item);
  }
	
  /**The function to get the Item of the Let statement.
   * @return Item The lambda Item of this Let statement.
   */
  public Item getItem(){
    return item;
  }



  /** The function to set the Expression to lambda reduce on.
   * @param exprIn The new predicate expression of the Let statement.
   * @return void
   */
  public void setExpression(Expression exprIn){
    removeChildObject(expr);
    expr = exprIn;
    addChildObject(expr);
  }

  /** The function to get the Expression to lambda reduce on.
   * @return Expression The predicate of the Let statement.
   */
  public Expression getExpression(){
    return expr;
  }










  /** To get the string representation of the object
   * @return String representing the string of this object
   */
  public String toString(){
    String answer;

    answer = "let (" + item + ") in " + expr + " end let";
    return answer;
  }


  /** Method to convert an Item's publishROM into one appropriate
   *  for a Let statement. Like normal but uses "be" instead of "is".
   * @param Item The item to translate.
   * @param indent The current string to use as a logical block indentor.
   * @return String It's declaration in a Let statement.
   */
  protected String letItemPublish(Item myItem, String indent)
  {
    String result;

    if (myItem == null)
      { return "NULL"; }

    result = myItem + " :: ";
    if (myItem.getType() != null)
      { result = result + myItem.getType().publishROM(indent + defaultPublishROMIndent); }
    else
      { result = result + "NULL"; }
    result = result + " be ";
    if (myItem.getValue() != null)
      { result = result + myItem.getValue().publishROM(indent + defaultPublishROMIndent); }
    else
      { result = result + "NULL"; }
    return result;
  }


  /** To get the string declaration of the object
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the declaration of this object
   */
  public String publishROM(String indent) {
    String answer;

    answer = "let (" + letItemPublish(item, indent) + ") in ";
    if (expr != null)
      { answer = answer + expr.publishROM(indent + defaultPublishROMIndent); }
    else
      { answer = answer + "NULL"; }
    answer = answer + " end let";
    return answer;
  }

}

