//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//      authors: Joseph josephk@cs.adelaide.edu.au
//
//  $Id: QuantifierReference.java,v 1.3 2002/07/16 07:15:48 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: QuantifierReference.java,v $
//  Revision 1.3  2002/07/16 07:15:48  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//
//============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents an <B>QuantifierReference</B> object. An instance
 *  of this class indicates an invocation of one of the special
 *  quantifier (grouped) Rosetta operators (eg. 'sel', 'forall').
 *  Of particular importance is that: the expression represents the predicate that
 *  the user wishes to group by; secondly, this reference also
 *  defines new items to run this predicate with, so be aware that
 *  these are not items being referenced from elsewhere.
 * @author <I> Joseph Kuehn </I>
 * (josephk@cs.adelaide.edu.au)
 */
public class QuantifierReference extends Expression {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "Reference", "Variables", "Expression" };


  private ItemReference quantifierRef;
  private ItemList locals;
  private Expression predicate;


  /** Primary constructor.
   * @param quantifier The reference to the quantifier being used.
   */
  public QuantifierReference(ItemReference quantifier)
    { this(quantifier, new ItemList(), null); }


  /** Secondary constructor.
   * @param quantifier The reference to the quantifier being used.
   * @param itemIn The quantifier function Item being referred.
   * @param params The new items declared for the predicate.
   * @param pred The Expression of the predicate to be used.
   */
    public QuantifierReference(ItemReference quantifier, ItemList params, 
			     Expression pred) {
      
      quantifierRef = quantifier;
      locals = params;
      predicate = pred;

      addChildObject(quantifierRef);
      addChildList(locals);
      addChildObject(predicate);
    }

  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public QuantifierReference(Map settings)
    {
    super(settings);

    quantifierRef = (ItemReference)getSetting(QuantifierReference.keySet[0]);
    locals = (ItemList)getSetting(QuantifierReference.keySet[1]);
    predicate = (Expression)getSetting(QuantifierReference.keySet[2]);

    addChildObject(quantifierRef);
    addChildList(locals);
    addChildObject(predicate);
    }




  /** Method to get the quantifier being referenced.
   * @return ItemReference The quantifier being referenced.
   */
  public ItemReference getReference()
    { return quantifierRef; }

  /** Method to set the quantifier being referenced.
   * @param reference The quantifier to be referenced.
   * @return void
   */
  public void setReference(ItemReference reference)
    {
    removeChildObject(quantifierRef);
    quantifierRef = reference;
    addChildObject(quantifierRef);
    }


  /** Method to add a local variable for use in the predicate.
   * @param paramIn The Item representing the local var to be added
   * @return void
   */
  public void addVariable(Item varIn)
    {
    if (locals == null)
      { locals = new ItemList(); }
    locals.add(varIn);
    addChildObject(varIn);
    }


  /** Method to retreive all the local variables usable in the predicate.
   * @return ItemList The list of local variables.
   */
  public ItemList getVariables()
    { return locals; }


  /** Method to set all the local variables at once (this deletes previous values).
   * @param variables The list of local variables for the predicate.
   * @return void
   */
  public void setVariables(ItemList variables)
    {
      removeChildList(locals);
      locals = variables;
      addChildList(locals);
    }


  /** Method to retreive the predicate.
   * @return Expression The value of the predicate.
   */
  public Expression getExpression()
    { return predicate; }


  /** Method to set predicate to be used in the quantifier evaluation.
   * @param pred The predicate.
   * @return void
   */
  public void setExpression(Expression pred)
    {
      removeChildObject(predicate);
      predicate = pred;
      addChildObject(predicate);    
    }


  /** To get the string representation of the object
   * @return String representing the string of this object
   */
  public String toString()
    {
    int pos;
    String result;
    
    result = getReference() + "(";
    if ((locals != null) && !locals.isEmpty())
      {
      result = result + locals.get(0);
      for (pos = 1; pos < locals.size(); pos++)
        { result = result + "; " + locals.get(pos); }
      }
    else
      { result = result + "NULL"; }
    result = result + " | " + predicate + ")";
    return result;
    }


  /** To get the string declaration of the object
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the declaration of this object
   */
  public String publishROM(String indent)
    {
    int pos;
    String result;

    result = getReference() + "(";
    if ((locals != null) && !locals.isEmpty())
      {
      result = result + locals.get(0).publishROM(indent + defaultPublishROMIndent);
      for (pos = 1; pos < locals.size(); pos++)
        { result = result + ", " + locals.get(pos).publishROM(indent + defaultPublishROMIndent); }
      }
    else
      { result = result + "NULL"; }
    result = result + " | ";
    if (predicate != null)
      { result = result + predicate.publishROM(indent + defaultPublishROMIndent); }
    else
      { result = result + "NULL"; }
    result = result + ")";
    return result;
    }

}

