package org.sldl.rosetta.ROM;


import java.lang.Math;

/** This class encapsulates the source file information used for tracking errors between
 *  the ROM and the source code. This is important because when giving a user type- or
 *  scope- or other forms of errors during mutli-pass parsing it is often <I>extremely</I>
 *  helpful to give them a clue where to look in their source code. Basically this
 *  information will help a tool provide a meaningful error location.
 * @author <I>Joseph Kuehn and Garrin Kimmell</I>
 * (josephk@cs.adelaide.edu.au, kimmell@ku.edu)
 */

public class SourceReference
  {
  private String myFilename;
  private int myStartCol, myStartRow, myEndCol, myEndRow;


  /** The primary Constructor. This should be used for creating entries for all the
   *  `atomic' language elements (such as labels, literals, etc...). More complex
   *  syntax can automatically generate this information.
   * @param filename This is the name of the file that the source comes from.
   * @param startC The column index of the first character of the syntax in the document.
   * @param startR The row index of the first character of the syntax in the document.
   * @param endC The column index of the last character of the syntax in the document.
   * @param endR The row index of the last character of the syntax in the document.
   */
  public SourceReference(String filename, int startC, int startR, int endC, int endR)
    {
    myFilename = filename;
    myStartCol = startC;
    myStartRow = startR;
    myEndCol = endC;
    myEndRow = endR;
    }


  /** A private constructor for use in deriving the source file attributes of compound
   *  syntax structures. This constructer creates a new entry that spans both of the
   *  old entries. It takes the filename from the first parameter (should they differ)
   *  or from the second parameter should the first filename be null.
   *  The names of the arguments do not need to follow the ordering in the file, the
   *  new entry spans them without assuming one entry occurs before or after the other.
   * @param lhs Some information about the first construct.
   * @param rhs Some information about the second construct.
   */
  private SourceReference(SourceReference lhs, SourceReference rhs)
    {
    if (lhs.myFilename != null)
      { myFilename = lhs.myFilename; }
    else
      { myFilename = rhs.myFilename; }
    myStartCol = Math.min(lhs.myStartCol, rhs.myStartCol);
    myStartRow = Math.min(lhs.myStartRow, rhs.myStartRow);
    myEndCol = Math.max(lhs.myEndCol, rhs.myEndCol);
    myEndRow = Math.max(lhs.myEndRow, rhs.myEndRow);
    }


  /** Retreive the column of the first character of the construct in the Document.
   * @return int The column index of the first character of the construct.
   */
  public int getStartCol()
    { return myStartCol; }

  /** Retreive the row of the first character of the construct in the Document.
   * @return int The row index of the first character of the construct.
   */
  public int getStartRow()
    { return myStartRow; }

  /** Retreive the column of the last character of the construct in the Document.
   * @return int The column index of the last character of the construct.
   */
  public int getEndCol()
    { return myEndCol; }

  /** Retreive the row of the last character of the construct in the Document.
   * @return int The row index of the last character of the construct.
   */
  public int getEndRow()
    { return myEndRow; }


  /** Retreive the filename of the Document the construct was taken from.
   * @return String The filename for where this entry was created.
   */
  public String getFilename()
    { return myFilename; }

  /** Set the filename of the Document the construct was taken from.
   * @param filename The new filename where this entry was created.
   * @return void
   */
  public void setFilename(String filename)
    { myFilename = filename; }


  /** Method to generate a string representation of this reference.
   * @return String The generated string.
   */
  public String toString()
    { return myFilename + " (" + myStartRow + "," + myStartCol + ")->(" + myEndRow + "," + myEndCol + ")"; }


  /** Method to generate a unique string representation of this reference.
   * @return String The generated string.
   */
  public String hashString()
    { return "___" + myFilename + "___(" + myStartRow + "," + myStartCol + ")->(" + myEndRow + "," + myEndCol + ")___"; }


  /** Method for creating an entry for compound language constructs from multiple
   *  sub-construct entries. For example in a mutli-part "if" statement the outer-most
   *  expression could <I>join()</I> all the sub-expressions together to correctly
   *  indicate the locations of the expressions within that if-statement. Note that
   *  expressions with an additional 'tail' (like "end let") may optionally create a
   *  throw-away entry of the tail and join it also to get a more asethically pleasing
   *  measure in the source Document. For the purposes of error-detection and
   *  reporting this is clearly not necessary though.
   * @return SourceReference A new entry that spans both this entry and the one given.
   */
  public SourceReference join(SourceReference otherEnd)
    {
    if (otherEnd == null)
      { return new SourceReference(this, this); }
    else
      { return new SourceReference(this, otherEnd); }
    }
  }
