//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Garrin kimmell@ittc.ukans.edu
//           Joseph josephk@cs.adelaide.edu.au
//
//  $Id: TypeExpression.java,v 1.3 2002/07/16 07:15:48 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: TypeExpression.java,v $
//  Revision 1.3  2002/07/16 07:15:48  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:10:47  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.2  2002/03/12 22:52:02  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:22  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.1.2.3  2002/01/04 12:30:50  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.1.2.2  2001/11/24 06:44:48  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//  Revision 1.1.2.1  2001/11/14 19:17:21  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents the basic <B>TypeExpression</B> object. An instance of this
 *  object indicates that the Item-Type relationship is a "many to many" relationship
 *  rather than a singleton "one to many" relationship provided by default.
 *  Conceptually this class represents an 'edge' in the type hierachy. There is also
 *  a 'singleton' edge for computational use only (ie. it should be used for Type
 *  arithmatic only rather than occur in user code).
 * @author <I> Joseph Kuehn and Garrin Kimmell </I>
 * (kimmell@ittc.ukans.edu, josephk@cs.adelaide.edu.au)
 */
public class TypeExpression extends Expression {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "Relationship", "Value" };


  public static final String typeRelationship = "type";
  public static final String subtypeRelationship = "subtype";
  public static final String setRelationship = "set";
  public static final String arrayRelationship = "array";
  public static final String singletonRelationship = "singleton";

  public static boolean isRelationshipValid(String relationship)
  {
    if (relationship == null)
      { return false; }
    return (typeRelationship.equalsIgnoreCase(relationship) || subtypeRelationship.equalsIgnoreCase(relationship)
	    || setRelationship.equalsIgnoreCase(relationship) || arrayRelationship.equalsIgnoreCase(relationship)
	    || singletonRelationship.equalsIgnoreCase(relationship));
  }



  private String myRelationship;
  private Expression myValue;


  /** The primary Constructor
   * @param relationship The String representing the type of Type relationship that exists.
   * @param value The expression that represents the 'super-type' of this relationship.
   */
  public TypeExpression(String relationship, Expression value)
  {
    myRelationship = relationship;
    myValue = value;
    addChildObject(myValue);
  }

  public TypeExpression() {
    this("NULL", null);
  }

  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public TypeExpression(Map settings)
    {
    super(settings);

    myRelationship = (String)getSetting(TypeExpression.keySet[0]);
    myValue = (Expression)getSetting(TypeExpression.keySet[1]);

    addChildObject(myValue);
    }



  /** Method for retrieving the expression this relationship concerns.
   *  @return Expression The value of the 'super-type'.
   */
  public Expression getExpression()
  { return myValue; }


  /** Method for setting the expression this relationship concerns.
   *  @param value The value of the 'super-type'.
   *  @return void
   */
  public void setExpression(Expression value)
  {
    removeChildObject(myValue);
    myValue = value;
    addChildObject(myValue);
  }



  /** Method for retrieving the type-code this relationship is.
   *  @return String The relationship to the 'super-type'.
   */
  public String getRelationship()
  { return myRelationship; }



  /** Method for setting the type-code of this relationship.
   *  @param relationship The value of the relationship.
   *  @return void
   */
  public void setRelationship(String relationship)
  { myRelationship = relationship; }




  /** To get the string representation of the object
   * @return String representing the string of this object
   */
  public String toString()
  { return myRelationship + "(" + myValue + ")"; }


  /** To get the full source string representation of the object
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the string of this object
   */
  public String publishROM(String indent)
  {
    if (myValue != null)
      { return myRelationship + "(" + myValue.publishROM(indent + defaultPublishROMIndent) + ")"; }
    else
      { return myRelationship; }
  }


}
