//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Joseph josephk@cs.adelaide.edu.au
//
//  $Id: Rational.java,v 1.1.1.1 2002/07/01 15:47:35 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Rational.java,v $
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.1  2002/01/17 00:22:22  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/04 12:40:48  cvs
//  Updated support for the number types Rosetta supports.
//
//
//=============================================================================
package org.sldl.rosetta.lang;

/** This represents a <B>Rational</B> number. A rational
 *  number consists of an Integer numerator and denominator.
 * @author <I>Joseph Kuehn</I> (josephk@cs.adelaide.edu.au)
 */
public final class Rational extends Number
  {
  /** Method to compute the greatest common divisor of two
   *  integers using Euclid's division equation.
   * @param lhs The first number to find gcd with respect to.
   * @param rhs The second number to find gcd with respect to.
   * @return int The gcd of the two given integers (or 1 if lhs = rhs = 0).
   */
  public static int gcd(int lhs, int rhs)
    {
    int q, r, factor;

    if ((lhs == 0) && (rhs == 0))
      { return 1; }

    if (Math.abs(rhs) > Math.abs(lhs))
      { return gcd(rhs, lhs); }

    if (rhs == 0)
      { return lhs; }
    q = lhs / rhs;
    r = lhs - (rhs * q);

    return gcd(rhs, r);
    }


  /** Method to generate a rational that matches a given string.
   * @param image The string to convert to a rational.
   * @throws NumberFormatException If the denominator is zero or the string illegal.
   */
  public static Rational valueOf(String image) throws NumberFormatException
    {
    int slashPos;
    Integer num, denom;

    slashPos = image.indexOf('/');
    num = Integer.valueOf(image.substring(0, slashPos));
    denom = Integer.valueOf(image.substring(slashPos+1, image.length()));
    return new Rational(num.intValue(), denom.intValue());
    }





  private Integer numerator, denominator;

  /** The primary Constructor.
   * @param num The numerator of the rational number.
   * @param den The denominator of the rational number.
   * @throws NumberFormatException If the denominator is zero.
   */
  public Rational(int num, int den) throws NumberFormatException
    {
    numerator = new Integer(num);
    denominator = new Integer(den);
    if (den == 0)
      { throw new NumberFormatException("A rational value cannot have a zero denominator."); }
    }

  /** The secondary Constructor. This is used only when
   *  a literal is forced to be seen as a rational.
   * @param num The numerator of the rational number.
   */
  public Rational(int num)
    { this(num, 1); }


  /** Method to retreive the numerator of the rational.
   * @return Integer The numerator.
   */
  public Integer getNumerator()
    { return numerator; }


  /** Method to retreive the denominator of the rational.
   * @return Integer The denominator.
   */
  public Integer getDenominator()
    { return denominator; }


  /** Method to reduce the rational numerators and denominators into
   *  their coprime form (that is 'simplify' the rational).
   * @return void
   */
  public void setCoprime()
    {
    int fact;
    fact = gcd(numerator.intValue(), denominator.intValue());
    numerator = new Integer(numerator.intValue() / fact);
    denominator = new Integer(denominator.intValue() / fact);
    }


  /** Method to generate a string for the rational.
   * @return String The string representation of the rational.
   */
  public String toString()
    { return (numerator + "/" + denominator); }




  /** Returns the value of the specified number as a byte.
   * @return byte The byte value of this rational.
   */
  public byte byteValue()
    { return new Double(numerator.doubleValue() / denominator.doubleValue()).byteValue(); }


  /** Returns the value of the specified number as a double.
   * @return double The double value of this rational.
   */
  public double doubleValue()
    { return (numerator.doubleValue() / denominator.doubleValue()); }


  /** Returns the value of the specified number as a float.
   * @return float The float value of this rational.
   */
  public float floatValue()
    { return (numerator.floatValue() / denominator.floatValue()); }


  /** Returns the value of the specified number as a int.
   * @return int The int value of this rational.
   */
  public int intValue()
    { return (numerator.intValue() / denominator.intValue()); }


  /** Returns the value of the specified number as a long.
   * @return long The long value of this rational.
   */
  public long longValue()
    { return (numerator.longValue() / denominator.longValue()); }


  /** Returns the value of the specified number as a short.
   * @return short The short value of this rational.
   */
  public short shortValue()
    { return new Double(numerator.doubleValue() / denominator.doubleValue()).shortValue(); }





  /** Method to add two rational numbers together.
   *  If the otherValue is null returns this value (an add of 'zero').
   * @param otherValue The rational to add to this one.
   * @return Rational The new rational that is the result of the addition.
   */
  public Rational add(Rational otherValue)
    {
    int newNum, newDen, fact;

    if (otherValue == null)
      { return this; }

    newDen = this.denominator.intValue() * otherValue.denominator.intValue();
    newNum = (this.numerator.intValue() * otherValue.denominator.intValue())
             + (otherValue.numerator.intValue() * this.denominator.intValue());
    fact = gcd(newNum, newDen);
    return new Rational(newNum/fact, newDen/fact);
    }


  /** Method to subtract a rational number from this one.
   *  If the otherValue is null returns this value (a subtr of 'zero').
   * @param otherValue The rational to subtract from this one.
   * @return Rational The new rational that is the result of the subtraction.
   */
  public Rational subtract(Rational otherValue)
    {
    Rational tempValue;

    if (otherValue == null)
      { return this; }

    tempValue = new Rational((-1) * otherValue.numerator.intValue(), otherValue.denominator.intValue());
    return this.add(tempValue);
    }


  /** Method to multiply a rational number with this one.
   *  If the otherValue is null returns 0/1 (a mult by 'zero').
   * @param otherValue The rational to multiply with this one.
   * @return Rational The new rational that is the result of the multiplication.
   */
  public Rational multiply(Rational otherValue)
    {
    int newNum, newDen, fact;

    if (otherValue == null)
      { return new Rational(0); }

    newDen = this.denominator.intValue() * otherValue.denominator.intValue();
    newNum = otherValue.numerator.intValue() * this.numerator.intValue();
    fact = gcd(newNum, newDen);
    return new Rational(newNum/fact, newDen/fact);
    }


  /** Method to divide a rational number from this one.
   *  If the otherValue is null throws exception (a div by 'zero').
   * @param otherValue The rational to divide from this one.
   * @return Rational The new rational that is the result of the division.
   * @throws NumberFormatException If the otherValue equals zero.
   */
  public Rational divide(Rational otherValue) throws NumberFormatException
    {
    Rational tempValue;

    if ((otherValue == null) || (otherValue.numerator.intValue() == 0))
      { throw new NumberFormatException("Attempt to divide a rational by zero."); }

    tempValue = new Rational(otherValue.denominator.intValue(), otherValue.numerator.intValue());
    return this.multiply(tempValue);
    }
  }
