//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: RadixNumber.java,v 1.1 2002/07/15 04:33:08 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: RadixNumber.java,v $
//  Revision 1.1  2002/07/15 04:33:08  cvs
//  Added the language-defined number implementations to the lang.logic package.
//
//
//=============================================================================
package org.sldl.rosetta.lang.logic;

/** This represents a <B>RadixNumber</B> number. A radix-number
 *  consists of a string of digits with an optional single decimal
 *  point in it somewhere, an integer radix that defines
 *  the interpretation of those digits and an integer exponent that
 *  allows for a shift of the decimal point.
 */
public final class RadixNumber extends Number
  {
  public static final String formatString = "<base_integer> '\\' <based_digits> ('.' <based_digits>)? '\\' (('e' | 'E') ('+' | '-')? <exponent_int>)?";

  /** Method to generate a radix-number that matches a given string.
   * @param image The string to convert to a radix-number.
   * @throws NumberFormatException If the string is illegal.
   */
  public static RadixNumber valueOf(String image) throws NumberFormatException
    {
    int firstPos, secondPos, pos, decimalCount;
    Integer base, exponent;
    String digits;

    firstPos = image.indexOf('\\');
    if ((firstPos <= 0) || (firstPos >= image.length()-1))
      { throw new NumberFormatException("Radix number must be represented by a string of the form: " + formatString); }
    secondPos = image.indexOf('\\', firstPos+1);
    if ((secondPos <= firstPos+1) || (secondPos >= image.length()))
      { throw new NumberFormatException("Radix number must be represented by a string of the form: " + formatString); }
    base = Integer.valueOf(image.substring(0, firstPos));
    digits = image.substring(firstPos+1, secondPos);
    if (secondPos < image.length() - 1)
      {
      if ((image.charAt(secondPos+1) != 'e') && (image.charAt(secondPos+1) != 'E'))
        { throw new NumberFormatException("Radix number must be represented by a string of the form: " + formatString); }
      exponent = Integer.valueOf(image.substring(secondPos+2, image.length()));
      }
    else
      { exponent = new Integer(0); }
    return new RadixNumber(base.intValue(), digits, exponent.intValue());
    }





  private int radix, exponent;
  private String digitString;

  /** The primary Constructor.
   * @param rad The radix of the digits.
   * @param digits The string that represents the number.
   * @param exp The exponent of the number.
   * @throws NumberFormatException If the radix is less than 2 or the
   *                               digits have more than one decimal point.
   */
  public RadixNumber(int rad, String digits, int exp) throws NumberFormatException
    {
    int deciPos;

    radix = rad;
    if (radix < 2)
      { throw new NumberFormatException("A radix of less than 2 cannot form a legal number."); }
    digitString = digits;
    deciPos = digitString.indexOf('.');
    if (deciPos != -1)
      {
      if (deciPos != digitString.substring(deciPos+1, digitString.length()).indexOf('.'))
        { throw new NumberFormatException("A radix-number cannot have more than one decimal point."); }
      }
    exponent = exp;
    }


  /** The secondary Constructor (assumes exponent = 0).
   * @param rad The radix of the digits.
   * @param digits The string that represents the number.
   * @throws NumberFormatException If the radix is less than 2 or the
   *                               digits have more than one decimal point.
   */
  public RadixNumber(int rad, String digits) throws NumberFormatException
    { this(rad, digits, 0); }


  /** Method to retreive the radix of this number.
   * @return int The radix of the string.
   */
  public int getRadix()
    { return radix; }

  /** Method to retreive the exponent of this number.
   * @return int The exponent of the string.
   */
  public int getExponent()
    { return exponent; }

  /** Method to retreive the digits of this number.
   * @return String The digits of the string.
   */
  public String getDigits()
    { return digitString; }




  /** Method to generate a string for the rational.
   * @return String The string representation of the rational.
   */
  public String toString()
    {
    if (exponent != 0)
      { return (radix + "\\" + digitString + "\\e" + exponent); }
    else
      { return (radix + "\\" + digitString + "\\"); }
    }




  /** Returns the value of the specified number as a byte.
   * @return byte The byte value of this rational.
   */
  public byte byteValue()
    { return (new Double(doubleValue())).byteValue(); }


  /** Returns the value of the specified number as a double.
   * @return double The double value of this rational.
   */
  public double doubleValue()
    {
    int pos, deciPoint, factor;
    double result;

    deciPoint = digitString.indexOf(".");
    if (deciPoint == -1)
      { deciPoint = digitString.length(); }
    result = 0.0;
    factor = 1;
    for (pos = 0; pos < deciPoint; pos++)
      {
      result = (result * factor) + Character.digit(digitString.charAt(pos), radix);
      factor *= radix;
      }
    factor = radix;
    for (pos = deciPoint+1; pos < digitString.length(); pos++)
      {
      result += Character.digit(digitString.charAt(pos), radix) / factor;
      factor *= radix;
      }

    for (pos = 0; pos < exponent; pos++)
      { result = result * radix; }
    for (pos = 0; pos > exponent; pos--)
      { result = result / radix; }

    return result;
    }


  /** Returns the value of the specified number as a float.
   * @return float The float value of this rational.
   */
  public float floatValue()
    { return (new Double(doubleValue())).floatValue(); }


  /** Returns the value of the specified number as a int.
   * @return int The int value of this rational.
   */
  public int intValue()
    { return (new Double(doubleValue())).intValue(); }


  /** Returns the value of the specified number as a long.
   * @return long The long value of this rational.
   */
  public long longValue()
    { return (new Double(doubleValue())).longValue(); }


  /** Returns the value of the specified number as a short.
   * @return short The short value of this rational.
   */
  public short shortValue()
    { return (new Double(doubleValue())).shortValue(); }


  }
