//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: RationalNumber.java,v 1.1 2002/07/15 04:33:08 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: RationalNumber.java,v $
//  Revision 1.1  2002/07/15 04:33:08  cvs
//  Added the language-defined number implementations to the lang.logic package.
//
//
//=============================================================================
package org.sldl.rosetta.lang.logic;

/** This represents a <B>RationalNumber</B> number. A rational
 *  number consists of an Integer numerator and denominator.
 */
public final class RationalNumber extends Number
  {
  /** Method to compute the greatest common divisor of two
   *  integers using Euclid's division equation.
   * @param lhs The first number to find gcd with respect to.
   * @param rhs The second number to find gcd with respect to.
   * @return int The gcd of the two given integers (or 1 if lhs = rhs = 0).
   */
  public static int gcd(int lhs, int rhs)
    {
    int q, r, factor;

    if ((lhs == 0) && (rhs == 0))
      { return 1; }

    if (Math.abs(rhs) > Math.abs(lhs))
      { return gcd(rhs, lhs); }

    if (rhs == 0)
      { return lhs; }
    q = lhs / rhs;
    r = lhs - (rhs * q);

    return gcd(rhs, r);
    }


  /** Method to generate a rational that matches a given string.
   * @param image The string to convert to a rational.
   * @return RationalNumber The parsed result of the string.
   * @throws NumberFormatException If the denominator is zero or the string illegal.
   */
  public static RationalNumber valueOf(String image) throws NumberFormatException
    {
    int slashPos;
    Integer num, denom;

    slashPos = image.indexOf('/');
    if ((slashPos <= 0) || (slashPos >= image.length()-1))
      { throw new NumberFormatException("Rational number must be represented by a forward slash with an integer on both sides."); }
    num = Integer.valueOf(image.substring(0, slashPos));
    denom = Integer.valueOf(image.substring(slashPos+1, image.length()));
    return new RationalNumber(num, denom);
    }






  private Integer numerator, denominator;

  /** The primary Constructor.
   * @param num The numerator of the rational number.
   * @param den The denominator of the rational number.
   * @throws NumberFormatException If the denominator is zero or null.
   */
  public RationalNumber(Integer num, Integer den) throws NumberFormatException
    {
    numerator = num;
    denominator = den;
    if ((denominator == null) || (denominator.intValue() == 0))
      { throw new NumberFormatException("A rational value cannot have a zero denominator."); }
    }

  /** The secondary Constructor.
   * @param num The numerator of the rational number.
   * @param den The denominator of the rational number.
   * @throws NumberFormatException If the denominator is zero.
   */
  public RationalNumber(int num, int den) throws NumberFormatException
    { this(new Integer(num), new Integer(den)); }

  /** The tertiary Constructor. This is used only when
   *  a literal is forced to be seen as a rational.
   * @param num The numerator of the rational number.
   */
  public RationalNumber(int num)
    { this(new Integer(num), new Integer(1)); }


  /** Method to retreive the numerator of the rational.
   * @return Integer The numerator.
   */
  public Integer getNumerator()
    { return numerator; }


  /** Method to retreive the denominator of the rational.
   * @return Integer The denominator.
   */
  public Integer getDenominator()
    { return denominator; }


  /** Method to reduce the rational numerators and denominators into
   *  their coprime form (that is 'simplify' the rational).
   * @return void
   */
  public void setCoprime()
    {
    int fact;
    fact = gcd(numerator.intValue(), denominator.intValue());
    numerator = new Integer(numerator.intValue() / fact);
    denominator = new Integer(denominator.intValue() / fact);
    }


  /** Method to generate a string for the rational.
   * @return String The string representation of the rational.
   */
  public String toString()
    { return (numerator + "/" + denominator); }




  /** Returns the value of the specified number as a byte.
   * @return byte The byte value of this rational.
   */
  public byte byteValue()
    { return new Double(numerator.doubleValue() / denominator.doubleValue()).byteValue(); }


  /** Returns the value of the specified number as a double.
   * @return double The double value of this rational.
   */
  public double doubleValue()
    { return (numerator.doubleValue() / denominator.doubleValue()); }


  /** Returns the value of the specified number as a float.
   * @return float The float value of this rational.
   */
  public float floatValue()
    { return (numerator.floatValue() / denominator.floatValue()); }


  /** Returns the value of the specified number as a int.
   * @return int The int value of this rational.
   */
  public int intValue()
    { return (numerator.intValue() / denominator.intValue()); }


  /** Returns the value of the specified number as a long.
   * @return long The long value of this rational.
   */
  public long longValue()
    { return (numerator.longValue() / denominator.longValue()); }


  /** Returns the value of the specified number as a short.
   * @return short The short value of this rational.
   */
  public short shortValue()
    { return new Double(numerator.doubleValue() / denominator.doubleValue()).shortValue(); }

  }
