//============================================================================
//  Copyright (c) 2002 University of Adelaide
//  All rights reserved.
//
//  AU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. AU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: ROMWalker.java,v 1.1.1.1 2002/07/01 15:47:35 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: ROMWalker.java,v $
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.1  2002/02/15 05:47:41  cvs
//  Added an NLR (node, left, right) walker to the CVS with an example
//  SymbolTable builder to demonstrate how it might be used.
//
//
//=============================================================================
package org.sldl.rosetta.parser;

import org.sldl.rosetta.ROM.BaseObject;

/** This is the <B>ROMWalker</B> class which provides an easy method for
 *  quickly visiting a specific subset of nodes within a ROM. In particular,
 *  when we only wish to perform an operation on a small subset of nodes and
 *  when this operation does not require much additional state information to be
 *  constructed during traversal (other than the nodes being visited) then this
 *  class is ideal.
 *  The traversal method is <B><U>N-L-R</U></B>.
 */
public class ROMWalker
  {
  /** Basic interface required for the class which is to perform the
   *  operations on the ROM tree.
   */
  public interface NodeVisitor
    {
    /** Method for obtaining the initial state of the walker.
     *  While the root of the ROM tree is passed in, this should _not_
     *  be thought of as the same as 'visiting' the first node, since
     *  if it is appropriate it will be visited with the next method
     *  once this method returns.
     * @param root The root of the ROM tree being walked.
     * @return Object The initial state of the walker.
     */
    public Object getInitialState(BaseObject root);


    /** Method for operating on this specific node.
     * @param node The current BaseObject node being examined.
     * @param previousState The previous state of the walker.
     * @return Object The new state of the walker.
     */
    public Object visitROMNode(BaseObject node, Object previousState);

    }



  /** A more advanced interface allows a user class to specify a
   *  small subset of classes which it is interested in visiting.
   *  This interface will cause the <B>Walker</B> to call the
   *  <I>visitROMNode</I> method only on those ROM nodes which
   *  are instanceof one of the Classes provided.
   */
  public interface AdvancedNodeVisitor extends NodeVisitor
    {
    /** Method for specifying that only some of the nodes should
     *  be visited by the walker. If the result is <I>null</I> then
     *  all classes will be visited, if the result is <I>new Class[0]</I>
     *  then no classes will be visited.
     * @return Class[] The classes of the nodes that should be visited.
     */
    public Class[] getWantedNodes();
    }





  private Class[] wantedNodes;
  private NodeVisitor myVisitor;

  /** Primary Constructor.
   * @param visitor The user class which wishes to operate on the ROM tree.
   */
  public ROMWalker(NodeVisitor visitor)
    {
    if (visitor instanceof AdvancedNodeVisitor)
      { wantedNodes = ((AdvancedNodeVisitor)visitor).getWantedNodes(); }
    else
      { wantedNodes = null; }
    myVisitor = visitor;
    }




  /** Method for checking if a node should be visited.
   * @param node The node currently being traversed.
   * @return boolean Should we visit this node?
   */
  private boolean isWanted(BaseObject node)
    {
    int pos;

    if (wantedNodes == null)
      { return true; }
    for (pos = 0; pos < wantedNodes.length; pos++)
      {
      if ((wantedNodes[pos] != null) && wantedNodes[pos].isInstance(node))
        { return true; }
      }
    return false;
    }


  /** Method for walking the ROM tree.
   *  The traversal method is <B><U>N-L-R</U></B>.
   * @param root The root of the ROM tree being walked.
   * @return Object The final state of the walker after traversing the ROM tree.
   */
  public Object walk(BaseObject root)
    { return walk(root, myVisitor.getInitialState(root)); }


  /** Sub-method for walking the ROM tree.
   *  The traversal method is <B><U>N-L-R</U></B>.
   * @param root The root of the ROM tree being walked.
   * @param previousState The previous state of the walker.
   * @return Object The new state of the walker after traversal of this sub-tree.
   */
  private Object walk(BaseObject root, Object previousState)
    {
    int pos;
    Object latestState;

    latestState = previousState;
    if (root == null)
      { return latestState; }

    if (isWanted(root))
      { latestState = myVisitor.visitROMNode(root, latestState); }
    for (pos = 0; pos < root.getChildCount(); pos++)
      { latestState = walk(root.getChildObjectAt(pos), latestState); }

    return latestState;
    }


  }

