//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//      authors: Garrin kimmell@ittc.ku.edu
//               Joseph josephk@cs.adelaide.edu.au
//
//  $Id: ROMLinker.java,v 1.1.1.1 2002/07/01 15:47:35 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: ROMLinker.java,v $
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.1  2002/01/17 00:22:24  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.1.2.2  2002/01/04 13:05:07  cvs
//  Updated the old 'AggregateReferences' to 'QuantifiedReferences'.
//
//  Revision 1.1.2.1  2002/01/04 12:28:05  cvs
//  Prototype straight-to-ROM parser.
//
//
//=============================================================================
package org.sldl.rosetta.parser.linker;

import org.sldl.rosetta.ROM.ItemList;
import org.sldl.rosetta.ROM.ExpressionList;
import org.sldl.rosetta.ROM.BaseObject;
import org.sldl.rosetta.ROM.Label;
import org.sldl.rosetta.ROM.Item;
import org.sldl.rosetta.ROM.DeferredItem;
import org.sldl.rosetta.ROM.Expression;
import org.sldl.rosetta.ROM.Application;
import org.sldl.rosetta.ROM.ItemReference;
import org.sldl.rosetta.ROM.EnumerationType;
import org.sldl.rosetta.ROM.ConstructedType;
import org.sldl.rosetta.ROM.ConstructedType.ConstructedTypeConstructor;
import org.sldl.rosetta.ROM.Facet;
import org.sldl.rosetta.ROM.Function;
import org.sldl.rosetta.ROM.QuantifierReference;
import org.sldl.rosetta.ROM.LetFunction;
import org.sldl.rosetta.ROM.ParameterListValue;


/** This represents the <B>ROMLinker</B> object.
 *  The purpose of this class is to perform the dereferencing needed
 *  to correctly link a ROM and more importantly to do it in such
 *  a way that the scoping rules of Rosetta linking are observed.
 *  NB: This file is much easier to read from the javadoc comments. ;)
 * @author <I> Garrin Kimmell, Joseph Kuehn </I>
 * (kimmell@ittc.ku.edu, josephk@cs.adelaide.edu.au)
 */
public class ROMLinker
  {
  public static final Label MissingItemsLabel = new Label("___Missing_Symbol_Items___");
  public static final Label WildCardLabel = new Label("_");



  /** This interface decouples the mechanism by which Rosetta models are stored
   *  from the dereferencing process. In particular a naive implementation of this
   *  interface could simply call the parser again on the new file.
   */
  public static interface LinkerLoader
    {
    /** The standard library to load. This library should contain the base domains.
     * @param focusItem The Item we want the library for to help avoid cycles.
     * @return Item This ROM should be the default 'rosetta.lang'-style object.
     */
    public Item getDefaultLibrary(Item focusItem);


    /** Method for accessing additional libraries triggered by the "use" clause.
     * @param delayedRef The delayed reference of the package that needs to be loaded.
     * @param params The parameter list to be used on the package - should be null or static.
     * @return Item The ROM of the corresponding package.
     */
    public Item loadPackage(DeferredItem delayedRef, ExpressionList params);


    /** Method for outputting errors encountered during linking.
     * @param errorMessage A string describing the error encountered.
     */
    public void linkingError(String errorMessage);
    }



  /** Standard stuff for what is needed for linking models.
   */
  public static interface ROMDereferencer
    {
    /** The sub-method for actually resolving delayed references in the ROM.
     * @param mySoftLink The soft-link that needs to be resolved to an Item.
     * @param root The scope where the reference actually occurred.
     * @param fileLoader The loader to use should external references occur.
     * @return Item The Item found to match the soft-link. If no match found, returns mySoftLink.
     */
    public Item dereferenceItem(DeferredItem mySoftLink, BaseObject root, LinkerLoader fileLoader);
    }
  public static class DefaultROMDereferencer implements ROMDereferencer
    {
    public Item dereferenceItem(DeferredItem mySoftLink, BaseObject root, LinkerLoader fileLoader)
      { return ROMLinker.dereferenceItem(mySoftLink, root, fileLoader); }
    }





  /** Method to generate an import signature tree for a library.
   * @param libItem The item to scan for exportable Items.
   * @return DeferredItem The root item.
   */
  public static DeferredItem generateImports(Item libItem)
    {
    DeferredItem root;
    root = generateImports(libItem, null);
    relinkImports(root);
    return root;
    }

  private static void relinkImports(BaseObject root)
    {
    int pos;
    for (pos = 0; pos < root.getChildCount(); pos++)
      { relinkImports(root.getChildObjectAt(pos)); }
    if (root instanceof ItemReference)
      {
      ItemReference myRef;
      myRef = (ItemReference)root;
      if (myRef.getItemReferred() instanceof DeferredItem)
        { return; }
      myRef.setItemReferred(findImport(myRef.getItemReferred().getLabel(), root));
      }
    }

  private static DeferredItem findImport(Label name, BaseObject startPos)
    {
    if (startPos == null)
      { return null; }
    if (startPos instanceof DeferredItem)
      {
      int pos;
      DeferredItem nextItem;
      nextItem = (DeferredItem)startPos;
      if (nextItem.getLabel().matches(name))
        { return nextItem; }
      for (pos = 0; pos < nextItem.getImportList().size(); pos++)
        {
        if (nextItem.getImportList().get(pos).getLabel().matches(name))
          { return (DeferredItem)(nextItem.getImportList().get(pos)); }
        }
      }
    return findImport(name, startPos.getParentObject());
    }



  /** Method to generate an import signature tree for a library.
   * @param libItem The item to scan for exportable Items.
   * @param parentItem The parent item to add cross-scope Items too (eg. enumerations).
   * @return DeferredItem The root item.
   */
  private static DeferredItem generateImports(Item libItem, DeferredItem parentItem)
    {
    DeferredItem answer, parent;

    if (libItem == null)
      { return null; }
    
    answer = new DeferredItem(libItem.getLabel(), libItem.getType());
    answer.setSourceReference(libItem.getSourceReference());
    if (libItem.getValue() == null)
      { return answer; }
    parent = parentItem;
    if (parentItem == null)
      { parent = answer; }

    if (libItem.getValue() instanceof Facet)
      {
      int pos;
      Facet myFacet;
      myFacet = (Facet)(libItem.getValue());
      if (myFacet.getExportItems() != null)
        {
        for (pos = 0; pos < myFacet.getExportItems().size(); pos++)
          { answer.addImportItem(generateImports(myFacet.getExportItems().get(pos), answer)); }
        }
      }
    else if (libItem.getValue() instanceof EnumerationType)
      {
      int pos;
      EnumerationType myEnum;
      Item nextItem;
      myEnum = (EnumerationType)(libItem.getValue());
      for (pos = 0; pos < myEnum.getElements().size(); pos++)
        {
        try
          {
          nextItem = ((ItemReference)(myEnum.getElementAt(pos))).getItemReferred();
          parent.addImportItem(generateImports(nextItem, parent));
          }
        catch(ClassCastException e)
          { }
        }
      }
    else if (libItem.getValue() instanceof ConstructedType)
      {
      int pos, obsv;
      Item nextItem;
      ConstructedType myConstr;
      ConstructedTypeConstructor nextConstructor;
      myConstr = (ConstructedType)(libItem.getValue());
      for (pos = 0; pos < myConstr.getConstructors().size(); pos++)
        {
        nextItem = myConstr.getConstructors().get(pos);
        parent.addImportItem(generateImports(nextItem, parent));
        nextConstructor = (ConstructedTypeConstructor)(nextItem.getValue());
        parent.addImportItem(generateImports(nextConstructor.getRecognizer(), parent));
        for (obsv = 0; obsv < nextConstructor.getObservers().size(); obsv++)
          { parent.addImportItem(generateImports(nextConstructor.getObservers().get(obsv), parent)); }
        }
      }     

    return answer;
    }










  /** The primary method.
   * @param rootROM The rosetta model to be linked.
   * @param fileLoader The object which can provide other packages used by this model.
   * @return void
   */
  public static void link(Item rootROM, LinkerLoader fileLoader)
    { linkROM(rootROM, fileLoader); }

  /** The second-pass method.
   * @param root The current rosetta model to be linked.
   * @param fileLoader The linker to use should external references occur.
   * @return void
   */
  public static void linkROM(BaseObject root, LinkerLoader fileLoader)
    {
    int pos;

    if ((root == null) || (fileLoader == null))
      { return; }

    if ((root instanceof Item) && (((Item)root).getValue() instanceof Facet) && (root.getParentObject() == null))
      {
      Facet rootFacet;
      Item defaultLibrary;
      rootFacet = (Facet)(((Item)root).getValue());
      defaultLibrary = null;
      if ((rootFacet.getUseClauses() != null) && !rootFacet.getUseClauses().isEmpty())
        {
        for (pos = 0; pos < rootFacet.getUseClauses().size(); pos++)
          {
          if ((rootFacet.getUseClauses().get(pos) instanceof ItemReference)
             && (((ItemReference)(rootFacet.getUseClauses().get(pos))).getItemReferred() != null)
             && (((ItemReference)(rootFacet.getUseClauses().get(pos))).getItemReferred().getLabel() != null)
             && (((ItemReference)(rootFacet.getUseClauses().get(pos))).getItemReferred().getLabel().matches(new Label("lang")))
             )
            { break; }
          }
        if (pos == rootFacet.getUseClauses().size())
          { defaultLibrary = fileLoader.getDefaultLibrary((Item)root); }
        }
      else
        { defaultLibrary = fileLoader.getDefaultLibrary((Item)root); }
      if (defaultLibrary != null)
        {
        DeferredItem defaultImport;
        defaultImport = generateImports(defaultLibrary);
        rootFacet.addUseClause(new ItemReference(defaultImport));
        }
      }
    else if (root instanceof Facet)
      {
      // Order is important here.
      linkUseClauses((Facet)root, fileLoader);
      linkDomain((Facet)root, fileLoader);
      resolveExports((Facet)root, fileLoader); // must go after Use Clause & Domain linking!
      }
    else if ((root instanceof ItemReference) && (((ItemReference)root).getItemReferred() instanceof DeferredItem))
      {
      ItemReference myRef;
      DeferredItem delayedRef;
      myRef = (ItemReference)root;
      delayedRef = (DeferredItem)(myRef.getItemReferred());
      if (delayedRef.isDelayedReference())
        { myRef.setItemReferred(dereferenceItem(delayedRef, root, fileLoader)); }
      }

    for (pos = 0; pos < root.getChildCount(); pos++)
      { linkROM(root.getChildObjectAt(pos), fileLoader); }
    }

  /** The sub-method for fixing use clauses. This requires using the LinkerLoader
   *  to resolve a DeferredItem reference to some external library with an optional
   *  static parameter list.
   * @param myFacet The facet whose use clauses need to be linked.
   * @param fileLoader The loader to use should external references occur.
   * @return void
   */
  private static void linkUseClauses(Facet myFacet, LinkerLoader fileLoader)
    {
    int pos, paramPos;
    Expression nextUseClause;
    ExpressionList paramList;
    Item packageItem;
    DeferredItem importRef, importImage;

    if ((myFacet == null) || (fileLoader == null) || (myFacet.getUseClauses() == null))
      { return; }

    for (pos = 0; pos < myFacet.getUseClauses().size(); pos++)
      {
      try
        {
        nextUseClause = myFacet.getUseClauses().get(pos);
        paramList = null;
        if (nextUseClause instanceof Application)
          {
          paramList = ((ParameterListValue)(((Application)nextUseClause).getRightOp())).getElements();
          nextUseClause = ((Application)nextUseClause).getLeftOp();
          for (paramPos = 0; paramPos < paramList.size(); paramPos++)
            { linkROM(paramList.get(paramPos), fileLoader); }
          }
        importRef = (DeferredItem)(((ItemReference)nextUseClause).getItemReferred());
        if (!importRef.isDelayedReference())
          { continue; }
        packageItem = fileLoader.loadPackage(importRef, paramList);
        importImage = generateImports(packageItem);
        if (importImage != null)
          { ((ItemReference)nextUseClause).setItemReferred(importImage); }
        }
      catch(Exception e)
        { fileLoader.linkingError("Could not resolve use clause #" + pos + " [" + myFacet.getUseClauses().get(pos).publishROM() + "] for Facet [" + myFacet + "] because of: " + e); }
      } // end for loop
    }


  /** The sub-method for fixing domain references. This is tricky as domain
   *  algebra can envolve partial evaluation rather than just linking.
   * @param myFacet The facet whose domain needs to be linked.
   * @param fileLoader The loader to use should external references occur.
   * @return void
   */
  private static void linkDomain(Facet myFacet, LinkerLoader fileLoader)
    {
    Expression myDomain;

/////// *  Needs a lot of thought & work and possibly a LinkerLoader callback for evaluation.

    myDomain = myFacet.getDomain();
    if (myDomain == null)
      { return; }

    linkROM(myDomain, fileLoader);

    if (myDomain instanceof ItemReference)
      {
      // Nothing needed here?
      }
    else
      { fileLoader.linkingError("Could not decode domain [" + myDomain + "] for entry [" + myFacet + "]."); }
    }

  /** The sub-method for fixing domain export lists.
   * @param myFacet The facet whose export list needs to be linked.
   * @param fileLoader The loader to use should external references occur.
   * @return void
   */
  public static final Label AllLabel = new Label("ALL");
  private static void resolveExports(Facet myFacet, LinkerLoader fileLoader)
    {
    int pos, declPos;
    DeferredItem nextItem;
    ItemList oldExportList, fixedExportList, searchList;

    if ((myFacet == null) || (fileLoader == null))
      { return; }
    oldExportList = myFacet.getExportItems();
    if ((oldExportList == null) || (oldExportList.size() == 0))
      { return; }

    final String searchListString = "declared/term/interface items";
    searchList = new ItemList();
    searchList.add(myFacet.getDeclaredItems());
    searchList.add(myFacet.getTermItems());
    searchList.add(myFacet.getInterfaceItems());

//    searchList.add(myFacet.getUseClauses().getItems());
//    searchList.add(myFacet.getDomain().getItems());

    for (pos = 0; pos < oldExportList.size(); pos++)
      {
      if (oldExportList.get(pos).getLabel().matches(AllLabel))
        {
        myFacet.setExportAll(true);
        myFacet.setExportItems(searchList);
        return;
        }
      }

    fixedExportList = new ItemList();
    for (pos = 0; pos < oldExportList.size(); pos++)
      {
      if (  !(oldExportList.get(pos) instanceof DeferredItem)
         || !(((DeferredItem)(oldExportList.get(pos))).isDelayedReference()))
        {
        fixedExportList.add(oldExportList.get(pos));
        continue;
        }

      nextItem = (DeferredItem)(oldExportList.get(pos));
      for (declPos = 0; declPos < searchList.size(); declPos++)
        {
        if (nextItem.getLabel().matches(searchList.get(declPos).getLabel()))
          {
          fixedExportList.add(searchList.get(declPos));
          break;
          }
        }
      if (declPos == searchList.size())
        {
        fileLoader.linkingError("Export item #" + pos + " [" + nextItem + "] for [" + myFacet + "] not found in " + searchListString + ".");
        fixedExportList.add(nextItem);
        }

      } // end for
    myFacet.setExportItems(fixedExportList);
    }














  /** The sub-method for actually resolving delayed references in the ROM.
   * @param mySoftLink The soft-link that needs to be resolved to an Item.
   * @param root The scope where the reference actually occurred.
   * @param fileLoader The loader to use should external references occur.
   * @return Item The Item found to match the soft-link. If no match found, returns mySoftLink.
   */
  public static Item dereferenceItem(DeferredItem mySoftLink, BaseObject root, LinkerLoader fileLoader)
    {
    Item result;
    ExpressionList facetList;
    int domainPos;
    BaseObject packageRoot;
    Facet packageFacet;
    ItemReference missingItemsRef;
    DeferredItem missingItems, dummyItem;

    if ((mySoftLink == null) || (root == null) || (fileLoader == null) || !mySoftLink.isDelayedReference())
      { return mySoftLink; }

    if (mySoftLink.getLabel() == null)
      {
      fileLoader.linkingError("NULL-labelleld item [" + mySoftLink + "] given for dereferencing at [" +  root.getDotString() + "]");
      return mySoftLink;
      }
    if (mySoftLink.getLabel().matches(WildCardLabel))
      { return mySoftLink; }

//// Do stuff with dot notation here - chop it up and do a search for the first piece, then
//// recurse with that as root until all the pieces are finished or we hit an instance
//// variable. ;)
    facetList = new ExpressionList();
    result = dereferenceItemRECURSIVE(mySoftLink, root, root, facetList, fileLoader, root);

    // Check to see if the static search was successful.
    if ((result != null) && (result != mySoftLink))
      {
      if (result instanceof DeferredItem)
        { ((DeferredItem)result).addReference(mySoftLink); }
      return result;
      }

    // No match in this file's scope => check Facet domains.
    for (domainPos = 0; domainPos < facetList.size(); domainPos++)
      {
      result = checkDomainForItem((Facet)(facetList.get(domainPos)), mySoftLink, fileLoader);
      if ((result != null) && (result != mySoftLink))
        { return result; }
      }

    // No match == badly linked item (and we must be at the package Item to know this).
    fileLoader.linkingError("Linking failed for [" + mySoftLink.getFullName() + "] at [" +  root.getDotString() + "]");
    for (packageRoot = root; packageRoot.getParentObject() != null; packageRoot = packageRoot.getParentObject())
      { ; }
    try
      {
      packageFacet = (Facet)(((Item)packageRoot).getValue());
      missingItemsRef = (ItemReference)(packageFacet.getUseClause(MissingItemsLabel.referenceString()));
      if (missingItemsRef == null)
        {
        missingItems = new DeferredItem(MissingItemsLabel.duplicateLabel(), null);
        missingItemsRef = new ItemReference(missingItems);
        packageFacet.addUseClause(missingItemsRef);
        }
      else
        { missingItems = (DeferredItem)(missingItemsRef.getItemReferred()); }
      dummyItem = new DeferredItem(mySoftLink.getLabel(), null);
      missingItems.addImportItem(dummyItem);
      return dummyItem;
      }
    catch(ClassCastException e)
      { fileLoader.linkingError("Detected Package-less structure! [" + mySoftLink.getFullName() + "] at [" +  root.getDotString() + "] @ [" + root.getSourceReference() + "]"); }
    return mySoftLink;
    }

  /** The sub-method for actually resolving DeferredItem references in the ROM. This is a very
   *  complicated piece of code because of the freedom Rosetta provides to recursively mix
   *  expressions and types. Combined with special scoping rules (some blocks have scope,
   *  some blocks have forward-only scope, and Facets have FOUR seperately scoped blocks ;).
   *  This means that each time we recurse up a level we need to determine which scope block
   *  we came from to determine what items at the current scope can be seen.
   *  PLEASE Don't alter play with this function without knowing about Rosetta scope! ;)
   * @param mySoftLink The soft-link that needs to be resolved to an Item.
   * @param scope The current scope to look for reference matches.
   * @param lastScope The last object that was looked at for reference matches.
   * @param facetList The facets encountered during dereferencing - because Domains have lowest priority (last).
   * @param fileLoader The loader to use should external references occur.
   * @param firstScope The scope where the reference originally occurred.
   * @return Item The Item found to match the soft-link. If no match found, returns mySoftLink.
   */
  private static Item dereferenceItemRECURSIVE(DeferredItem mySoftLink, BaseObject scope, BaseObject lastScope, ExpressionList facetList, LinkerLoader fileLoader, BaseObject firstScope)
    {
    Label target;

    if ((mySoftLink == null) || (scope == null) || (lastScope == null) || (facetList == null) || (fileLoader == null))
      { return mySoftLink; }

    target = mySoftLink.getLabel();
    if (target == null)
      { return mySoftLink; }

    if (scope instanceof Function)
      {
      Function myFunction;
      int posLimit, pos;
      myFunction = (Function)scope;
      if (myFunction.getInterfaceItems() == null)
        { return dereferenceItemRECURSIVE(mySoftLink, scope.getParentObject(), scope, facetList, fileLoader, firstScope); }
      for (posLimit = 0; posLimit < myFunction.getInterfaceItems().size(); posLimit++)
        {
        if (lastScope == myFunction.getInterfaceItems().get(posLimit))
          { break; }
        }
      for (pos = 0; pos < posLimit; pos++)
        {
        if (target.matches(myFunction.getInterfaceItems().get(pos).getLabel()))
          { return myFunction.getInterfaceItems().get(pos); }
        }
      }
    else if (scope instanceof QuantifierReference)
      {
      QuantifierReference myFunction;
      int posLimit, pos;
      myFunction = (QuantifierReference)scope;
      if (myFunction.getVariables() == null)
        { return dereferenceItemRECURSIVE(mySoftLink, scope.getParentObject(), scope, facetList, fileLoader, firstScope); }
      for (posLimit = 0; posLimit < myFunction.getVariables().size(); posLimit++)
        {
        if (lastScope == myFunction.getVariables().get(posLimit))
          { break; }
        }
      for (pos = 0; pos < posLimit; pos++)
        {
        if (target.matches(myFunction.getVariables().get(pos).getLabel()))
          { return myFunction.getVariables().get(pos); }
        }
      }
    else if (scope instanceof LetFunction)
      {
      LetFunction myFunction;
      int posLimit, pos;
      myFunction = (LetFunction)scope;
      if (myFunction.getItemList() == null)
        { return dereferenceItemRECURSIVE(mySoftLink, scope.getParentObject(), scope, facetList, fileLoader, firstScope); }
      for (posLimit = 0; posLimit < myFunction.getItemList().size(); posLimit++)
        {
        if (lastScope == myFunction.getItemList().get(posLimit))
          { break; }
        }
      for (pos = 0; pos < posLimit; pos++)
        {
        if (target.matches(myFunction.getItemList().get(pos).getLabel()))
          { return myFunction.getItemList().get(pos); }
        }
      }
    else if (scope instanceof Facet)
      {
      int pos, posLimit;
      Facet myFacet;
      Item nextItem;
      ItemList searchList;

      myFacet = (Facet)scope;

      if (myFacet.getUseClauses() != null)
        {
        for (pos = 0; pos < myFacet.getUseClauses().size(); pos++)
          {
          if (lastScope == myFacet.getUseClauses().get(pos))
            { return dereferenceItemRECURSIVE(mySoftLink, scope.getParentObject(), scope, facetList, fileLoader, firstScope); }
          }
        }

      if (lastScope == myFacet.getDomain())
        {
        nextItem = checkUseClausesForItem(myFacet, mySoftLink, fileLoader);
        if ((nextItem != null) && (nextItem != mySoftLink))
          { return nextItem; }
        return dereferenceItemRECURSIVE(mySoftLink, scope.getParentObject(), scope, facetList, fileLoader, firstScope);
        }

      if (myFacet.getInterfaceItems() != null)
        {
        for (posLimit = 0; posLimit < myFacet.getInterfaceItems().size(); posLimit++)
          {
          if (lastScope == myFacet.getInterfaceItems().get(posLimit))
            { break; }
          }
        if (posLimit < myFacet.getInterfaceItems().size())
          {
          for (pos = 0; pos < posLimit; pos++)
            {
            if (target.matches(myFacet.getInterfaceItems().get(pos)))
              { return myFacet.getInterfaceItems().get(pos); }
            }
          nextItem = checkUseClausesForItem(myFacet, mySoftLink, fileLoader);
          if ((nextItem != null) && (nextItem != mySoftLink))
            { return nextItem; }
          facetList.add(myFacet);
          return dereferenceItemRECURSIVE(mySoftLink, scope.getParentObject(), scope, facetList, fileLoader, firstScope);
          }
        }

      searchList = new ItemList();
      searchList.add(myFacet.getTermItems());
      searchList.add(myFacet.getDeclaredItems());
      searchList.add(myFacet.getInterfaceItems()); // This must go last!
      for (pos = 0; pos < searchList.size(); pos++)
        {
        nextItem = searchList.get(pos);
        if (lastScope == nextItem)
          { continue; }
        if (target.matches(nextItem.getLabel()))
          { return searchList.get(pos); }
        if (nextItem.getValue() instanceof EnumerationType)
          {
          int enumPos;
          EnumerationType myEnum;
          myEnum = (EnumerationType)(nextItem.getValue());
          for (enumPos = 0; enumPos < myEnum.getElements().size(); enumPos++)
            {
            if ((myEnum.getElements().get(enumPos) instanceof ItemReference)
               && (target.matches(((ItemReference)(myEnum.getElements().get(enumPos))).getItemReferred().getLabel())))
              { return ((ItemReference)(myEnum.getElements().get(enumPos))).getItemReferred(); }
            }
          }
        else if (nextItem.getValue() instanceof ConstructedType)
          {
          int constrPos, obsvrPos;
          ConstructedType myConstructedType;
          Item nextConstructorItem;
          ConstructedTypeConstructor nextConstructor;
          myConstructedType = (ConstructedType)(nextItem.getValue());
          for (constrPos = 0; constrPos < myConstructedType.getConstructors().size(); constrPos++)
            {
            nextConstructorItem = myConstructedType.getConstructors().get(constrPos);
            if (target.matches(nextConstructorItem.getLabel()))
              { return nextConstructorItem; }
            nextConstructor = (ConstructedTypeConstructor)(nextConstructorItem.getValue());
            for (obsvrPos = 0; obsvrPos < nextConstructor.getObservers().size(); obsvrPos++)
              {
              if (target.matches(nextConstructor.getObservers().get(obsvrPos).getLabel()))
                { return nextConstructor.getObservers().get(obsvrPos); }
              }
            if (target.matches(nextConstructor.getRecognizer().getLabel()))
              { return nextConstructor.getRecognizer(); }
            }
          }
        }

      if (myFacet.getParentObject() instanceof Item)
        {
        nextItem = (Item)(myFacet.getParentObject());
        if ((nextItem.getType() != myFacet) && target.matches(nextItem.getLabel()))
          { return nextItem; }
        }
      nextItem = checkUseClausesForItem(myFacet, mySoftLink, fileLoader);
      if ((nextItem != null) && (nextItem != mySoftLink))
        { return nextItem; }
      facetList.add(myFacet);
      }
    else if (scope instanceof Item)
      {
      Item scopeItem;
      scopeItem = (Item)scope;
      if ((lastScope != scopeItem.getType()) && target.matches(scopeItem.getLabel()))
        { return (Item)scope; }
      }


    // No other matches, recurse up.
    if (scope.getParentObject() != null)
      { return dereferenceItemRECURSIVE(mySoftLink, scope.getParentObject(), scope, facetList, fileLoader, firstScope); }
    else
      { return mySoftLink; }
    }

  /** The sub-method for checking for definitions imported by a use-clause.
   * @param Facet The facet whose Use clauses need to be searched for the item.
   * @param mySoftLink The item reference being searched for.
   * @param fileLoader The loader to use should external references occur.
   * @return Item The Item found to match the soft-link. If no match found, returns mySoftLink.
   */
  private static Item checkUseClausesForItem(Facet myFacet, DeferredItem mySoftLink, LinkerLoader fileLoader)
    {
    int pos, innerPos;
    Expression nextUseClause;
    DeferredItem nextImportedItem;
    ItemList subImportList, exportedItems;
    Item packageItem;
    Label target;

    if (myFacet.getUseClauses() == null)
      { return mySoftLink; }
    target = mySoftLink.getLabel();

    for (pos = 0; pos < myFacet.getUseClauses().size(); pos++)
      {
      try
        {
        nextUseClause = myFacet.getUseClauses().get(pos);
        if (nextUseClause instanceof Application)
          { nextUseClause = ((Application)nextUseClause).getLeftOp(); }
        nextImportedItem = (DeferredItem)(((ItemReference)nextUseClause).getItemReferred());
        subImportList = nextImportedItem.getImportList();
        }
      catch(Exception e)
        {
        fileLoader.linkingError("Could not decode use-clause [" + myFacet.getUseClauses().get(pos).publishROM() + "] from [" + myFacet + "] because of: " + e);
        continue;
        }

      // check current imports
      if (subImportList != null)
        {
        for (innerPos = 0; innerPos < subImportList.size(); innerPos++)
          {
          if (target.matches(subImportList.get(innerPos).getLabel()))
            { return subImportList.get(innerPos); }
          }
        }
      }

    return mySoftLink;
    }

  /** The sub-method for checking for definitions straight from the domain.
   *  This can be tricky as it may envolve facet algebra.
   * @param myFacet The facet where the item occurs. Use this to extract the domain.
   * @param mySoftLink The item reference being searched for.
   * @param fileLoader The loader to use should external references occur.
   * @return Item The Item found to match the soft-link. If no match found, returns mySoftLink.
   */
  private static Item checkDomainForItem(Facet myFacet, DeferredItem mySoftLink, LinkerLoader fileLoader)
    {
    Expression myDomain;
    Label target;

    myDomain = myFacet.getDomain();
    if (myDomain == null)
      { return mySoftLink; }
    target = mySoftLink.getLabel();

/////// *  Needs a lot of thought & work and possibly a LinkerLoader callback.

    if (myDomain instanceof ItemReference)
      {
      int pos;
      ItemReference domainRef;
      ItemList searchList;

      domainRef = (ItemReference)myDomain;
      searchList = null;
      try
        {
        // Is it an imported domain or a static definition?
        if (domainRef.getItemReferred() instanceof DeferredItem)
          { searchList = ((DeferredItem)(domainRef.getItemReferred())).getImportList(); }
        else
          { searchList = ((Facet)(domainRef.getItemReferred().getValue())).getExportItems(); }
        }
      catch(Exception e)
        { fileLoader.linkingError("Error opening domain reference [" + domainRef + "] for entry [" + myFacet + "] because of: " + e); }

      if (searchList != null)
        {
        for (pos = 0; pos < searchList.size(); pos++)
          {
          if (target.matches(searchList.get(pos).getLabel()))
            { return searchList.get(pos); }
          }
        }

      } // end if (myDomain instanceof ItemReference)
    else
      { fileLoader.linkingError("Could not decode domain [" + myDomain + "] for entry [" + myFacet + "]."); }

    return mySoftLink;
    }

  }
