//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//      authors: Garrin kimmell@ittc.ku.edu
//               Joseph josephk@cs.adelaide.edu.au
//
//  $Id: ROMSymbolTable.java,v 1.1.1.1 2002/07/01 15:47:35 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: ROMSymbolTable.java,v $
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.1  2002/01/17 00:22:24  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.1.2.2  2002/01/04 13:05:07  cvs
//  Updated the old 'AggregateReferences' to 'QuantifiedReferences'.
//
//  Revision 1.1.2.1  2002/01/04 12:28:05  cvs
//  Prototype straight-to-ROM parser.
//
//
//=============================================================================
package org.sldl.rosetta.parser.linker;

import org.sldl.rosetta.ROM.BaseObject;
import org.sldl.rosetta.ROM.Label;
import org.sldl.rosetta.ROM.Item;
import org.sldl.rosetta.ROM.Expression;
import org.sldl.rosetta.ROM.ItemReference;
import org.sldl.rosetta.ROM.EnumerationType;
import org.sldl.rosetta.ROM.ConstructedType;
import org.sldl.rosetta.ROM.ConstructedType.ConstructedTypeConstructor;
import org.sldl.rosetta.ROM.Facet;
import org.sldl.rosetta.ROM.Function;
import org.sldl.rosetta.ROM.QuantifierReference;
import org.sldl.rosetta.ROM.LetFunction;

import org.sldl.rosetta.parser.linker.symboltable.BasicSymbolTable;
import org.sldl.rosetta.parser.linker.symboltable.BasicSymbolTableEntry;



/** This represents the <B>ROMSymbolTable</B> object.
 *  This class is for building the symbol table containing entries for
 *  all the Item declared in a package.
 * @author <I> Garrin Kimmell, Joseph Kuehn </I>
 * (kimmell@ittc.ku.edu, josephk@cs.adelaide.edu.au)
 */
public class ROMSymbolTable
  {
  /** This class encapsulates any additional data desired for each ROM item
   *  being declared. It's use is currently superfluous, as each item in the
   *  ROM contains all of it's related data, hence this class is used simply
   *  to make the SymbolTable readable by humans for better understanding of
   *  the model.
   */
  public static class SymbolTableData
    {
    public final Expression myType, myValue;
    public SymbolTableData(Expression type, Expression value)
      {
      myType = type;
      myValue = value;
      }
    public String toString()
      { return myValue + " :: " + myType; }
    }




  /** This interface decouples the mechanism by which Rosetta models are stored
   *  from the SymbolTable lookup.
   */
  public static interface SymbolTableLoader
    {
    /** Method for outputting errors encountered during linking.
     * @param errorMessage A string describing the error encountered.
     */
    public void linkingError(String errorMessage);


    /** Method to help keep this file as implementation independant as possible.
     * @param rootObject The object which is to be the root of the symbol table.
     * @return BasicSymbolTable The new symbol table for that object.
     */
    public BasicSymbolTable createSymbolTable(Object rootObject);
    }



  /** The primary method.
   * @param rootROM The rosetta model to be checked for linking determinism.
   * @param fileLoader The object which can provide other packages used by this model.
   * @return BasicSymbolTable The symbol table used for linking.
   */
  public static BasicSymbolTable checkModel(Item rootROM, SymbolTableLoader fileLoader)
    {
    BasicSymbolTable myTable;

    if ((rootROM == null) || (fileLoader == null))
      { return null; }

    myTable = null;
    try
      {
      myTable = buildTable(rootROM, fileLoader);
      if (sanityCheck(myTable.getRootEntry(), fileLoader) > 0)
        {
        fileLoader.linkingError("Sanity check failed for [" + rootROM + "], cannot continue.");
        return myTable;
        }
      }
    catch(Exception e)
      {
      fileLoader.linkingError("Building of symbol table failed for [" + rootROM + "] because of: " + e);
      return myTable;
      }

    return myTable;
    }







  /** A tool-box method that can be used to 'sanity-check' the symbol-table (ie verify that 
   *  there are no ambiguities caused by two entries at the same scope having the same string.
   *  Note that the sanity check is case insensitive.
   * @param myScope The current symbol table scope to search for errors.
   * @return int The number of errors encountered during the pass - 0 is desired. ;)
   */
  public static int sanityCheck(BasicSymbolTableEntry myScope, SymbolTableLoader fileLoader)
    {
    int myCount, pos, innerPos;
    String match;

    if ((myScope == null) || (fileLoader == null))
      { return 0; }

    myCount = 0;
    for (pos = 0; pos < myScope.getChildCount(); pos++)
      {
      try
        {
        match = myScope.getEntryAt(pos).getObject().toString();
        for (innerPos = pos + 1; innerPos < myScope.getChildCount(); innerPos++)
          {
          if (myScope.getEntryAt(innerPos).getObject().toString().equalsIgnoreCase(match))
            {
            myCount++;
            fileLoader.linkingError("Sanity check failed at [" + myScope + "] because entry #" + pos + " [" + myScope.getEntryAt(pos) + "] conflicts with entry #" + innerPos + " [" + myScope.getEntryAt(innerPos) + "]");
            }
          }
        }
      catch(Exception e)
        {
        myCount++;
        fileLoader.linkingError("Sanity check failed because of [" + e + "] at [" + myScope + "] entry #" + pos + " [" + myScope.getEntryAt(pos) + "]");
        }
      }

    for (pos = 0; pos < myScope.getChildCount(); pos++)
      { myCount += sanityCheck(myScope.getEntryAt(pos), fileLoader); }

    return myCount;
    }







  /** Method for generating and resolving dynamic names given to anonymous expressions.
   *  This method should produce an otherwise illegal Rosetta label so that there is no
   *  danger of collisions in the symbol-table with user-labelled items.
   * @param Expression The dynamic construct that needs a name.
   * @return String The matching name generated for that construct.
   */
  public static String generateName(BaseObject anonymousItem)
    {
    if (anonymousItem == null)
      { return ""; }
    if (anonymousItem.getSourceReference() != null)
      { return anonymousItem.getSourceReference().hashString(); }
    return "___" + anonymousItem.getDotString() + "___";
    }








  /** The first-pass method. This method builds the main symbol table recursively using the
   *  <I>visitor</I> pattern on the BaseObject <I>TreeNode</I> interface. This is the method
   *  to change if the underlying SymbolTable implementation needs to be changed.
   * @param root The rosetta model to be searched for items.
   * @return BasicSymbolTable The symboltable that results from the first pass.
   */
  public static BasicSymbolTable buildTable(Item root, SymbolTableLoader fileLoader)
    {
    int pos;
    BasicSymbolTable myTable;

    if ((root == null) || (fileLoader == null))
      { return null; }

    myTable = fileLoader.createSymbolTable(root);
    myTable.getCurrentEntry().setData(new SymbolTableData(root.getType(), root.getValue()));
    for (pos = 0; pos < root.getChildCount(); pos++)
      { buildTable(root.getChildObjectAt(pos), myTable.getRootEntry(), fileLoader); }
    return myTable;
    }

  /** The first-pass sub-method. This method builds the main symbol table recursively using the
   *  <I>visitor</I> pattern on the BaseObject <I>TreeNode</I> interface.
   * @param root The current rosetta model to be searched for items.
   * @param table The symbol table being built.
   * @param fileLoader The loader to use should external references occur.
   * @return void
   */
  private static void buildTable(BaseObject root, BasicSymbolTableEntry scope, SymbolTableLoader fileLoader)
    {
    int pos;
    Item myItem;
    BasicSymbolTableEntry childScope;

    if ((root == null) || (scope == null))
      { return; }

    childScope = scope;
    if (root instanceof Item)
      {
      myItem = (Item)root;
      scope.addChild(myItem);
      scope.getChildEntry(myItem).setData(new SymbolTableData(myItem.getType(), myItem.getValue()));
      childScope = scope.getChildEntry(myItem);
      if (myItem.getValue() instanceof EnumerationType)
        {
        Item nextItem;
        EnumerationType myEnum;
        myEnum = (EnumerationType)(myItem.getValue());
        for (pos = 0; pos < myEnum.getElements().size(); pos++)
          {
          try
            { scope.addChild(((ItemReference)(myEnum.getElementAt(pos))).getItemReferred()); }
          catch(ClassCastException e)
            { }
          }
        return; // Do not go any deeper - otherwise entries will be added twice!
        }
      else if (myItem.getValue() instanceof ConstructedType)
        {
        int obsv;
        Item nextItem;
        ConstructedType myConstr;
        ConstructedTypeConstructor nextConstructor;

        myConstr = (ConstructedType)(myItem.getValue());
        for (pos = 0; pos < myConstr.getConstructors().size(); pos++)
          {
          nextItem = myConstr.getConstructors().get(pos);
          scope.addChild(nextItem);
          nextConstructor = (ConstructedTypeConstructor)(nextItem.getValue());
          scope.addChild(nextConstructor.getRecognizer());
          for (obsv = 0; obsv < nextConstructor.getObservers().size(); obsv++)
            { scope.addChild(nextConstructor.getObservers().get(obsv)); }
          }
        return; // Do not go any deeper - otherwise entries will be added twice!
        }
      }
    else if ((root instanceof QuantifierReference) || (root instanceof Facet)
             || (root instanceof Function) || (root instanceof LetFunction))
      {
      myItem = new Item(new Label(generateName(root)), null, Expression.safeDupe((Expression)root));
      if (root.getSourceReference() != null)
        { myItem.setSourceReference(root.getSourceReference().join(null)); }
      scope.addChild(myItem);
      scope.getChildEntry(myItem).setData(new SymbolTableData(myItem.getType(), myItem.getValue()));
      childScope = scope.getChildEntry(myItem);
      }

    for (pos = 0; pos < root.getChildCount(); pos++)
      { buildTable(root.getChildObjectAt(pos), childScope, fileLoader); }
    }

  }
