//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: DefaultWalker.java,v 1.1 2002/07/15 04:56:39 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: DefaultWalker.java,v $
//  Revision 1.1  2002/07/15 04:56:39  cvs
//  Added a pattern for a depth-first traversal of the ROM.
//
//
//=============================================================================
package org.sldl.rosetta.parser.patterns;

import java.util.Map;
import org.sldl.rosetta.ROM.BaseObject;


/** This is the <B>DefaultWalker</B> class which provides a basic
 *  implementation of the <code>Walker</code> interface.
 */
public class DefaultWalker implements Walker
  {
  private WalkerNodeVisitor myVisitor;
  private Class[] wantedNodes;
  private Boolean nlr;


  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public DefaultWalker(Map settings)
    {
    if (settings == null)
      {
      myVisitor = null;
      nlr = new Boolean(true);
      }
    else
      {
      myVisitor = (WalkerNodeVisitor)settings.get(Walker.keySet[0]);
      nlr = (Boolean)settings.get(Walker.keySet[1]);
      }
    wantedNodes = null;
    setWalkerNodeVisitor(myVisitor);
    }

  /** Secondary Constructor.
   * @param visitor The WalkerNodeVisitor this walker applies.
   * @param isNLR Is the node-walker NLR?
   */
  public DefaultWalker(WalkerNodeVisitor visitor, Boolean isNLR)
    {
    myVisitor = visitor;
    nlr = isNLR;
    wantedNodes = null;
    setWalkerNodeVisitor(myVisitor);
    }



  /** Method for retreiving the Visitor being applied to this ROM.
   * @return WalkerNodeVisitor The Visitor being applied to this ROM.
   */
  public WalkerNodeVisitor getWalkerNodeVisitor()
    { return myVisitor; }

  /** Method for setting the Visitor to be applied to this ROM.
   * @param visitor The Visitor to be applied to this ROM.
   * @return void
   */
  public void setWalkerNodeVisitor(WalkerNodeVisitor visitor)
    {
    myVisitor = visitor;
    if ((myVisitor != null) && (myVisitor instanceof AdvancedWalkerNodeVisitor))
      { wantedNodes = ((AdvancedWalkerNodeVisitor)myVisitor).getWantedNodes(); }
    else
      { wantedNodes = null; }
    }



  /** Method for setting the traversal mode of the Walker.
   *  If true this will make the traversal <B><U>N-L-R</U></B>,
   *  if false (or null) this will make the traversal <B><U>L-R-N</U></B>.
   * @param setting Is the traversal to be N-L-R?
   * @return void
   */
  public void setIsNLR(Boolean setting)
    { nlr = setting; }

  /** Method for checking if the traversal mode is <B><U>N-L-R</U></B>.
   * @return Boolean Is the traversal mode NLR?
   */
  public Boolean getIsNLR()
    { return nlr; }









  /** Method for checking if a node should be visited.
   * @param node The node currently being traversed.
   * @return boolean Should we visit this node?
   */
  private boolean isWanted(BaseObject node)
    {
    int pos;

    if (wantedNodes == null)
      { return true; }
    for (pos = 0; pos < wantedNodes.length; pos++)
      {
      if ((wantedNodes[pos] != null) && wantedNodes[pos].isInstance(node))
        { return true; }
      }
    return false;
    }

  /** Method for walking the ROM tree.
   *  The traversal method is user-specified.
   * @param root The root of the ROM tree being walked.
   * @return Object The final state of the walker after traversing the whole ROM tree.
   */
  public Object walk(BaseObject root)
    { return walk(root, myVisitor.getInitialState(root)); }

  /** Sub-method for walking the ROM tree.
   * @param root The root of the ROM tree being walked.
   * @param previousState The previous state of the walker.
   * @return Object The new state of the walker after traversal of this sub-tree.
   */
  private Object walk(BaseObject root, Object previousState)
    {
    int pos;
    Object latestState;

    latestState = previousState;
    if (root == null)
      { return latestState; }

    if ((nlr == null) || !nlr.booleanValue())
      {
      for (pos = 0; pos < root.getChildCount(); pos++)
        { latestState = walk(root.getChildObjectAt(pos), latestState); }
      if (isWanted(root))
        { latestState = myVisitor.visitROMNode(root, latestState); }
      }
    else
      {
      if (isWanted(root))
        { latestState = myVisitor.visitROMNode(root, latestState); }
      for (pos = 0; pos < root.getChildCount(); pos++)
        { latestState = walk(root.getChildObjectAt(pos), latestState); }
      }

    return latestState;
    }

  }

