//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: Walker.java,v 1.1 2002/07/15 04:56:39 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Walker.java,v $
//  Revision 1.1  2002/07/15 04:56:39  cvs
//  Added a pattern for a depth-first traversal of the ROM.
//
//
//=============================================================================
package org.sldl.rosetta.parser.patterns;

import java.io.Serializable;
import org.sldl.rosetta.ROM.BaseObject;


/** This is the <B>Walker</B> interface which provides an easy method for
 *  quickly visiting a specific subset of nodes within a ROM. In particular,
 *  when we only wish to perform an operation on a small subset of nodes and
 *  when this operation does not require much additional state information to be
 *  constructed during traversal (other than the nodes being visited) then this
 *  class is ideal.<BR>
 *  The traversal method can be either <B><U>N-L-R</U></B> or <B><U>L-R-N</U></B>
 *  as specified by the user (since the tree is not binary <I>L-N-R</I> is not appropriate).
 */
public interface Walker extends Serializable
  {
  public static final String[] keySet = { "WalkerNodeVisitor", "IsNLR" };


  /** Basic interface required for the class which is to perform the
   *  operations on the ROM tree.
   */
  public static interface WalkerNodeVisitor extends Serializable
    {
    public static final String[] keySet = {};


    /** Method for obtaining the initial state of the walker.
     *  While the root of the ROM tree is passed in, this should _not_
     *  be thought of as the same as 'visiting' the first node, since
     *  if it is appropriate it will be visited with the next method
     *  once this method returns.
     * @param root The root of the ROM tree being walked.
     * @return Object The initial state of the walker.
     */
    public Object getInitialState(BaseObject root);

    /** Method for operating on this specific node.
     * @param node The current BaseObject node being examined.
     * @param previousState The previous state of the walker.
     * @return Object The new state of the walker.
     */
    public Object visitROMNode(BaseObject node, Object previousState);
    }



  /** A more advanced interface allows a user class to specify a
   *  small subset of classes which it is interested in visiting.
   *  This interface will cause the <B>Walker</B> to call the
   *  <I>visitROMNode</I> method only on those ROM nodes which
   *  are instanceof one of the Classes provided.
   */
  public interface AdvancedWalkerNodeVisitor extends WalkerNodeVisitor
    {
    public static final String[] keySet = { "WantedNodes" };


    /** Method for specifying that only some of the nodes should
     *  be visited by the walker. If the result is <I>null</I> then
     *  all classes will be visited, if the result is <I>new Class[0]</I>
     *  then no classes will be visited.
     * @return Class[] The classes of the nodes that should be visited.
     */
    public Class[] getWantedNodes();
    }






  /** Method for retreiving the Visitor being applied to this ROM.
   * @return WalkerNodeVisitor The Visitor being applied to this ROM.
   */
  public WalkerNodeVisitor getWalkerNodeVisitor();

  /** Method for setting the Visitor to be applied to this ROM.
   * @param visitor The Visitor to be applied to this ROM.
   * @return void
   */
  public void setWalkerNodeVisitor(WalkerNodeVisitor visitor);



  /** Method for setting the traversal mode of the Walker.
   *  If true this will make the traversal <B><U>N-L-R</U></B>,
   *  if false (or null) this will make the traversal <B><U>L-R-N</U></B>.
   * @param setting Is the traversal to be N-L-R?
   * @return void
   */
  public void setIsNLR(Boolean setting);

  /** Method for checking if the traversal mode is <B><U>N-L-R</U></B>.
   * @return Boolean Is the traversal mode NLR?
   */
  public Boolean getIsNLR();



  /** Method for walking the ROM tree.
   *  The traversal method is user-specified.
   * @param root The root of the ROM tree being walked.
   * @return Object The final state of the walker after traversing the whole ROM tree.
   */
  public Object walk(BaseObject root);

  }
