//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id:
//
//  Revision History:
//  ----------------
//  $Log:
//
//=============================================================================
package org.sldl.rosetta.parser.tarp.antlr;

import java.io.*;

//ROM interface
//import org.sldl.rosetta.rom.*;
//import org.sldl.rosetta.rom.tarp.*;
//now using KU ROM
import org.sldl.rosetta.ROM.*;
import org.sldl.rosetta.parser.tarp.misc.*;

public class Driver {

    /**
     *Command-line method to read in a file or directory structure containing
     *Rosetta code (.sld extension), transform those files into ROM objects, and perform
     *basic linking
     *<br><i>Usage: java org.sldl.rosetta.parser.tarp.antlr.Driver [-rom] [-o output_file] (directory or file name)</i>
     *@param String -rom : display the generated ROM object as a tree-structure
     *@param String -o : serialize the ROM to the designated output-file
     *@param String filename.sld/directory
     */
    public static void main(String[] args) {
	boolean showRom = false;
	boolean linkRom = false;
	ObjectOutputStream outputFile = null;
	// Use a try/catch block for parser exceptions
	try {
	    // if we have at least one command-line argument
	    if (args.length > 0 ) {
		System.err.println("Analysing...");
		
		// for each directory/file specified on the command line
		for(int i=0; i< args.length;i++) {
		    if ((args[i]).equals("-rom")) //show the generated ROM in a tree
			{ showRom = true; }
		    else if ((args[i]).equals("-o")) //output the rom to the specified file
			{
			    FileOutputStream FOS = new FileOutputStream(args[i+1]);
			    outputFile = new ObjectOutputStream(FOS);
			    i++;
			}
		    else if ((args[i]).equals("-link")) //link the generated ROM BaseObject
			{
			    linkRom = true;
			}
		    else 
			{ doFile( new File(args[i]), showRom, linkRom, outputFile ); } //parse the file
		}
	    }
	    else
		System.err.println("Usage: java org.sldl.rosetta.parser.tarp.antlr.Driver [-rom] [-link] [-o <output file>] <directory or file name>");
	}
	catch(Exception e) {
	    System.err.println("exception: "+e);
	    e.printStackTrace(System.err);   // so we can get stack trace
	}
    }
    
    
    /**
     *This method decides what action to take based on the type of
     *file we are looking at (directory, or .sld file?)
     *@param File the file/directory being parsed
     *@param boolean showRom
     *@param boolean linkRom Do we link the generated Rom?
     *@param ObjectOutputStream the output File into which the ROM should be serialized
     */
    public static void doFile(File f, boolean showRom, boolean linkRom, ObjectOutputStream outputFile)
	throws Exception {
	// If this is a directory, walk each file/dir in that directory
	if (f.isDirectory()) {
	    String files[] = f.list();
	    for(int i=0; i < files.length; i++)
		doFile(new File(f, files[i]), showRom, linkRom, outputFile);
	}
	
	// otherwise, if this is a .sld file, parse it
	else if ((f.getName().length()>5) &&
		 f.getName().substring(f.getName().length()-4).equals(".sld")) {
	    System.err.println("   "+f.getAbsolutePath());
	    parseInputStream(f.getName(), new FileReader(f), showRom, linkRom, outputFile);
	}
    }
    
    /**
     *This method parses and links an input stream of rosetta code. 
     *Dispay of the ROM is handled here
     *@param String the originating filename
     *@param InputStream the rosetta code as an input stream
     *@param boolean showRom
     *@param boolean linkRom Do we link the generated Rom?
     *@param ObjectOutputStream the output File into which the ROM should be serialized
     */
    public static void parseInputStream(String f, FileReader fr, boolean showRom, boolean linkRom, ObjectOutputStream outputFile) throws Exception
    {
	BaseObject generatedRomItemList = null;
	try {
	    Parser myParser=new Parser();
	    generatedRomItemList=myParser.parseReader(fr, true);
	    if ( linkRom )
		{
		    //remove linking for the moment
		    //Linker myLinker = new Linker();
		    //generatedRomItemList=myLinker.Link(generatedRomItemList);
		}
	    //parser.setFilename(f);
	    if ( showRom )
		{
		    if (generatedRomItemList!=null)
			{
			    ShowTreeNode.Display(generatedRomItemList);
			}
		}
	    if ( outputFile!=null ) //want to output the ROM as a serialized object
		{
		    outputFile.writeObject(generatedRomItemList);
		}
	}
	catch (Exception e) {
	    System.err.println("parser exception: "+e);
	    e.printStackTrace();   // so we can get stack trace		
	}
    }
}
