//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id:
//
//  Revision History:
//  ----------------
//  $Log:
//
//=============================================================================
package org.sldl.rosetta.parser.tarp.antlr;

import org.sldl.rosetta.parser.tarp.antlr.MyAST;
import org.sldl.rosetta.parser.tarp.antlr.MyCommonToken;

import org.sldl.rosetta.parser.tarp.antlr.RosettaTreeParser;
import org.sldl.rosetta.parser.tarp.antlr.RosettaParser;
import org.sldl.rosetta.parser.tarp.antlr.RosettaLexer;
import java.io.*;
import java.util.Vector;
import antlr.collections.AST;
import antlr.collections.impl.*;
import antlr.debug.misc.*;
import antlr.*;
import java.awt.event.*;

//showing the generated ROM
import javax.swing.tree.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;

//ROM interface
//import org.sldl.rosetta.rom.*;
//import org.sldl.rosetta.rom.tarp.*;
//using the KU ROM
import org.sldl.rosetta.ROM.*;
import org.sldl.rosetta.parser.tarp.misc.*;

public class Launcher {

    /**
     *Method to command the parser to read the pre-supplied input stream and
     *perform any necessary translation steps desired before returning the resulting ROM object.
     *External Programs should use this, the most generic, method, and handle any serialization
     *themselves.
     *@return BaseObject ROM Model of the input stream for tree-display purposes
     *@throws Exception if any parsing exception occured in the process.
     */
    public BaseObject parseReader(Reader r) throws Exception {
	// Create a scanner that reads from the input stream passed to us
	RosettaLexer lexer = new RosettaLexer(r);
	
	lexer.setTokenObjectClass("org.sldl.rosetta.parser.tarp.antlr.MyCommonToken");        
	
	// Create a parser that reads from the scanner
	RosettaParser parser = new RosettaParser(lexer);
	parser.setASTNodeClass("org.sldl.rosetta.parser.tarp.antlr.MyAST");
	
	// start parsing at the design_file rule
	// parser now carries with it an AST object
	parser.design_file();
	
	BaseObject myDesignFile = null;
	if ( parser.getAST()!=null ) {
	    RosettaTreeParser tparse = new RosettaTreeParser();
	    
	    try {
		myDesignFile = tparse.design_file(parser.getAST());
		//display any warnings generated during the tree-parse.
		displayWarnings(tparse.myWarnings);
	    }catch (antlr.RecognitionException e) {
		System.err.println(e.getMessage());
		e.printStackTrace();
	    }
	}
	return myDesignFile;
    }

    /**
     *Command-line method to read in a file or directory structure containing
     *Rosetta code (.sld extension) and transform those files into ROM objects.
     *<br><i>Usage: java org.sldl.rosetta.parser.tarp.Launcher [-ast] [-rom] [-o output_file] (directory or file name)</i>
     *@param String -ast : display the abstract syntax tree used in the parsing of the file
     *@param String -rom : display the generated ROM object as a tree-structure
     *@param String -o : serialize the ROM to the designated output-file
     *@param String filename.sld/directory
     */
    public static void main(String[] args) {
	boolean showRom = false;
	boolean showTree = false;
	ObjectOutputStream outputFile = null;
	// Use a try/catch block for parser exceptions
	try {
	    // if we have at least one command-line argument
	    if (args.length > 0 ) {
		System.err.println("Analysing...");
		
		// for each directory/file specified on the command line
		for(int i=0; i< args.length;i++) {
		    if ((args[i]).equals("-ast")) //show the abstract syntax tree
			{showTree = true;}
		    else if ((args[i]).equals("-rom")) //show the generated ROM in a tree
			{ showRom = true; }
		    else if ((args[i]).equals("-o")) //output the rom to the specified file
			{
			    FileOutputStream FOS = new FileOutputStream(args[i+1]);
			    outputFile = new ObjectOutputStream(FOS);
			    i++;
			}
		    else 
			{ doFile( new File(args[i]), showRom, showTree, outputFile ); } //parse the file
		}
	    }
	    else
		System.err.println("Usage: java org.sldl.rosetta.parser.tarp.Launcher [-ast] [-rom] [-o <output file>] <directory or file name>");
	}
	catch(Exception e) {
	    System.err.println("exception: "+e);
	    e.printStackTrace(System.err);   // so we can get stack trace
	}
    }
    
    
    /**
     *This method decides what action to take based on the type of
     *file we are looking at (directory, or .sld file?)
     *@param File the file/directory being parsed
     *@param boolean showRom
     *@param boolean showTree
     *@param ObjectOutputStream the output File into which the ROM should be serialized
     */
    public static void doFile(File f, boolean showRom, boolean showTree, ObjectOutputStream outputFile)
	throws Exception {
	// If this is a directory, walk each file/dir in that directory
	if (f.isDirectory()) {
	    String files[] = f.list();
	    for(int i=0; i < files.length; i++)
		doFile(new File(f, files[i]), showRom, showTree, outputFile);
	}
	
	// otherwise, if this is a .sld file, parse it
	else if ((f.getName().length()>5) &&
		 f.getName().substring(f.getName().length()-4).equals(".sld")) {
	    System.err.println("   "+f.getAbsolutePath());
	    parseInputStream(f.getName(), new FileInputStream(f), showRom, showTree, outputFile);
	}
    }
    
    /**
     *This method parses an input stream of rosetta code. 
     *Dispay of the ROM is handled here
     *@param String the originating filename
     *@param InputStream the rosetta code as an input stream
     *@param boolean showRom
     *@param boolean showTree
     *@param ObjectOutputStream the output File into which the ROM should be serialized
     */
    public static void parseInputStream(String f, InputStream s, boolean showRom, boolean showTree, ObjectOutputStream outputFile) throws Exception
    {
	BaseObject generatedRomItemList = null;
	try {
	    // Create a scanner that reads from the input stream passed to us
	    RosettaLexer lexer = new RosettaLexer(s);
	    lexer.setFilename(f);
	    lexer.setTokenObjectClass("org.sldl.rosetta.parser.tarp.antlr.MyCommonToken");	    

	    // Create a parser that reads from the scanner
	    RosettaParser parser = new RosettaParser(lexer);
	    parser.setFilename(f);
	    parser.setASTNodeClass("org.sldl.rosetta.parser.tarp.antlr.MyAST");
	    
	    // start parsing at the design_file rule
	    // parser now carries with it a MyAST object
	    parser.design_file();

	    //display any warnings generated
	    displayWarnings(parser.myWarnings);
	    
	    // generate ROM from the AST
	    generatedRomItemList = doTreeAction(f, (MyAST)parser.getAST(), parser.getTokenNames(), showTree);
	    if ( showRom )
		{
		    if (generatedRomItemList!=null)
			{
			    showTreeNode(generatedRomItemList);
			}
		}
	    if ( outputFile!=null ) //want to output the ROM as a serialized object
		{
		    outputFile.writeObject(generatedRomItemList);
		}
	}
	catch (Exception e) {
	    System.err.println("parser exception: "+e);
	    e.printStackTrace();   // so we can get stack trace		
	}
    }
    
    /**
     *This Method takes in a Vector (of warnings) and displays the contents similar
     *to the display in the KU parser
     *@param Vector contains the warnings with message and SourceReference
     */
    private static void displayWarnings(java.util.Vector myWarnings)
    {
	int warningPos;
        for (warningPos = 0; warningPos < myWarnings.size(); warningPos++)
        {
            System.out.println("WARNING: " + myWarnings.get(warningPos).toString());
        }
    }

    /**
     *This Method walks the generated Abstract Syntax Tree and produces a ROM object.
     *This object is returned as a Function simply to encapsulate the entire ItemList generated
     *rather than a single item.
     *@param String the originating filename
     *@param MyAST the Abstract Syntax Tree representing the design file which has been parsed
     *@param String[] a list of the node types in the Abstract Syntax Tree
     *@param boolean showTree
     *@return Function the Rosetta Object Model (ROM) of the input design file
     */
    public static BaseObject doTreeAction(String f, MyAST t, String[] tokenNames, boolean showTree) {
	BaseObject romItemList = null;
	if ( t!=null ) {
	    if ( showTree )
		{
		    ((MyAST)t).setVerboseStringConversion(true, tokenNames);
		    showAST(t);
		}
	    RosettaTreeParser tparse = new RosettaTreeParser();
	    //tparse.setFilename(f);
	    try {
		romItemList = tparse.design_file(t);
		displayWarnings(tparse.myWarnings);
	    }
	    catch (RecognitionException e) {
		System.err.println(e.getMessage());
		e.printStackTrace();
	    }
	}
	return romItemList;
    }

    /**
     *This method displays an ANTLR AST in a JFrame
     *@param AST the Abstract Syntax Tree to display
     */
    public static void showAST( AST thisAST ) {
	final ASTFrame frame = new ASTFrame("Rosetta AST", thisAST);
	frame.setVisible(true);
	frame.addWindowListener(
				new WindowAdapter() {
				    public void windowClosing (WindowEvent e) {
					frame.setVisible(false); // hide the Frame
					frame.dispose();
					System.exit(0);
				    }
				}
				);
    }

    /**
     *This method displays a Java TreeNode in a JFrame.
     *Note: only some objects in the ROM heirarchy implement the javax.swing.tree.TreeNode interface
     *@param TreeNode the TreeNode object to display
     */
    public static void showTreeNode(TreeNode thisTreeNode) {
	JTree jtree = new JTree(thisTreeNode);
	JScrollPane scrollpane = new JScrollPane() ;
	jtree.setEditable(false) ;
	jtree.setShowsRootHandles(true) ;
	jtree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION) ;
	jtree.setAutoscrolls(true);
	jtree.setRootVisible(true);
	jtree.setScrollsOnExpand(true) ;
	scrollpane.getViewport().add(jtree);
	JFrame jframe = new JFrame("ROM Tree Viewer");
	jframe.setContentPane(scrollpane );
	jframe.setVisible(true);
	jframe.setResizable(true) ;
	jframe.pack() ;
	jframe.show() ;

    }

}
