//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: ArrayMapParser.java,v 1.3 2002/10/30 17:33:11 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: ArrayMapParser.java,v $
//  Revision 1.3  2002/10/30 17:33:11  cvs
//  Added the files relevant to ANTLR to the cvs.
//  --Murthy.
//
//  Revision 1.3  2002/09/12 03:40:38  josephk
//  Re-implemented the ArrayMapParser to handle Arrays in a more efficient manner.
//
//  Revision 1.2  2002/09/11 08:28:50  josephk
//  Fixed the VectorMapParser to use java arrays instead of BaseObject.ListEntry.
//  Fixed the ArrayMapParser to use it's own class instead of BaseObject.ListEntry.
//  Added ArrayMapParser.ArrayListEntry to the default MapParser population.
//  Added a depth register when debugging the MapParser.
//  Cleaned up HashMapParser slightly.
//
//  Revision 1.1  2002/08/27 07:15:25  josephk
//  Added the new implimentation of the Mapping tools.
//
//
//=============================================================================
package org.sldl.rosetta.parser.tarp.map;

import java.io.Serializable;
import java.lang.reflect.Array;
import java.util.HashMap;
import java.util.Map;


/** This represents the <B>ArrayMapParser</B> object. This object
 *  provides the ability to construct/deconstruct arrays.
 */
public class ArrayMapParser implements MapObjectDeconstructor, MapObjectReconstructor
  {
  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { };


  /** Lists the keys used to encode/decode arrays.
   */
  public static final String[] arrayKeySet = { "ArrayType", "Size" };
  public static final String multiArrayClassHeader = "[";
  public static final String singleArrayClassHeader = "[L";

  protected static final String keyKey = MapParser.keyKey;



  /** Method to get the Array key.
   * @return String The key that corresponds to Array deconstructors/reconstructors.
   */
  public static String getArrayClassName()
    { return "java.Array"; }



  /** Primary Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public ArrayMapParser(Map settings)
    { }



  /** Method to check if this deconstructor can handle a given object.
   * @param target The object to deconstruct.
   * @return boolean Can this deconstructor handle that object?
   */
  public boolean isValidDeconstructor(Object target)
    { return ((target != null) && target.getClass().isArray()); }


  /** Method to perform the deconstruction.
   * @param object The object to be deconstructed.
   * @return Map The map containing the contents of the object.
   * @throws Exception If an error occurs it should be propogated.
   */
  public Map deconstruct(Object object) throws Exception
    {
    if (!isValidDeconstructor(object))
      { return null; }

    Map result;
    result = new HashMap();
    result.put(keyKey, getArrayClassName());
    result.put(arrayKeySet[0], object.getClass().getName());
    result.put(arrayKeySet[1], new Integer(Array.getLength(object)));
    convertArray(object, result);
    return result;
    }





  /** Method to check if this reconstructor can handle a given map.
   * @param target The map to reconstruct.
   * @return boolean Can this reconstructor handle that map?
   */
  public boolean isValidReconstructor(Map target)
    {
    if ((target == null) || !(target.get(keyKey) instanceof String))
      { return false; }
    return getArrayClassName().equals((String)target.get(keyKey));
    }


  /** Method to perform the reconstruction.
   * @param map The map containing the contents of the object.
   * @return Object The reconstructed object.
   * @throws Exception If an error occurs it should be propogated.
   */
  public Object reconstruct(Map map) throws Exception
    {
    int pos, size;
    String arrayType, subType;
    Object tempArray;
    Class arrayClass;

    if (!isValidReconstructor(map))
      { return null; }

    arrayType = (String)(map.get(arrayKeySet[0]));
    size = ((Integer)(map.get(arrayKeySet[1]))).intValue();

    subType = arrayType;
    /* Prune the ";" from the end of the Class.getName() string. */
    if ((arrayType != null) && (arrayType.endsWith(";")))
      { arrayType = arrayType.substring(0, arrayType.length() - 1); }
    /* MUST BE FIRST: Check for a multi-dimensional array, prune only a single "[" off. */
    if ((arrayType != null) && (arrayType.startsWith(multiArrayClassHeader)))
      { subType = arrayType.substring(multiArrayClassHeader.length()); }
    /* MUST BE SECOND: Check for a single-dimension array, prune the "[L" off. */
    if ((arrayType != null) && (arrayType.startsWith(singleArrayClassHeader)))
      { subType = arrayType.substring(singleArrayClassHeader.length()); }

    arrayClass = Class.forName(subType);
    tempArray = Array.newInstance(arrayClass, size);

    for (pos = 0; pos < size; pos++)
      { Array.set(tempArray, pos, map.get(new Integer(pos))); }
    return tempArray;
    }




  /** Method to save the elements of an Array into a Store.
   * @param array The array to flatten.
   * @param store The Map to place the array elements into.
   * @return void
   */
  public static void convertArray(Object array, Map store)
    {
    int pos;

    if ((array == null) || (store == null) || !array.getClass().isArray())
      { return; }

    for (pos = 0; pos < Array.getLength(array); pos++)
      { store.put(new Integer(pos), Array.get(array, pos)); }
    }

  }

