//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: DefaultMapObjectDeconstructor.java,v 1.3 2002/10/30 17:33:11 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: DefaultMapObjectDeconstructor.java,v $
//  Revision 1.3  2002/10/30 17:33:11  cvs
//  Added the files relevant to ANTLR to the cvs.
//  --Murthy.
//
//  Revision 1.1  2002/08/27 07:15:25  josephk
//  Added the new implimentation of the Mapping tools.
//
//
//=============================================================================
package org.sldl.rosetta.parser.tarp.map;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;


/** Basic implementation to provide the deconstruction transformation
 *  from an object to a map of (key,value) pairs.
 *  You can use this class if your objects all provide a
 *  set of interfaces with a <B>public String[] keySet</B>
 *  field that names the members to interrogate by pre-pending
 *  the string "get" to each and searching for a matching method.
 */
public class DefaultMapObjectDeconstructor implements MapObjectDeconstructor
  {
  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { };


  protected static final String fieldKey = MapParser.fieldKey;
  protected static final String keyKey = MapParser.keyKey;


  /** Primary Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public DefaultMapObjectDeconstructor(Map settings)
    { }



  /** Method to check if this deconstructor can handle a given object.
   * @param target The object to deconstruct.
   * @return boolean Can this deconstructor handle that object?
   */
  public boolean isValidDeconstructor(Object target)
    {
    try
      { target.getClass().getField(fieldKey).get(target); }
    catch(Exception e)
      { return false; }

    return true;
    }


    
  /** Method to perform the deconstruction.
   * @param object The object to be deconstructed.
   * @return Map The map containing the contents of the object.
   * @throws Exception If an error occurs it should be propogated.
   */
  public Map deconstruct(Object object) throws Exception
    {
    int pos, fieldPos;
    String[] nextKeys;
    String nextKey;
    Vector classList, fieldKeys;
    Class objectClass, nextClass;
    Map result;

    
    if (!isValidDeconstructor(object))
      { return null; }

    objectClass = object.getClass();

    result = new HashMap();
    result.put(keyKey, objectClass.getName());

    classList = new Vector();
    getClasses(objectClass, classList);

    fieldKeys = new Vector();
    for (pos = 0; pos < classList.size(); pos++)
      {
      try
        {
        nextClass = (Class)(classList.get(pos));
        nextKeys = (String[])(nextClass.getField(fieldKey).get(null));
        for (fieldPos = 0; fieldPos < nextKeys.length; fieldPos++)
          {
          if (!fieldKeys.contains(nextKeys[fieldPos]))
            { fieldKeys.add(nextKeys[fieldPos]); }
          }
        }
      catch(Exception e)
        { }
      }

    for (pos = 0; pos < fieldKeys.size(); pos++)
      {
      nextKey = (String)(fieldKeys.get(pos));
      result.put(nextKey, objectClass.getMethod("get" + nextKey, null).invoke(object, null));
      }
    return result;
    }

  /** Method to add to a list all of the classes a given
   *  class implements. These are added in NLR order.
   * @param node The current class of interest.
   * @param container The container to place the super-classes/interfaces in.
   * @return void
   */
  public static void getClasses(Class node, Vector container)
    {
    int pos;
    Class[] interfaces;
  
    if ((node == null) || (container == null))
      { return; }

    container.add(node);
    getClasses(node.getSuperclass(), container);

    interfaces = node.getInterfaces();
    for (pos = 0; pos < interfaces.length; pos++)
      {
      container.add(interfaces[pos]);
      getClasses(interfaces[pos], container);
      }
    }


  }
