//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: DefaultMapObjectReconstructor.java,v 1.3 2002/10/30 17:33:11 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: DefaultMapObjectReconstructor.java,v $
//  Revision 1.3  2002/10/30 17:33:11  cvs
//  Added the files relevant to ANTLR to the cvs.
//  --Murthy.
//
//  Revision 1.1  2002/08/27 07:15:25  josephk
//  Added the new implimentation of the Mapping tools.
//
//
//=============================================================================
package org.sldl.rosetta.parser.tarp.map;

import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.Map;


/** Basic implementation to provide the reconstruction transformation
 *  from a map of (key,value) pairs to an object.
 *  You can use this class if your objects all provide a
 *  constructor that takes in just a Map containing all of the
 *  information needed to initialise the object.
 */
public class DefaultMapObjectReconstructor implements MapObjectReconstructor
  {
  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "Reconstructables" };


  protected static final String fieldKey = MapParser.fieldKey;
  protected static final String keyKey = MapParser.keyKey;


  private String[] myReconstructables;

  /** Primary Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public DefaultMapObjectReconstructor(Map settings)
    {
    Map mySettings;

    mySettings = settings;
    if (mySettings == null)
      { mySettings = new HashMap(); }

    myReconstructables = (String[])mySettings.get(DefaultMapObjectReconstructor.keySet[0]);
    if (myReconstructables == null)
      { myReconstructables = new String[0]; }
    }


  /** Method to get the names of the classes this object can reconstruct.
   * @return String[] The target class names.
   */
  public String[] getReconstructables()
    { return myReconstructables; }


  /** Method to check if this reconstructor can handle a given map.
   * @param target The map to reconstruct.
   * @return boolean Can this reconstructor handle that map?
   */
  public boolean isValidReconstructor(Map target)
    {
    int pos;
    String targetName;

    if ((target == null) || !(target.get(keyKey) instanceof String))
      { return false; }
    targetName = (String)target.get(keyKey);
    for (pos = 0; pos < myReconstructables.length; pos++)
      {
      if (targetName.equals(myReconstructables[pos]))
        { return true; }
      }
    return false;
    }


  /** Method to perform the reconstruction.
   * @param map The map containing the contents of the object.
   * @return Object The reconstructed object.
   * @throws Exception If an error occurs it should be propogated.
   */
  public Object reconstruct(Map map) throws Exception
    {
    int pos, paramPos;
    Object[] values;
    Class[] types;
    Object myObject;
    Constructor[] choices;
    String myClassname;

    if (!isValidReconstructor(map))
      { return null; }

    types = new Class[] { Class.forName("java.util.Map") };
    values = new Object[] { map };

    myClassname = (String)map.get(keyKey);
    choices = Class.forName(myClassname).getConstructors();
    myObject = null;
    for (pos = 0; (myObject == null) && (pos < choices.length); pos++)
      {
      if (choices[pos].getParameterTypes().length != types.length)
        { continue; }
      for (paramPos = 0; paramPos < types.length; paramPos++)
        {
        if (types[paramPos] == null)
          { continue; }
        if (!choices[pos].getParameterTypes()[paramPos].isAssignableFrom(types[paramPos]))
          { break; }
        }
      if (paramPos == types.length)
        { myObject = choices[pos].newInstance(values); }
      }

    // This will throw a MethodNotFound exception since no Constructor exists.
    if (myObject == null)
      { myObject = Class.forName(myClassname).getConstructor(types).newInstance(values); }

    return myObject;
    }

  }

