//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: HashMapParser.java,v 1.3 2002/10/30 17:33:11 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: HashMapParser.java,v $
//  Revision 1.3  2002/10/30 17:33:11  cvs
//  Added the files relevant to ANTLR to the cvs.
//  --Murthy.
//
//  Revision 1.2  2002/09/11 08:28:50  josephk
//  Fixed the VectorMapParser to use java arrays instead of BaseObject.ListEntry.
//  Fixed the ArrayMapParser to use it's own class instead of BaseObject.ListEntry.
//  Added ArrayMapParser.ArrayListEntry to the default MapParser population.
//  Added a depth register when debugging the MapParser.
//  Cleaned up HashMapParser slightly.
//
//  Revision 1.1  2002/08/27 07:15:25  josephk
//  Added the new implimentation of the Mapping tools.
//
//
//=============================================================================
package org.sldl.rosetta.parser.tarp.map;

import java.util.HashMap;
import java.util.Map;


/** This represents the <B>HashMapParser</B> object. This object
 *  provides the ability to construct/deconstruct Maps.
 */
public class HashMapParser implements MapObjectDeconstructor, MapObjectReconstructor
  {
  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { };

  /** Lists the keys used to encode/decode maps.
   */
  public static final String[] hashMapKeySet = { "Keys", "Contents" };


  protected static final String keyKey = MapParser.keyKey;


  /** Method to get the HashMap key.
   * @return String The key that corresponds to HashMap deconstructors/reconstructors.
   */
  public static String getHashMapClassName()
    { return "java.util.HashMap"; }




  /** Primary Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public HashMapParser(Map settings)
    { }


  /** Method to check if this deconstructor can handle a given object.
   * @param target The object to deconstruct.
   * @return boolean Can this deconstructor handle that object?
   */
  public boolean isValidDeconstructor(Object target)
    { return (target instanceof HashMap); }


  /** Method to perform the deconstruction.
   * @param object The object to be deconstructed.
   * @return Map The map containing the contents of the object.
   * @throws Exception If an error occurs it should be propogated.
   */
  public Map deconstruct(Object object) throws Exception
    {
    Object[] keys;
    Map result;

    if (!isValidDeconstructor(object))
      { return null; }

    result = new HashMap();
    result.put(keyKey, getHashMapClassName());
    keys = ((HashMap)object).keySet().toArray();
    result.put(hashMapKeySet[0], keys);
    result.put(hashMapKeySet[1], getValues((HashMap)object, keys));

    return result;
    }




  /** Method to check if this reconstructor can handle a given map.
   * @param target The map to reconstruct.
   * @return boolean Can this reconstructor handle that map?
   */
  public boolean isValidReconstructor(Map target)
    {
    if ((target == null) || !(target.get(keyKey) instanceof String))
      { return false; }
    return getHashMapClassName().equals((String)target.get(keyKey));
    }


  /** Method to perform the reconstruction.
   * @param map The map containing the contents of the object.
   * @return Object The reconstructed object.
   * @throws Exception If an error occurs it should be propogated.
   */
  public Object reconstruct(Map map) throws Exception
    {
    int pos;
    HashMap result;
    Object[] keys;
    Object[] contents;

    if (!isValidReconstructor(map))
      { return null; }

    result = new HashMap();
    keys = (Object[])map.get(hashMapKeySet[0]);
    contents = (Object[])map.get(hashMapKeySet[1]);
    for (pos = 0; (keys != null) && (pos < keys.length); pos++)
      {
      if ((contents != null) && (pos < contents.length))
        { result.put(keys[pos], contents[pos]); }
      else
        { result.put(keys[pos], null); }
      }
    return result;
    }


  /** Method to extract an ordered list of values from a map.
   * @param object The map to list keys for.
   * @param keys The keys to use to order the list of values.
   * @return Object[] The value-list for this map ordered by the keys.
   */
  public static Object[] getValues(Map object, Object[] keys)
    {
    int pos;
    Object[] result;

    if ((object == null) || (keys == null))
      { return null; }

    result = new Object[keys.length];
    for (pos = 0; pos < keys.length; pos++)
      { result[pos] = object.get(keys[pos]); }
    return result;
    }

  }

