//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: MapParser.java,v 1.3 2002/10/30 17:33:11 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: MapParser.java,v $
//  Revision 1.3  2002/10/30 17:33:11  cvs
//  Added the files relevant to ANTLR to the cvs.
//  --Murthy.
//
//  Revision 1.3  2002/09/12 03:40:38  josephk
//  Re-implemented the ArrayMapParser to handle Arrays in a more efficient manner.
//
//  Revision 1.2  2002/09/11 08:28:50  josephk
//  Fixed the VectorMapParser to use java arrays instead of BaseObject.ListEntry.
//  Fixed the ArrayMapParser to use it's own class instead of BaseObject.ListEntry.
//  Added ArrayMapParser.ArrayListEntry to the default MapParser population.
//  Added a depth register when debugging the MapParser.
//  Cleaned up HashMapParser slightly.
//
//  Revision 1.1  2002/08/27 07:15:26  josephk
//  Added the new implimentation of the Mapping tools.
//
//
//=============================================================================
package org.sldl.rosetta.parser.tarp.map;

import java.io.IOException;
import java.io.Serializable;
import java.io.Writer;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Vector;

import org.sldl.rosetta.ROM.BaseObjectList.ListEntry;


/** This represents the <B>MapParser</B> class. The goal of this class
 *  is to provide a way to represent all ROM objects as maps from labels
 *  to children values. In particular, the goal is that by implementing this
 *  interface certain tools (such as an XML interchange file) can be used
 *  regardless of future changes to the ROM definition.
 */
public class MapParser implements Serializable
  {
  /** This is the key to be used when resolving a map to a class constructor.
   */
  public static final String keyKey = "ClassName";

  /** This is the key to be used when searching an object for the names
   *  of methods to interrogate to extract the contents of that object.
   *  Each interface of an object will be checked for a String[] of this
   *  name and those names will be invoked in a ["get"+name (<void>)] call
   *  to add to the Map.<BR>
   *  <B>This should not be overloaded in any way.</B>
   */
  public static final String fieldKey = "keySet";

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "Deconstructors", "Reconstructors", "IsDebugOn", "DebugWriter" };




  private Boolean myIsDebugOn;
  private Writer myDebugWriter;
  private int debugDepth;
  private List myDeconstructors, myReconstructors;


  /** Primary Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public MapParser(Map settings)
    {
    Map mySettings;

    if (settings == null)
      { mySettings = new HashMap(); }
    else
      { mySettings = settings; }

    debugDepth = 0;
    setDebug((Boolean)mySettings.get(keySet[2]), (Writer)mySettings.get(keySet[3]));
    setDeconstructors((List)mySettings.get(keySet[0]));
    setReconstructors((List)mySettings.get(keySet[1]));
    }

  /** Secondary Constructor.
   */
  public MapParser()
    {
    this(null);
    setDeconstructors(getDefaultMapObjectDeconstructors());
    setReconstructors(getDefaultMapObjectReconstructors());
    }




  /** Method to fetch the current debugging settings.
   * @return Boolean Is the MapParser set to print debug messages?
   */
  public Boolean getIsDebugOn()
    { return myIsDebugOn; }

  /** Method to fetch the current debugging settings.
   * @return Writer The destination Writer for debug info (can be null if false).
   */
  public Writer getDebugWriter()
    { return myDebugWriter; }

  /** Method to alter the amount of Debugging info printed and where to.
   * @param isDebugOn The new debug setting mode.
   * @param dest The destination Writer for debug info (can be null if false).
   * @return void
   */
  public void setDebug(Boolean isDebugOn, Writer dest)
    {
    myIsDebugOn = isDebugOn;
    myDebugWriter = dest;
    debug("MapFactory now printing debugging information."); 
    }

  /** Internal method to indicate a sub-call has been made.
   * @return void
   */
  protected void enterCall()
    { debugDepth++; }

  /** Internal method to indicate a sub-call has been terminated.
   * @return void
   */
  protected void leaveCall()
    { debugDepth--; }

  /** Internal method to cleanly handle debugging information.
   * @param msg The string to print to the Writer.
   * @return void
   */
  protected void debug(String msg)
    {
    if ((myIsDebugOn != null) && myIsDebugOn.booleanValue())
      {
      try
        {
        myDebugWriter.write("#" + debugDepth + " :: " + msg + "\n");
        myDebugWriter.flush();
        }
      catch(IOException e)
        { System.err.println("MapParser Writer error on [" + msg + "]: " + e); }
      }
    }

  /** Internal method to debug List information.
   * @param msg The string to print to explain the List.
   * @param list The list whose contents to print.
   * @return void
   */
  protected void debugList(String msg, List list)
    {
    int pos;

    debug(msg);
    for (pos = 0; pos < list.size(); pos++)
      { debug("  #" + pos + " = [" + list.get(pos) + "]"); }
    }

  /** Simple method to SHALLOW copy the contents of a list into a new Vector.
   * @param oldList The list to copy.
   * @return List The copied list, empty if oldList was null.
   */
  protected static List copyList(List oldList)
    {
    int pos;
    Vector result;

    result = new Vector();
    for (pos = 0; (oldList != null) && (pos < oldList.size()); pos++)
      { result.add(oldList.get(pos)); }
    return result;
    }




  /** Method to specify another object deconstructor to use.
   * @param deconstructor The object deconstructor to add to the list.
   * @return void
   */
  public void addDeconstructor(MapObjectDeconstructor deconstructor)
    {
    debug("Adding deconstructor #" + myDeconstructors.size() + " [" + deconstructor + "].");
    myDeconstructors.add(deconstructor);
    }

  /** Method to remove an object deconstructor from use.
   * @param pos The position of the object deconstructor to remove.
   * @return MapObjectDeconstructor The object deconstructor removed from the list.
   */
  public MapObjectDeconstructor removeDeconstructor(int pos)
    {
    debug("Removing deconstructor #" + pos + " [" + myDeconstructors.get(pos) + "].");
    return (MapObjectDeconstructor)(myDeconstructors.remove(pos));
    }
  
  /** Method to get the current list of object deconstructors being used.
   *  This list should be a seperate copy, so altering it does not
   *  affect the MapFactory.
   * @return List The list of map deconstructors being used.
   */
  public List getDeconstructors()
    { return copyList(myDeconstructors); }

  /** Method to set the current list of object deconstructors being used.
   *  The contents of the list will be copied, rather than the list itself.
   * @param deconstructors The list of map deconstructors to be used.
   * @return void
   */
  public void setDeconstructors(List deconstructors)
    { myDeconstructors = copyList(deconstructors); }






  /** Method to specify another object reconstructor to use.
   * @param reconstructor The object reconstructor to add to the list.
   * @return void
   */
  public void addMapObjectReconstructor(MapObjectReconstructor reconstructor)
    {
    debug("Adding reconstructor #" + myReconstructors.size() + " [" + reconstructor + "].");
    myReconstructors.add(reconstructor);
    }

  /** Method to remove an object reconstructor from use.
   * @param pos The position of the object reconstructor to remove.
   * @return MapObjectReconstructor The object reconstructor removed from the list.
   */
  public MapObjectReconstructor removeMapObjectReconstructor(int pos)
    {
    debug("Removing reconstructor #" + pos + " [" + myReconstructors.get(pos) + "].");
    return (MapObjectReconstructor)(myReconstructors.remove(pos));
    }

  /** Method to get the current list of object reconstructors being used.
   *  This list should be a seperate copy, so altering it does not
   *  affect the MapFactory.
   * @return List The list of map reconstructors being used.
   */
  public List getMapObjectReconstructors()
    { return copyList(myReconstructors); }

  /** Method to set the current list of object reconstructors being used.
   *  The contents of the list will be copied, rather than the list itself.
   * @param reconstructors The list of map weconstructors to be used.
   * @return void
   */
  public void setReconstructors(List reconstructors)
    { myReconstructors = copyList(reconstructors); }








  /** Method to extract a map from an Object.
   * @param node The object to attempt to abstract as a map.
   * @return Map The map of the object.
   * @throws Exception If an error occurs it should be propogated.
   */
  public Map getMap(Object node) throws Exception
    {
    int pos;
    MapObjectDeconstructor nextMapObjectDeconstructor;
    Map result;

    enterCall();

    if (node == null)
      {
      debug("Method getMap called on null value.");
      leaveCall();
      return null;
      }
    for (pos = 0; pos < myDeconstructors.size(); pos++)
      {
      try
        { nextMapObjectDeconstructor = (MapObjectDeconstructor)(myDeconstructors.get(pos)); }
      catch(ClassCastException e)
        {
        debug("MapObjectDeconstructors contains an unknown object in it [" + myDeconstructors.get(pos) + "]!");
        continue;
        }
      if ((nextMapObjectDeconstructor != null) && nextMapObjectDeconstructor.isValidDeconstructor(node))
        {
        debug("Matching deconstructor [" + nextMapObjectDeconstructor + "] found for [" + node + "].");
        result = nextMapObjectDeconstructor.deconstruct(node);
        debug("Deconstruction finished: [" + node + "] -> [" + result + "].");
        leaveCall();
        return result;
        }
      }

    debug("No deconstructor found for [" + node + "].");
    leaveCall();
    return null;
    }

  /** Method to extract a complete map from an Object, with all of its
   *  children maps, etc... In particular, any cycles that occur should
   *  be detected and mapped into cycles in the Mapping structure.
   * @param node The object to attempt to abstract as a map.
   * @return Map The map of the object.
   * @throws Exception If an error occurs it should be propogated.
   */
  public Map getCompleteMap(Object node) throws Exception
    {
    Map object2Key, key2Value;
    Vector pendingNodes, mapList;

    enterCall();
    if (node == null)
      {
      debug("Method getCompleteMap called on null value.");
      leaveCall();
      return null;
      }
    debug("Method getCompleteMap called on [" + node + "].");

    object2Key = new HashMap();
    object2Key.put(node, new Integer(object2Key.size()));
    key2Value = new HashMap();
    pendingNodes = new Vector();
    pendingNodes.add(node);
    mapList = new Vector();

    while (pendingNodes.size() > 0)
      { findNextMapping(pendingNodes, object2Key, key2Value, mapList); }

    fixNodes(mapList, key2Value);
    debug("getCompleteMap finished tasks for [" + node + "].");
    leaveCall();
    return (Map)(key2Value.get(object2Key.get(node)));
    }

  /** Method to complete a mapping for the next unmapped node.
   * @param pendingNodes The nodes which have not yet been expanded.
   * @param object2Key A mapping from each object to their intermediate key value.
   * @param key2Value A mapping from each intermediate value to its final value.
   * @param mapList A list of Maps which need their entry values de-curried.
   * @return void
   * @throws Exception If an error occurs it should be propogated.
   */
  protected void findNextMapping(Vector pendingNodes, Map object2Key, Map key2Value, Vector mapList) throws Exception
    {
    int pos;
    Object nextNode, nextChild;
    Map referenceMap, actualMap;
    Object[] keyEntries;

    enterCall();
    nextNode = pendingNodes.remove(0);
    debug("Now finding Map for [" + nextNode + "].");

    actualMap = getMap(nextNode);
    if (actualMap == null)
      {
      debug("Mapping failed for [" + nextNode + "]");
      key2Value.put(object2Key.get(nextNode), nextNode);
      leaveCall();
      return;
      }

    debug("Now translating [" + nextNode + "]'s object Map [" + actualMap + "] into a reference Map, object2Key size = " + object2Key.size() + ".");
    referenceMap = new HashMap();
    mapList.add(referenceMap);
    key2Value.put(object2Key.get(nextNode), referenceMap);
    keyEntries = actualMap.keySet().toArray();
    for (pos = 0; pos < keyEntries.length; pos++)
      {
      nextChild = actualMap.get(keyEntries[pos]);
      if (!object2Key.containsKey(nextChild))
        {
        object2Key.put(nextChild, new Integer(object2Key.size()));
        pendingNodes.add(nextChild);
        }
      referenceMap.put(keyEntries[pos], object2Key.get(nextChild));
      }
    debug("Finished translating [" + nextNode + "]'s object Map [" + actualMap + "] into a reference Map [" + referenceMap + "], object2Key size = " + object2Key.size() + ".");
    leaveCall();
    }

  /** Method to find links to nodes that did not produce maps (eg. Strings)
   *  and change the links from the empty maps to the original unmapped object.
   * @param mapList A list of Maps which need their entry values de-curried.
   * @param key2Value A mapping from each intermediate value to its final value.
   * @return void
   */
  protected void fixNodes(Vector mapList, Map key2Value)
    {
    int mapPos, entryPos;
    Map nextMap;
    Object[] keyEntries;

    enterCall();
    for (mapPos = 0; mapPos < mapList.size(); mapPos++)
      {
      nextMap = (Map)(mapList.get(mapPos));
      debug("Now fixing links to reference map [" + nextMap + "].");
      keyEntries = nextMap.keySet().toArray();
      for (entryPos = 0; entryPos < keyEntries.length; entryPos++)
        { nextMap.put(keyEntries[entryPos], key2Value.get(nextMap.get(keyEntries[entryPos]))); }
      debug("Finished fixing links to reference map [" + nextMap + "].");
      }
    leaveCall();
    }







  /** Method to use a map to generate an Object. If no reconstructor
   *  matching the class of the map can be found then NULL will be returned.
   * @param map The map of the object.
   * @return Object The object to represented by the map.
   * @throws Exception If an error occurs it should be propogated.
   */
  public Object getObject(Map map) throws Exception
    {
    int pos;
    MapObjectReconstructor nextReconstructor;
    Object result;

    enterCall();
    if (map == null)
      {
      debug("Method getObject called on null value.");
      leaveCall();
      return null;
      }
    for (pos = 0; pos < myReconstructors.size(); pos++)
      {
      try
        { nextReconstructor = (MapObjectReconstructor)(myReconstructors.get(pos)); }
      catch(ClassCastException e)
        {
        debug("MapObjectReconstructors contains an unknown object in it [" + myReconstructors.get(pos) + "]!");
        continue;
        }
      if ((nextReconstructor != null) && nextReconstructor.isValidReconstructor(map))
        {
        debug("Matching reconstructor [" + nextReconstructor + "] found for [" + map + "].");
        result = nextReconstructor.reconstruct(map);
        debug("Reconstruction finished: [" + map + "] -> [" + result + "].");
        leaveCall();
        return result;
        }
      }
    debug("No reconstructor found for [" + map + "].");
    leaveCall();
    return null;
    }


  /** Method to use a map to generate an Object. If no reconstructor
   *  matching the class of the map can be found then NULL will be returned.
   * @param map The map of the object, with children as maps also.
   * @return Object The object to represented by the map.   
   * @throws Exception If anything goes wrong that error is propogated.
   */
  public Object getCompleteObject(Map map) throws Exception
    {
    Map finishedObjects;

    enterCall();
    if (map == null)
      {
      debug("Method getCompleteObject called on null value.");
      leaveCall();
      return null;
      }
    debug("Method getCompleteObject called on [" + map + "].");

    finishedObjects = new HashMap();
    considerNextMap(map, finishedObjects);
    leaveCall();
    return finishedObjects.get(map);
    }

  /** Method to rebuild a Map, by recursively rebuilding it's children.
   * @param nextMap The next map to be rebuilt.
   * @param finishedObjects The current mappings of rebuilt objects.
   * @return void
   * @throws Exception If anything goes wrong that error is propogated.
   */
  protected void considerNextMap(Map nextMap, Map finishedObjects) throws Exception
    {
    int pos;
    Map buildMap;
    Object[] keyEntries;
    Object result;

    enterCall();
    if (finishedObjects.containsKey(nextMap))
      { leaveCall(); return; }
    finishedObjects.put(nextMap, null);

    debug("Now entering [" + nextMap + "] for rebuilding.");
    buildMap = new HashMap();
    keyEntries = nextMap.keySet().toArray();
    for (pos = 0; pos < keyEntries.length; pos++)
      {
      if (nextMap.get(keyEntries[pos]) instanceof Map)
        {
        considerNextMap((Map)(nextMap.get(keyEntries[pos])), finishedObjects);
        buildMap.put(keyEntries[pos], finishedObjects.get(nextMap.get(keyEntries[pos])));
        }
      else
        { buildMap.put(keyEntries[pos], nextMap.get(keyEntries[pos])); }
      }

    result = getObject(buildMap);
    if (result == null)
      {
      debug("Reconstruction failed for [" + buildMap + "] - leaving its original value unchanged.");
      finishedObjects.put(nextMap, nextMap);
      }
    else
      {
      debug("Reconstruction complete for [" + nextMap + "] - replacing with value [" + result + "].");
      finishedObjects.put(nextMap, result);
      }
    leaveCall();
    }




  /** Method to build an object without having to first build a map.
   *  This method simply creates a map with the given (key, value) pairs
   *  and calls the <I>getObject</I> method.
   * @param name The name to use as a key for the reconstructor.
   * @param keys The key names of the parameters.
   * @param params The parameters to use when building the object.
   * @throws Exception If anything goes wrong that error is propogated.
   */
  public Object build(String name, String[] keys, Object[] params) throws Exception
    {
    Map myMap;
    int pos;
    Object result;

    enterCall();
    if (name == null)
      {
      debug("Method build called with null name, keys = [" + keys + "] and params = [" + params + "].");
      leaveCall();
      return null;
      }
    myMap = buildMap(keys, params);
    myMap.put(keyKey, name);

    debug("Method build calling getObject on [" + myMap + "].");
    result = getObject(myMap);
    debug("Method build called getObject on [" + myMap + "] and received [" + result + "].");
    leaveCall();
    return result;
    }



  /** Method to construct a Map object on-the-fly.
   * @param keys The keys of each entry.
   * @param params The values of each entry.
   * @return Map A map formed with the given keys and values.
   */
  public static Map buildMap(Object[] keys, Object[] params)
    {
    int pos;
    Map myMap;

    myMap = new HashMap();
    for (pos = 0; (keys != null) && (pos < keys.length); pos++)
      {
      if ((params != null) && (pos < params.length))
        { myMap.put(keys[pos], params[pos]); }
      else
        { myMap.put(keys[pos], null); }
      }

    return myMap;
    }





  /** Method to form a list of all the MapObjectDeconstructor objects
   *  for the standard ROM implementation.
   * @return List The MapObjectDeconstructor list.
   */
  public static List getDefaultMapObjectDeconstructors()
    {
    Vector result;

    result = new Vector();
    result.add(new HashMapParser(null));
    result.add(new VectorMapParser(null));
    result.add(new ArrayMapParser(null));
    result.add(new DefaultMapObjectDeconstructor(null));
    return result;
    }

  /** Method to form a list of all the MapObjectReconstructor objects
   *  for the standard ROM implementation.
   * @return List The MapObjectReconstructor list.
   */
  public static List getDefaultMapObjectReconstructors()
    {
    Vector result;

    result = new Vector();
    result.add(new HashMapParser(null));
    result.add(new VectorMapParser(null));
    result.add(new ArrayMapParser(null));
    result.add(new DefaultMapObjectReconstructor(buildMap(DefaultMapObjectReconstructor.keySet, new Object[] { defaultReconstructorClasses })));
    result.add(new DefaultMapObjectReconstructor(buildMap(DefaultMapObjectReconstructor.keySet, new Object[] { extraReconstructorClasses })));
    return result;
    }


  /** Default list of classes that can be reconstructed automatically
   *  by invoking getClass().getConstructor([Map.getClass()]).
   */
  public static final String[] defaultReconstructorClasses = {
      "org.sldl.rosetta.ROM.BaseObjectList$ListEntry",
      "org.sldl.rosetta.ROM.ExpressionList",
      "org.sldl.rosetta.ROM.LabelList",
      "org.sldl.rosetta.ROM.ItemList",

      "org.sldl.rosetta.ROM.Item",

      "org.sldl.rosetta.ROM.Label",
      "org.sldl.rosetta.ROM.CharacterLiteral",
      "org.sldl.rosetta.ROM.NumberLiteral",
      "org.sldl.rosetta.ROM.StringLiteral",

      "org.sldl.rosetta.ROM.ArrayValue",
      "org.sldl.rosetta.ROM.ConstructedValue",
      "org.sldl.rosetta.ROM.EnumerationType",
      "org.sldl.rosetta.ROM.SetValue",

      "org.sldl.rosetta.ROM.Application",
      "org.sldl.rosetta.ROM.CaseExpression",
      "org.sldl.rosetta.ROM.ConstructedType",
      "org.sldl.rosetta.ROM.ConstructedType$ConstructedTypeConstructor",
      "org.sldl.rosetta.ROM.Function",
      "org.sldl.rosetta.ROM.IfExpression",
      "org.sldl.rosetta.ROM.ItemReference",
      "org.sldl.rosetta.ROM.LetFunction",
      "org.sldl.rosetta.ROM.QuantifierReference",
      "org.sldl.rosetta.ROM.TypeExpression",

      "org.sldl.rosetta.ROM.Component",
      "org.sldl.rosetta.ROM.Domain",
      "org.sldl.rosetta.ROM.Facet",
      "org.sldl.rosetta.ROM.Interaction",
      "org.sldl.rosetta.ROM.Package"
    };


  /** Extra list of classes that can be reconstructed automatically
   *  by invoking getClass().getConstructor([Map.getClass()]).
   */
  public static final String[] extraReconstructorClasses = {
      "org.sldl.rosetta.parser.tarp.misc.SourceReference",
      "org.sldl.rosetta.parser.tarp.misc.ROMCreator",
      "org.sldl.rosetta.parser.tarp.map.ArrayMapParser",
      "org.sldl.rosetta.parser.tarp.map.DefaultMapObjectDeconstructor",
      "org.sldl.rosetta.parser.tarp.map.DefaultMapObjectReconstructor",
      "org.sldl.rosetta.parser.tarp.map.HashMapParser",
      "org.sldl.rosetta.parser.tarp.map.VectorMapParser",
      "org.sldl.rosetta.parser.tarp.map.MapParser"

    };
  }

