//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: VectorMapParser.java,v 1.3 2002/10/30 17:33:11 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: VectorMapParser.java,v $
//  Revision 1.3  2002/10/30 17:33:11  cvs
//  Added the files relevant to ANTLR to the cvs.
//  --Murthy.
//
//  Revision 1.2  2002/09/11 08:28:51  josephk
//  Fixed the VectorMapParser to use java arrays instead of BaseObject.ListEntry.
//  Fixed the ArrayMapParser to use it's own class instead of BaseObject.ListEntry.
//  Added ArrayMapParser.ArrayListEntry to the default MapParser population.
//  Added a depth register when debugging the MapParser.
//  Cleaned up HashMapParser slightly.
//
//  Revision 1.1  2002/08/27 07:15:26  josephk
//  Added the new implimentation of the Mapping tools.
//
//
//=============================================================================
package org.sldl.rosetta.parser.tarp.map;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;


/** This represents the <B>VectorMapParser</B> object. This object
 *  provides the ability to construct/deconstruct Vectors.
 */
public class VectorMapParser implements MapObjectDeconstructor, MapObjectReconstructor
  {
  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { };

  /** Lists the keys used to encode/decode vectors.
   */
  public static final String[] vectorKeySet = { "Contents" };


  protected static final String keyKey = MapParser.keyKey;


  /** Method to get the Vector key.
   * @return String The key that corresponds to Vector deconstructors/reconstructors.
   */
  public static String getVectorClassName()
    { return "java.util.Vector"; }




  /** Primary Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public VectorMapParser(Map settings)
    { }


  /** Method to check if this deconstructor can handle a given object.
   * @param target The object to deconstruct.
   * @return boolean Can this deconstructor handle that object?
   */
  public boolean isValidDeconstructor(Object target)
    { return (target instanceof Vector); }


  /** Method to perform the deconstruction.
   * @param object The object to be deconstructed.
   * @return Map The map containing the contents of the object.
   * @throws Exception If an error occurs it should be propogated.
   */
  public Map deconstruct(Object object) throws Exception
    {
    Map result;

    if (!isValidDeconstructor(object))
      { return null; }

    result = new HashMap();
    result.put(keyKey, getVectorClassName());
    result.put(vectorKeySet[0], convertList((List)object));

    return result;
    }




  /** Method to check if this reconstructor can handle a given map.
   * @param target The map to reconstruct.
   * @return boolean Can this reconstructor handle that map?
   */
  public boolean isValidReconstructor(Map target)
    {
    if ((target == null) || !(target.get(keyKey) instanceof String))
      { return false; }
    return getVectorClassName().equals((String)target.get(keyKey));
    }


  /** Method to perform the reconstruction.
   * @param map The map containing the contents of the object.
   * @return Object The reconstructed object.
   * @throws Exception If an error occurs it should be propogated.
   */
  public Object reconstruct(Map map) throws Exception
    {
    int pos;
    Vector result;
    Object[] contents;

    if (!isValidReconstructor(map))
      { return null; }

    result = new Vector();
    contents = (Object[])(map.get(vectorKeySet[0]));
    for (pos = 0; (contents != null) && (pos < contents.length); pos++)
      { result.add(contents[pos]); }
    return result;
    }


  /** Method to convert a List into an array structure.
   * @param list The list to convert.
   * @return Object[] The converted list.
   */
  public static Object[] convertList(List list)
    {
    int pos;
    Object[] contents;

    if (list == null)
      { return null; }
    contents = new Object[list.size()];
    for (pos = 0; pos < contents.length; pos++)
      { contents[pos] = list.get(pos); }
    return contents;
    }

  }

