//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: Duplicator.java,v 1.5 2002/11/21 05:34:13 josephk Exp $
//
//  Revision History:
//  ----------------
//  $Log: Duplicator.java,v $
//  Revision 1.5  2002/11/21 05:34:13  josephk
//  Updating KU ANTLR mirror.
//
//  Revision 1.1  2002/08/27 07:16:07  josephk
//  Moved the various 'one-off' files to a common misc directory.
//
//
//=============================================================================
package org.sldl.rosetta.parser.tarp.misc;

import java.io.Serializable;
import org.sldl.rosetta.parser.tarp.map.MapParser;
import org.sldl.rosetta.ROM.BaseObject;
import org.sldl.rosetta.ROM.BaseObjectList;
import org.sldl.rosetta.ROM.Expression;
import org.sldl.rosetta.ROM.ExpressionList;
import org.sldl.rosetta.ROM.Item;
import org.sldl.rosetta.ROM.ItemList;
import org.sldl.rosetta.ROM.ItemReference;
import org.sldl.rosetta.ROM.Label;
import org.sldl.rosetta.ROM.LabelList;



/** This is the <B>Duplicator</B> class which provides an easy set of
 *  methods for quickly making a deep but non-linking copy of a ROM tree.
 */
public class Duplicator implements Serializable
  {
  private MapParser myMapParser;



  /** Primary Constructor.
   * @param mapParser The MapParser to use to create new ROM pieces.
   */
  public Duplicator(MapParser mapParser)
    { myMapParser = mapParser; }



  /** Method to retreive the MapParser used to create new ROM pieces.
   * @return MapParser The MapParser being used.
   */
  public MapParser getMapParser()
    { return myMapParser; }

  /** Method for setting the MapParser to use to create new ROM pieces.
   * @param mapParser The new MapParser to use.
   * @return void.
   */
  public void setMapParser(MapParser mapParser)
    { myMapParser = mapParser; }






  /** Method to try and duplicate a generic object that is
   *  recognised by the MapParser this Duplicator is using.
   *  This method should be used when a ROM implementation
   *  contains a non-standard decoration that is not a ROM object.
   * @param node The object to be duplicated.
   * @return Object The duplicated object.
   * @throws Exception If anything goes wrong in the duplication process.
   */
  public Object duplicateObject(Object node) throws Exception
    { return myMapParser.getCompleteObject(myMapParser.getCompleteMap(node)); }


  /** Method to try and duplicate a generic object without throwing an Exception.
   * @param node The object to be duplicated.
   * @return Object The duplicated object.
   */
  protected Object duplicateObjectQuietly(Object node)
    {
    try
      { return duplicateObject(node); }
    catch(Exception e)
      { return null; }
    }



  /** Method for duplicating a SourceReference.
   * @param node The node of the ROM tree being duplicated.
   * @return SourceReference The new duplicate version.
   */
  public SourceReference duplicateSourceReference(SourceReference node)
    { return (SourceReference)duplicateObjectQuietly(node); }

  /** Method for duplicating a BaseObjectList.
   * @param node The node of the ROM tree being duplicated.
   * @return BaseObjectList The new duplicate version.
   */
  public BaseObjectList duplicateBaseObjectList(BaseObjectList node)
    { return (BaseObjectList)duplicateObjectQuietly(node); }

  /** Method for duplicating an ItemList.
   * @param node The node of the ROM tree being duplicated.
   * @return ItemList The new duplicate version.
   */
  public ItemList duplicateItemList(ItemList node)
    { return (ItemList)duplicateObjectQuietly(node); }

  /** Method for duplicating a LabelList.
   * @param node The node of the ROM tree being duplicated.
   * @return LabelList The new duplicate version.
   */
  public LabelList duplicateLabelList(LabelList node)
    { return (LabelList)duplicateObjectQuietly(node); }

  /** Method for duplicating an ExpressionList.
   * @param node The node of the ROM tree being duplicated.
   * @return ExpressionList The new duplicate version.
   */
  public ExpressionList duplicateExpressionList(ExpressionList node)
    { return (ExpressionList)duplicateObjectQuietly(node); }

  /** Method for duplicating an BaseObject.
   * @param node The node of the ROM tree being duplicated.
   * @return BaseObject The new duplicate version.
   */
  public BaseObject duplicateBaseObject(BaseObject node)
    { return (BaseObject)duplicateObjectQuietly(node); }

  /** Method for duplicating an Item.
   * @param node The node of the ROM tree being duplicated.
   * @return Item The new duplicate version.
   */
  public Item duplicateItem(Item node)
    { return (Item)duplicateObjectQuietly(node); }

  /** Method for duplicating an Expression.
   * @param node The node of the ROM tree being duplicated.
   * @return Expression The new duplicate version.
   */
  public Expression duplicateExpression(Expression node)
    { return (Expression)duplicateObjectQuietly(node); }

  /** Method for duplicating a Label.
   * @param node The node of the ROM tree being duplicated.
   * @return Label The new duplicate version.
   */
  public Label duplicateLabel(Label node)
    { return (Label)duplicateObjectQuietly(node); }

  /** Method for duplicating an ItemReference.
   * @param node The node of the ROM tree being duplicated.
   * @return ItemReference The new duplicate version.
   */
  public ItemReference duplicateItemReference(ItemReference node)
    { return (ItemReference)duplicateObjectQuietly(node); }

  }


