//============================================================================
//  Copyright (c) 2002 University of Adelaide
//  All rights reserved.
//
//  UoA MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. UoA SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: ROMCreator.java,v 1.6 2002/11/21 05:34:13 josephk Exp $
//
//  Revision History:
//  ----------------
//  $Log: ROMCreator.java,v $
//  Revision 1.6  2002/11/21 05:34:13  josephk
//  Updating KU ANTLR mirror.
//
//  Revision 1.10  2002/11/18 04:32:11  josephk
//  Altered the ConstructedType method to produce a dummy ItemList until the
//  ROM class can be more cleanly defined.
//
//  Revision 1.9  2002/11/18 03:10:41  josephk
//  Altered the ConstructedTypeConstructor method to produce a dummy ItemReference
//  and a dummy Item until the ROM class can be more cleanly defined.
//
//  Revision 1.8  2002/11/18 03:08:55  josephk
//  Extended the signature for ItemReference to take the original string used
//  to create the reference as well as the LabelList it turns into.
//
//  Revision 1.7  2002/11/14 12:11:06  josephk
//  Changed the ItemReference to create a the local name and the original
//  reference string that produced the reference.
//
//  Revision 1.6  2002/09/11 08:33:57  josephk
//  Added another Component build method.
//
//  Revision 1.5  2002/09/11 01:23:44  josephk
//  Changed the TypeExpression and Component build routines to the new ROM.
//
//  Revision 1.4  2002/09/09 08:59:31  josephk
//  Fixed a bug in the PackageSignature. Removed ConstructedValue methods.
//
//  Revision 1.3  2002/09/09 04:35:40  josephk
//  Fixed the function call for buildIfExpression.
//
//  Revision 1.2  2002/09/06 06:00:22  josephk
//  Updated some problems in the Creator routines.
//
//  Revision 1.1  2002/08/27 07:16:07  josephk
//  Moved the various 'one-off' files to a common misc directory.
//
//
//=============================================================================
package org.sldl.rosetta.parser.tarp.misc;

import java.lang.reflect.InvocationTargetException;
import java.util.Map;
import java.util.HashMap;
import org.sldl.rosetta.ROM.*;
import org.sldl.rosetta.ROM.Package;
import org.sldl.rosetta.ROM.ConstructedType.ConstructedTypeConstructor;


/** This is a wrapper class for making it easier to call the large
 *  number of constructors a parser will call when building a ROM,
 *  such that changes in the ROM need only be updated here.
 */
public class ROMCreator implements ConstructorSignatures
  {
  /** Primary Constructor.
   */
  public ROMCreator()
    {
    }



  /** Master function - wraps each build call in exception handling code.
   * @param ClassName The name of the class of object to build.
   * @param signature The keynames of each parameter of the object class constructor.
   * @param values The values to initialise the object with.
   * @return Object The built object, or null if an error.
   */
  public static Object buildObject(String ClassName, String[] signature, Object[] values)
    {
    try
      {
      int pos;
      Class targetClass;
      Class[] constructorSignature;
      Map constructorArgument;

      constructorArgument = new HashMap();
      for (pos = 0; pos < signature.length; pos++)
        { constructorArgument.put(signature[pos], values[pos]); }
      targetClass = Class.forName(ClassName);
      constructorSignature = new Class[] { Class.forName("java.util.Map") };
      return targetClass.getConstructor(constructorSignature).newInstance(new Object[] { constructorArgument});
      }
    catch(InvocationTargetException e)
      {
      System.err.println("Wrappered ROMCreator Error [" + ClassName + "]: " + e + " --> " + e.getTargetException());
      e.getTargetException().printStackTrace();
      return null;
      }
    catch(Exception e)
      {
      System.err.println("ROMCreator Error [" + ClassName + "]: " + e);
      e.printStackTrace();
      return null;
      }
    }

  /** Master function - wraps each build call in exception handling code.
   * @param ClassName The name of the class of object to build.
   * @param signature The keynames of each parameter of the object class constructor.
   * @param values The values to initialise the object with.
   * @return Object The built object, or null if an error.
   */
  public Object build(String ClassName, String[] signature, Object[] values)
    { return buildObject("org.sldl.rosetta.ROM." + ClassName, signature, values); }



  /** Make an empty List.
   * @return ExpressionList A new empty list.
   */
  public ExpressionList makeExpressionList()
    { return (ExpressionList)build("ExpressionList", ExpressionListSignature, new Object[] { null }); }

  /** Make an empty List.
   * @return ItemList A new empty list.
   */
  public ItemList makeItemList()
    { return (ItemList)build("ItemList", ItemListSignature, new Object[] { null }); }

  /** Make an empty List.
   * @return LabelList A new empty list.
   */
  public LabelList makeLabelList()
    { return (LabelList)build("LabelList", LabelListSignature, new Object[] { null }); }


  /** Create a new Label.
   * @param data The string value of this label.
   * @return Label The new Label.
   */
  public Label buildLabel(String data)
    { return (Label)build("Label", LabelSignature, new Object[] { data }); }

  /** Create a new CharacterLiteral.
   * @param data The character value of this literal.
   * @return CharacterLiteral The new CharacterLiteral.
   */
  public CharacterLiteral buildCharacterLiteral(Character data)
    { return (CharacterLiteral)build("CharacterLiteral", CharacterLiteralSignature, new Object[] { data }); }

  /** Create a new NumberLiteral.
   * @param data The numerical value of this literal.
   * @return NumberLiteral The new NumberLiteral.
   */
  public NumberLiteral buildNumberLiteral(Number data)
    { return (NumberLiteral)build("NumberLiteral", NumberLiteralSignature, new Object[] { data }); }

  /** Create a new StringLiteral.
   * @param data The string value of this literal.
   * @return StringLiteral The new StringLiteral.
   */
  public StringLiteral buildStringLiteral(String data)
    { return (StringLiteral)build("StringLiteral", StringLiteralSignature, new Object[] { data }); }



  /** Create a new ArrayValue.
   * @param contentList The initial contents.
   * @return ArrayValue The new ArrayValue.
   */
  public ArrayValue buildArrayValue(ExpressionList contentList)
    { return (ArrayValue)build("ArrayValue", ArrayValueSignature, new Object[] { contentList }); }

  /** Create a new EnumerationType.
   * @param contentList The initial contents.
   * @return EnumerationType The new EnumerationType.
   */
  public EnumerationType buildEnumerationType(ExpressionList contentList)
    { return (EnumerationType)build("EnumerationType", EnumerationTypeSignature, new Object[] { contentList }); }

  /** Create a new SetValue.
   * @param contentList The initial contents.
   * @return SetValue The new SetValue.
   */
  public SetValue buildSetValue(ExpressionList contentList)
    { return (SetValue)build("SetValue", SetValueSignature, new Object[] { contentList }); }





  /** Create a new ItemReference.
   * @param relativeName The relative qualified name of this reference.
   * @return ItemReference The new ItemReference.
   */
  public ItemReference buildItemReference(LabelList relativeName)
    { return (ItemReference)build("ItemReference", ItemReferenceSignature, new Object[] { relativeName, relativeName.toString(".") }); }


  /** Create a new QuantifierReference.
   * @param reference The name of this quantifier.
   * @param items The items which form the parameters of this Quantifier.
   * @param expr The predicate of this Quantifier.
   * @return QuantifierReference The new QuantifierReference.
   */
  public QuantifierReference buildQuantifierReference(ItemReference reference, ItemList items, Expression expr)
    { return (QuantifierReference)build("QuantifierReference", QuantifierReferenceSignature, new Object[] { reference, items, expr }); }


  /** Create a new Application.
   * @param left The lhs expression of the Application.
   * @param right The rhs expression of the Application.
   * @return Application The new Application.
   */
  public Application buildApplication(Expression left, ExpressionList right)
    { return (Application)build("Application", ApplicationSignature, new Object[] { left, right }); }



  /** Create a new Function.
   * @param params The parameter items of this function.
   * @param range The return range Expression of this function.
   * @param value The value of the Expression of this function.
   * @return Function The new Function.
   */
  public Function buildFunction(ItemList params, Expression range, Expression value)
    { return (Function)build("Function", FunctionSignature, new Object[] { params, range, value }); }

  /** Create a new LetFunction.
   * @param item The lambda-item of this function.
   * @param value The value of the Expression of this function.
   * @return LetFunction The new LetFunction.
   */
  public LetFunction buildLetFunction(Item item, Expression value)
    { return (LetFunction)build("LetFunction", LetFunctionSignature, new Object[] { item, value }); }

  /** Create a new CaseExpression.
   * @param testExpr The initial expression to evaluate.
   * @param tests The list of tests to match against.
   * @param results The list of results corresponding to each test.
   * @return CaseExpression The new CaseExpression.
   */
  public CaseExpression buildCaseExpression(Expression testExpr, ExpressionList tests, ExpressionList results)
    { return (CaseExpression)build("CaseExpression", CaseExpressionSignature, new Object[] { testExpr, tests, results }); }

  /** Create a new IfExpression.
   * @param ifs The if-Expression of the if-statement.
   * @param thens The then-Expression of the then-statements.
   * @param elseExpr The else-Expression (if one exists).
   * @return IfExpression The new IfExpression.
   */
  public IfExpression buildIfExpression(Expression ifs, Expression thens, Expression elseExpr)
    { return (IfExpression)build("IfExpression", IfExpressionSignature, new Object[] { ifs, thens, elseExpr }); }


  /** Create a new ConstructedType.
   * @param constructors The constructors for this ConstructedType.
   * @return ConstructedType The new ConstructedType.
   */
  public ConstructedType buildConstructedType(ExpressionList constructors)
    {
    int pos;
    ItemList myConstructors;

    myConstructors = makeItemList();
    for (pos = 0; pos < constructors.size(); pos++)
      { myConstructors.add(buildItem(((ConstructedTypeConstructor)constructors.get(pos)).getConstructorReference().getRelativeName().get(0), null, constructors.get(pos))); }
    return (ConstructedType)build("ConstructedType", ConstructedTypeSignature, new Object[] { null, myConstructors });
    }


  /** Create a new ConstructedTypeConstructor.
   * @param constructor The label be used when this constructor is called.
   * @param observers The observer functions to 'un-pack' these types of constructed values.
   * @param recognizer The label which recognizes this constructor.
   * @return ConstructedTypeConstructor The new ConstructedTypeConstructor.
   */
  public ConstructedTypeConstructor buildConstructedTypeConstructor(Label constructor, ItemList observers, Label recognizer)
    {
    LabelList myConstructorList;
    myConstructorList = makeLabelList();
    myConstructorList.add(constructor);
    return (ConstructedTypeConstructor)build("ConstructedType$ConstructedTypeConstructor", ConstructedTypeConstructorSignature, new Object[] { buildItemReference(myConstructorList), observers, buildItem(recognizer, null, null) });
    }




  /** Create a new Facet.
   * @param library The source library of this object.
   * @param uses The use clauses of this facet.
   * @param interfaces The interface items for this facet.
   * @param domain The domain of this facet.
   * @param declared The declared items of this facet.
   * @param exportAll Are all the items in this facet being exported explicitly.
   * @param exports The items being exported from this facet.
   * @param terms The term items of this facet.
   * @return Facet The new Facet.
   * @deprecated Need to start using the 'library' tag.
   */
  public Facet buildFacet(ExpressionList uses, ItemList interfaces, Expression domain, ItemList declared, Boolean exportAll, ExpressionList exports, ItemList terms)
    { return (Facet)build("Facet", FacetSignature, 
             new Object[] { uses, domain, interfaces, declared, terms, exports, exportAll }); }

  public Facet buildFacet(LabelList library, ExpressionList uses, ItemList interfaces, Expression domain, ItemList declared, Boolean exportAll, ExpressionList exports, ItemList terms)
    { return (Facet)build("Facet", FacetSignature,
             new Object[] { uses, domain, interfaces, declared, terms, exports, exportAll }); }

  /** Create a new Domain.
   * @param library The source library of this object.
   * @param uses The use clauses of this facet.
   * @param interfaces The interface items for this facet.
   * @param domain The domain of this facet.
   * @param declared The declared items of this facet.
   * @param exportAll Are all the items in this facet being exported explicitly.
   * @param exports The items being exported from this facet.
   * @param terms The term items of this facet.
   * @return Domain The new Domain.
   * @deprecated Need to start using the 'library' tag.
   */
  public Domain buildDomain(ExpressionList uses, ItemList interfaces, Expression domain, ItemList declared, Boolean exportAll, ExpressionList exports, ItemList terms)
    { return (Domain)build("Domain", DomainSignature,
             new Object[] { uses, domain, interfaces, declared, terms, exports, exportAll }); }

  public Domain buildDomain(LabelList library, ExpressionList uses, ItemList interfaces, Expression domain, ItemList declared, Boolean exportAll, ExpressionList exports, ItemList terms)
    { return (Domain)build("Domain", DomainSignature, 
             new Object[] { uses, domain, interfaces, declared, terms, exports, exportAll }); }

  /** Create a new Package.
   * @param library The source library of this object.
   * @param uses The use clauses of this facet.
   * @param interfaces The interface items for this facet.
   * @param domain The domain of this facet.
   * @param declared The declared items of this facet.
   * @param exportAll Are all the items in this facet being exported explicitly.
   * @param exports The items being exported from this facet.
   * @return Package The new Package.
   * @deprecated Need to start using the 'library' tag.
   */
  public Package buildPackage(ExpressionList uses, ItemList interfaces, Expression domain, ItemList declared, Boolean exportAll, ExpressionList exports)
    { return (Package)build("Package", PackageSignature,
             new Object[] { uses, domain, interfaces, declared, exports, exportAll }); }

  public Package buildPackage(LabelList library, ExpressionList uses, ItemList interfaces, Expression domain, ItemList declared, Boolean exportAll, ExpressionList exports)
    { return (Package)build("Package", PackageSignature,
             new Object[] { uses, domain, interfaces, declared, exports, exportAll }); }

  /** Create a new Interaction.
   * @param library The source library of this object.
   * @param uses The use clauses of this facet.
   * @param interfaces The interface items for this facet.
   * @param domain The domain of this facet.
   * @param declared The declared items of this facet.
   * @param exportAll Are all the items in this facet being exported explicitly.
   * @param exports The items being exported from this facet.
   * @param terms The term items of this facet.
   * @return Interaction The new Interaction.
   * @deprecated Need to start using the 'library' tag.
   */
  public Interaction buildInteraction(ExpressionList uses, ItemList interfaces, Expression domain, ItemList declared, Boolean exportAll, ExpressionList exports, ItemList terms)
    { return (Interaction)build("Interaction", InteractionSignature, new Object[] { uses, interfaces, domain, declared, exportAll, exports, terms }); }

  public Interaction buildInteraction(LabelList library, ExpressionList uses, ItemList interfaces, Expression domain, ItemList declared, Boolean exportAll, ExpressionList exports, ItemList terms)
    { return (Interaction)build("Interaction", InteractionSignature, new Object[] { uses, interfaces, domain, declared, exportAll, exports, terms }); }

  /** Create a new Component.
   * @param library The source library of this object.
   * @param uses The use clauses of this facet.
   * @param interfaces The interface items for this facet.
   * @param domain The domain of the component.
   * @param assumptExpr The assumption expression.
   * @param defnExpr The definition expression.
   * @param verExpr The verification expression.
   * @return Component The new Component.
   * @deprecated Need to start using the 'library' tag.
   */
  public Component buildComponent(ExpressionList uses, Expression domain, ItemList interfaces, ItemList declarations, Expression assumptExpr, Expression defnExpr, Expression verExpr)
    { return (Component)build("Component", ComponentSignature, new Object[] { uses, domain, interfaces, declarations, assumptExpr, defnExpr, verExpr }); }

  public Component buildComponent(LabelList library, ExpressionList uses, Expression domain, ItemList interfaces, ItemList declarations, Expression assumptExpr, Expression defnExpr, Expression verExpr)
    { return (Component)build("Component", ComponentSignature, new Object[] { uses, domain, interfaces, declarations, assumptExpr, defnExpr, verExpr }); }

  public Component buildComponent(LabelList library, ExpressionList uses, ItemList interfaces, Expression assumptExpr, Expression defnExpr, Expression verExpr)
    { return buildComponent(uses, null, interfaces, null, assumptExpr, defnExpr, verExpr); }



  /** Create a new Item.
   * @param labelIn The label of this item.
   * @param typeIn The type of this item.
   * @param valueIn The value of this item.
   * @return Item The new Item.
   */
  public Item buildItem(Label labelIn, Expression typeIn, Expression valueIn)
    { return (Item)build("Item", ItemSignature, new Object[] { labelIn, typeIn, valueIn, null }); }

  /** Create a new Item.
   * @param labelIn The label of this item.
   * @param typeIn The type of this item.
   * @param valueIn The value of this item.
   * @param valueMod The value modifier of this item.
   * @return Item The new Item.
   */
  public Item buildItem(Label labelIn, Expression typeIn, Expression valueIn, ItemReference valueMod)
    { return (Item)build("Item", ItemSignature, new Object[] { labelIn, typeIn, valueIn, valueMod }); }





  /** Create a new TypeExpression.
   * @param relationship The value of the relationship.
   * @param value The value of the 'super-type' (can be null).
   * @return TypeExpression The new TypeExpression.
   */
  public TypeExpression buildTypeExpression(String relationship, Expression value)
    { return (TypeExpression)build("TypeExpression", TypeExpressionSignature, new Object[] { relationship, value }); }



  /** Create a new SourceReference.
   * @return SourceReference The new SourceReference.
   */
  public SourceReference buildSourceReference(LabelList filename, int beginCol, int beginRow, int endCol, int endRow)
    { return buildSourceReference(filename, new Integer(beginCol), new Integer(beginRow), new Integer(endCol), new Integer(endRow)); }

  public SourceReference buildSourceReference(LabelList filename, Integer beginCol, Integer beginRow, Integer endCol, Integer endRow)
    { return (SourceReference)buildObject("org.sldl.rosetta.parser.tarp.misc.SourceReference", SourceReferenceSignature, new Object[] { filename, beginCol, beginRow, endCol, endRow }); }

  }
