//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: SourceReference.java,v 1.5 2002/11/21 05:34:13 josephk Exp $
//
//  Revision History:
//  ----------------
//  $Log: SourceReference.java,v $
//  Revision 1.5  2002/11/21 05:34:13  josephk
//  Updating KU ANTLR mirror.
//
//  Revision 1.2  2002/09/19 05:17:55  josephk
//  Added methods to propagate a filename top-down through a ROM, and to
//  update the ranges bottom-up through a ROM.
//
//  Revision 1.1  2002/08/27 07:16:07  josephk
//  Moved the various 'one-off' files to a common misc directory.
//
//
//=============================================================================
package org.sldl.rosetta.parser.tarp.misc;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;
import org.sldl.rosetta.ROM.BaseObject;
import org.sldl.rosetta.ROM.LabelList;


/** This class encapsulates the source file information used for tracking
 *  errors between the ROM and the source code. This is important because
 *  when giving a user type- or scope- or other forms of errors during
 *  mutli-pass parsing it is often <I>extremely</I> helpful to give them
 *  a clue where to look in their source code. Basically this information
 *  will help a tool provide a meaningful error location.
 *
 *  This object will be added to a ROM by annotating each BaseObject using
 *  the with the <code>SourceReference.annotationKey</code> field.
 */
public class SourceReference implements Serializable
  {
  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "Name", "StartCol", "StartRow", "EndCol", "EndRow" };

  /** The <code>annotationKey</code> value provides the key that
   *  will be used to add this object to a BaseObject via annotation.
   */
  public static final String annotationKey = "org.sldl.rosetta.parser.tarp.SourceReference";



  private LabelList myName;
  private Integer myStartCol, myStartRow, myEndCol, myEndRow;

  /** Primary Constructor.
   * @param settings A Map containing the default values of the SourceReference.
   */
  public SourceReference(Map settings)
    {
    Map mySettings;

    mySettings = settings;
    if (mySettings == null)
      { mySettings = new HashMap(); }
    myName = (LabelList)mySettings.get(SourceReference.keySet[0]);
    myStartCol = (Integer)mySettings.get(SourceReference.keySet[1]);
    myStartRow = (Integer)mySettings.get(SourceReference.keySet[2]);
    myEndCol = (Integer)mySettings.get(SourceReference.keySet[3]);
    myEndRow = (Integer)mySettings.get(SourceReference.keySet[4]);
    }


  /** Secondary Constructor. See the 'join' method for a full description.
   * @param lhs Must be a non-null SourceReference.
   * @param rhs Another non-null SourceReference, possibly equals lhs.
   */
  protected SourceReference(SourceReference lhs, SourceReference rhs)
    {
    myName = lhs.getName();
    myStartCol = new Integer(Math.min(lhs.getStartCol().intValue(), rhs.getStartCol().intValue()));
    myStartRow = new Integer(Math.min(lhs.getStartRow().intValue(), rhs.getStartRow().intValue()));
    myEndCol = new Integer(Math.min(lhs.getEndCol().intValue(), rhs.getEndCol().intValue()));
    myEndRow = new Integer(Math.min(lhs.getEndRow().intValue(), rhs.getEndRow().intValue()));
    }



  /** Retreive the column of the first character of the construct in the Document.
   * @return Integer The column index of the first character of the construct.
   */
  public Integer getStartCol()
    { return myStartCol; }

  /** Retreive the row of the first character of the construct in the Document.
   * @return Integer The row index of the first character of the construct.
   */
  public Integer getStartRow()
    { return myStartRow; }

  /** Retreive the column of the last character of the construct in the Document.
   * @return Integer The column index of the last character of the construct.
   */
  public Integer getEndCol()
    { return myEndCol; }

  /** Retreive the row of the last character of the construct in the Document.
   * @return Integer The row index of the last character of the construct.
   */
  public Integer getEndRow()
    { return myEndRow; }


  /** Retreive the class name of the Document the construct was taken from.
   * @return LabelList The qualified class name for where this entry was created.
   */
  public LabelList getName()
    { return myName; }

  /** Set the class name of the Document the construct was taken from.
   * @param classname The new class name where this entry was created.
   * @return void
   */
  public void setName(LabelList classname)
    { myName = classname; }



  /** Method to generate a unique string representation of this reference.
   * @return String The generated string.
   */
  public String hashString()
    { return ("___" + this + "___"); }


  /** Method to generate a nice string representation of this reference.
   * @return String The generated string (not necessarily unique).
   */
  public String toString()
    {
    if (myName != null)
      { return (myName.toString(".") + ".[" + myStartRow + "," + myStartCol + "]->[" + myEndRow + "," + myEndCol + "]"); }
    else
      { return ("?[" + myStartRow + "," + myStartCol + "]->[" + myEndRow + "," + myEndCol + "]"); }
    }

  /** Method for creating an entry for compound language constructs from multiple
   *  sub-construct entries. For example in a mutli-part "if" statement the outer-most
   *  expression could <I>join()</I> all the sub-expressions together to correctly
   *  indicate the locations of the expressions within that if-statement. Note that
   *  expressions with an additional 'tail' (like "end let") may optionally create a
   *  throw-away entry of the tail and join it also to get a more asethically pleasing
   *  measure in the source Document. For the purposes of error-detection and
   *  reporting this is clearly not necessary though.
   * @return SourceReference A new entry that spans both this entry and the one given.
   */
  public SourceReference join(SourceReference otherEnd)
    {
    if (otherEnd == null)
      { return new SourceReference(this, this); }
    else
      { return new SourceReference(this, otherEnd); }
    }


  /** Method to propogate the SourceReference information in a ROM tree.
   *  Each ROM leaf with an SR extends all of it's penultimate parents
   *  with that SR information, so that each node has an SR that completely
   *  contains all the area represented by all sub-nodes' SRs.
   * @param root The root of the ROM tree to propogate SRs along.
   * @return void
   */
  public static void propagateSourceReferences(BaseObject root)
    { propagateSourceReferences(new Vector(), root); }

  /** Sub-method to prevent a cycle in a ROM description from causing
   *  the SR propogation method to infinitely loop.
   * @param history The nodes which have already been traversed.
   * @param node The next node to examine the children of.
   */
  protected static void propagateSourceReferences(Vector history, BaseObject node)
    {
    int pos, position;
    BaseObject[] kids;
    SourceReference mySR, nextSR;
    Object decoration;

    if ((history == null) || (node == null))
      { return; }
    for (pos = 0; pos < history.size(); pos++)
      {
      if (history.get(pos) == node)
        { return; }
      }

    mySR = null;
    decoration = node.getDecoration(annotationKey);
    if ((decoration != null) && (decoration instanceof SourceReference))
      { mySR = (SourceReference)decoration; }

    position = history.size();
    history.add(node);
    kids = node.getAllChildObjects();
    for (pos = 0; pos < kids.length; pos++)
      {
      propagateSourceReferences(history, kids[pos]);
      decoration = kids[pos].getDecoration(annotationKey);
      if ((decoration != null) && (decoration instanceof SourceReference))
        { mySR = new SourceReference(mySR, (SourceReference)decoration); }
      }
    history.remove(position);

    node.addDecoration(annotationKey, mySR);
    }


  /** Method to set the filename for all the SourceReferences in a ROM tree.
   * @param root The root of the ROM tree to propogate the filename along.
   * @param filename The new filename to be set to.
   * @return void
   */
  public static void propagateSourceReferencesFilename(BaseObject root, LabelList filename)
    { propagateSourceReferencesFilename(new Vector(), root, filename); }

  /** Sub-method to prevent a cycle in a ROM description from causing
   *  the SR filename propogation method to infinitely loop.
   * @param history The nodes which have already been traversed.
   * @param node The next node to examine the children of.
   * @param filename The new filename to be set to.
   */
  protected static void propagateSourceReferencesFilename(Vector history, BaseObject node, LabelList filename)
    {
    int pos, position;
    BaseObject[] kids;
    Object decoration;

    if ((history == null) || (node == null))
      { return; }
    for (pos = 0; pos < history.size(); pos++)
      {
      if (history.get(pos) == node)
        { return; }
      }

    decoration = node.getDecoration(annotationKey);
    if ((decoration != null) && (decoration instanceof SourceReference))
      { ((SourceReference)decoration).setName(filename); }

    position = history.size();
    history.add(node);
    kids = node.getAllChildObjects();
    for (pos = 0; pos < kids.length; pos++)
      { propagateSourceReferencesFilename(history, kids[pos], filename); }
    history.remove(position);
    }

  }
