//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//      authors: Garrin kimmell@ittc.ku.edu
//               Joseph josephk@cs.adelaide.edu.au
//
//  $Id: ROMTypeChecker.java,v 1.2 2002/07/03 15:59:44 kimmell Exp $
//
//  Revision History:
//  ----------------
//  $Log: ROMTypeChecker.java,v $
//  Revision 1.2  2002/07/03 15:59:44  kimmell
//  1. Made isFailure, message, location non-final variables.
//  2. Set default variables for the above variables.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:36  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.1  2002/01/17 00:22:26  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/04 12:28:06  cvs
//  Prototype straight-to-ROM parser.
//
//
//=============================================================================
package org.sldl.rosetta.parser.typechecker;

import org.sldl.rosetta.ROM.ExpressionList;
import org.sldl.rosetta.ROM.BaseObject;
import org.sldl.rosetta.ROM.Item;
import org.sldl.rosetta.ROM.DeferredItem;
import org.sldl.rosetta.ROM.Label;
import org.sldl.rosetta.ROM.Facet;
import org.sldl.rosetta.ROM.ItemReference;

import org.sldl.rosetta.parser.linker.ROMLinker.LinkerLoader;
//import Parser.ROMLinker.LinkerLoader;

/** This represents the <B>ROMTypeChecker</B> class.
 *  The goal of this object is to serve as a skeleton for both a static type
 *  resolver and a type checker.
 * @author <I> Garrin Kimmell, Joseph Kuehn </I>
 * (kimmell@ittc.ku.edu, josephk@cs.adelaide.edu.au)
 */
public class ROMTypeChecker
  {
  public static final Label langLabel = new Label("lang");
  public static final DeferredItem myLogicItem = new DeferredItem(new Label("logic"), null);


  /** The output of the type checking process is two-fold, first the ROM itself can
   *  have the type-tags in the Expression class altered, but the type checking process
   *  can also generate a list of 'conditions' or 'failures' in the type-correctness
   *  of the model being walked for the user.
   */
  public static class TCC {
    public  boolean isFailure = false; // some things are wrong, others are warnings
    public  String message = "";
    public  BaseObject location = null;

    public TCC(String msg, BaseObject loc, boolean failed)
      { message = msg; location = loc; isFailure = failed; }
    public String toString()
      {
      if (isFailure)
        { return "TCC [failure] because of [" + message + "] at " + location.getSourceReference() + " in [" + location + "]"; }
      else
        { return "TCC [warning] because of [" + message + "] at " + location.getSourceReference() + " in [" + location + "]"; }
      }
    }


  /** This interface decouples the mechanism by which Rosetta models are type checked
   *  from any particular user-interface.
   */
  public static interface TypeCheckerMonitor
    {
    /** Method for outputting uncertain type conditionals detected during type checking.
     * @param requirement The TCC that must be satisfied for this model to be type correct.
     */
    public void addTCC(TCC requirement); // warnings - eg "y::nat = sqrt(x::nat)" + "must prove x square"

    /** Method for outputting errors encountered during type checking.
     * @param errorMessage A string describing the error encountered.
     */
    public void typeCheckingError(String errorMessage);
    }




  /** At each piece of the ROM we attempt to type check it by applying a cascade of
   *  different type checkers. We start with a simple type checker, which if it fails
   *  makes us call a more sophisticated type checker, etc... until we determine that
   *  we are completely unable to type check the BaseObject.
   */
  public static class UnableToTypeCheckException extends Exception
    {
    public UnableToTypeCheckException(String message)
      { super(message); }
    }

  /** This interface describes the mechanisms used for assigning a type to each BaseObject.
   */
  public static interface TypeResolver
    {
    public void typeCheckBaseObject(BaseObject root, TypeCheckerMonitor monitor) throws UnableToTypeCheckException;
    }




  /** A default mode for type-checking. Assumes the monitor is also a LinkerLoader.
   */
  public static void typeCheck(Item rootItem, TypeCheckerMonitor monitor)
    {
    TypeResolver[] myResolvers;
    DeferredItem langImport;

    if (rootItem.getLabel().matches(langLabel))
      { return; }

    langImport = null;
    try
      {
      int pos;
      ExpressionList uses;
      DeferredItem nextLibrary;
      uses = ((Facet)(rootItem.getValue())).getUseClauses();
      for (pos = 0; pos < uses.size(); pos++)
        {
        if (uses.get(pos) instanceof ItemReference)
          {
          nextLibrary = (DeferredItem)(((ItemReference)(uses.get(pos))).getItemReferred());
          if (nextLibrary.getLabel().matches(langLabel))
            { langImport = nextLibrary; }
          }
        }
      langImport.getDotString();
      }
    catch(Exception e)
      {
      monitor.typeCheckingError("Error [" + e + "] trying to load 'lang' library from [" + rootItem + "]");
      return;
      }

    myResolvers = new TypeResolver[1];
    myResolvers[0] = new BasicTypeResolver(langImport);
    typeCheck(rootItem, myResolvers, monitor);
    }



  public static void typeCheck(BaseObject root, TypeResolver[] myResolvers, TypeCheckerMonitor monitor)
    { typeCheckRecursive(root, myResolvers, monitor); }

  private static boolean typeCheckRecursive(BaseObject root, TypeResolver[] myResolvers, TypeCheckerMonitor monitor)
    {
    boolean status;
    int pos;

    if ((root == null) || (myResolvers == null) || (myResolvers.length == 0) || (monitor == null))
      { return false; }

    if (root instanceof DeferredItem)
      { return true; }

    // The leaves need to be type-checked first.
    status = true;
    for (pos = 0; pos < root.getChildCount(); pos++)
      {
      if (!typeCheckRecursive(root.getChildObjectAt(pos), myResolvers, monitor))
        { status = false; }
      }

    for (pos = 0; pos < myResolvers.length; pos++)
      {
      try
        {
        myResolvers[pos].typeCheckBaseObject(root, monitor);
        break;
        }
      catch(UnableToTypeCheckException e)
        { }
      }

    if (status && (pos == myResolvers.length))
      {
      monitor.addTCC(new TCC("All type checkers failed", root, false));
      status = false;
      }

    return status;
    }

  }
