//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
// Revision History:
// $Log: SimplexMethod.java,v $
// Revision 1.2  2002/09/09 15:22:00  cvs
// Added KU License to all the files and made some minor changes.
// --Murthy.
//

package org.sldl.rosetta.tools.generatedata;


import java.lang.*;
import java.util.*;


/**
 * THe class <code>SimplexMethod</code> contains a method which is the
 * implementation of multi-dimension Simplex algorithm, one of the
 * famous numerical optimization algorithms in engineering and science.
 *
 * Created: Mon Mar  4 09:22:35 2002
 *
 * @author <a href="mailto:zwang@kirk.ittc.ku.edu">Zhongjun Wang</a>
 * @version 1.0
 */
public class SimplexMethod extends Object {
  

  /**
   * The <code>SmplxOne</code> method is the Simplex algorithm
   * minimizes f(x) (a method of minclass), where x is one dimensional
   * array with n elements. dx contains the initial perturbation in
   * x. sigma should be set according the tolrance required; when sd <
   * 0 the algorithm calls f(x) m times.
   *
   * @param n an <code>int</code> value, which is the number of
   * variables in the cost functions. The dimension of optimization.
   * @param x[] a <code>double</code> value, which is an array stores
   * the variables in the cost function.
   * @param funcInterface a <code>SimplexFunc</code> value, which is a
   * class implemetns the interface of SimplexFunc.
   * @param dx[] a <code>double</code> value, which ia an array stores
   * the initial search direction in the Simplex algorithm. Generally,
   * dx is about 1/10 of x if none appropriate data is available
   * @param sigma a <code>double</code> value, whcih is the tolerance
   * to terminate the optimization process. Generally, sigma =
   * 0.0000001, if sigma < 0.0, then the cost functions will be called
   * at least m times 
   * @param m an <code>int</code> value, which gives the maximum
   * number of times that the cost function is called.
   * @return a <code>double</code> value, the minimum cost function
   * value obtained is returned.
   */
  public static double SmplxOne(int n, double x[], SimplexFunc funcInterface,
				double dx[],double sigma, int m) {
  //double y0, double ymin){

    double[] xx = new double[n];
    double[] xc = new double[n];
    double[] y  = new double[n+1];
    double[][] v= new double[n+1][n+1];
  

    int nv = n + 1;

    //
    // give the values of matrix v
    //
    for ( int i = 0; i < n; i++)
      {
	for (int j = 0; j < nv; j++)
	  {
	    v[i][j] = x[i];
	  }
	v[i][i+1] = x[i] + dx[i];
      }
    // initialization of y[0:n]

    double y0 = funcInterface.costFunc(x);
    y[0] = y0;


    //    System.out.println( " y0= "+y0);

    for (int j = 1; j < nv; j++)
      {
	for (int i = 0; i < n; i++)
	  xx[i] = v[i][j];
	
	y[j] = funcInterface.costFunc(xx);
      }
    
    //
    // initial number of times to calculate function value
    //
    int k = nv;
    
    // the loop to calculate the minimum value of the destination function 
    
    for ( ; ;)
      {

	//
	// initialize the maximum and minimum function values and
	// their corresponding indexies
	//
	double yh = y[0];
	double yl = y[0];
	
	int nh = 0;
	int nl = 0;
	//
	// find the maximum and minimum values of the function and
	// their corresponding indexies
	//
	for ( int j = 1; j < nv; j++)
	  {
	    if ( y[j] > yh)
	      {
		yh = y[j];
		nh = j;
	      }
	    else if ( y[j] < yl)
	      {
		yl = y[j];
		nl = j;
	      }
	  }

	//	System.out.println(" max= "+yh+"  min="+yl);

	//
	// find the average function value among thoses nv vectors
	//	
	double yb = y[0];
	
	for (int j = 1; j < nv; j++)
	  {
	    yb = yb +y[j];
	  }
	
	yb = yb/(double)nv;
	//
	// find the average least square differences between y[i] and
	// y average
	//
	double d = 0.0;
	
	for (int j = 0; j< nv; j++)
	  {
	    d = d + Math.pow( (y[j]-yb), 2.0 );
	  }
	
	double sda = Math.sqrt(d/(double)nv);
	//
	// if the number of times to calculate the function value is
	// greater than the assigned value m or the avergae least
	// square differences is small than the last iteration,
	// terminate the execution.
	//	
	if ( (k >= m) || (sda <= sigma) )
	  {
	    sigma = sda;
	    m = k;
	    yl = y[nl]; // the minimum value attained
	    for (int i = 0; i< n ; i++)
	      {
		x[i] = v[i][nl]; // output the optimum solution
		return yl;
	      }
	  }

	//
	// prepare the next iteration
	//
	for (int i = 0; i < n; i++)
	  {
	    xc[i] = 0.0;
	    
	    for (int j = 0; j < nv; j++)
	      {
		if ( j != nh )
		  xc[i] = xc[i] + v[i][j];
	      }
	    xc[i] = xc[i]/(double)n;
	  }
	
	for (int i = 0; i < n; i++)
	  {
	    x[i] = xc[i] + xc[i] - v[i][nh];
	  }
	//
	// increment the number of times to call function by 1
	// and perform the next step calculations
	//
	k = k+1;
	
	double yr = funcInterface.costFunc(x);  //initial function value
	
	//System.out.println(" x after 11: "+x[0]+"    "+x[1]);
	//System.out.println(" yr after 11: "+yr);

	if ( yr < yl)
	  {
	    for (int i = 0; i < n; i++)
	      xx[i] = x[i] +x[i] -xc[i];
	    
	    k = k + 1;
	    double ye = funcInterface.costFunc(xx);
	   

	    // System.out.println(" ye after 12"+ ye);
 
	    if ( ye < yr) 
	      {
		y[nh] = ye;
		for (int i = 0; i < n; i++)
		  {
		    v[i][nh] = xx[i];
		  }
	      }
	    else
	      {
		y[nh] = yr;
		for (int i = 0; i < n; i++)
		  {
		    v[i][nh] = x[i];
		  }
	      }
	    continue; //go to 4;
	  }
	
	double y2 = y[nl];
	
	for (int j = 0; j < nv; j++)
	  {
	    if ( (j != nl) && (j != nh) && (y[j] > y2) )
	      y2 = y[j];
	  }
	
	if ( yr < yh) 
	  {
	    y[nh] = yr;
	    for (int i = 0; i < n; i++)
	      {
		v[i][nh] = x[i];
	      }
	    if ( yr < y2) 
	      continue;	      //goto 4;
	  }
	
	for (int i = 0; i < n; i++)
	  {
	    xx[i] = 0.5*(v[i][nh] + xc[i]);
	  }
	
	k = k + 1;
	double yc = funcInterface.costFunc(xx);
	
	if ( yc < yh) 
	  {
	    y[nh] = yc;
	    for (int i = 0; i < n; i++)
	      {
		v[i][nh] = xx[i];
	      }
	  }
	else
	  {
	    for (int j = 0; j < nv; j++)
	      {
		for (int i = 0; i < n; i++)
		  {
		    v[i][j] = 0.5* ( v[i][j] + v[i][nl] );
		  }
		if ( j != nl) 
		  {
		    for (int ii = 0; ii < n; ii++)
		      xx[ii] = v[ii][j];
		    
		    y[j] = funcInterface.costFunc(xx);

		  }
	      }

	    k = k + n;
	  }

      } // end of the for loop
  }    // end of this method
}// end of class

		      

