//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: Linker.java,v 1.2 2002/11/15 16:51:02 kimmell Exp $
//
// Revision History:
// ----------------
// $Log: Linker.java,v $
// Revision 1.2  2002/11/15 16:51:02  kimmell
// Removed references to DataLiteral, since it is now eliminated from the
// ROM.
// -kimmell
//
// Revision 1.1.1.1  2002/07/01 15:47:36  cvs
// Rosetta tools and publications repository
//
// Revision 1.1.2.7  2002/06/07 17:00:43  kimmell
// Made a change to use the File class to eliminate problems with pathnames when moving from Unix to Windows.
// -kimmell
//
// Revision 1.1.2.6  2002/06/06 13:45:47  kimmell
// Added a new constructor that allows the logLevel to be set.
// -kimmell
//
// Revision 1.1.2.5  2002/05/28 21:29:52  kimmell
// Updated the linker to correctly set a facet's extended domain.
// Most of these changes are in bookkeeping code for Linker.java. Most of the implementation is in SymbolTable.java.
// -kimmell
//
// Revision 1.1.2.4  2002/05/24 21:08:56  kimmell
// 1. Added code in process(Facet) so all symbols from the extended domain are exported.
// -kimmell
//
// Revision 1.1.2.3  2002/04/17 19:57:02  cvs
// Commiting minor bugfixes.
// -kimmell
//
// Revision 1.1.2.2  2002/03/21 20:30:55  cvs
// Removed some logging for debugging purposes.
// -kimmell
//
// Revision 1.1.2.1  2002/03/12 22:27:21  cvs
// Added a linker that uses the multi-dispatch visitor.
// -kimmell
//
package org.sldl.rosetta.tools.glinker;

import java.util.Iterator;
import java.util.List;
import java.io.*;


import org.sldl.rosetta.ROM.Application;
import org.sldl.rosetta.ROM.BaseObject;
import org.sldl.rosetta.ROM.Expression;
import org.sldl.rosetta.ROM.ExpressionList;
import org.sldl.rosetta.ROM.Facet;
import org.sldl.rosetta.ROM.Function;
import org.sldl.rosetta.ROM.Item;
import org.sldl.rosetta.ROM.ItemList;
import org.sldl.rosetta.ROM.ItemReference;
import org.sldl.rosetta.ROM.QuantifierReference;

import org.sldl.rosetta.parser.primitive.Primitive;
import org.sldl.rosetta.parser.primitive.PrimitiveProvider;

import org.sldl.rosetta.tools.multivisitor.MultiVisitor;
import org.sldl.rosetta.tools.testparser.TestParser;
/**
 * Linker.java
 *
 *
 * Created: Thu Feb 28 13:43:42 2002
 *
 * @author <a href="mailto:kimmell@ku.edu">Garrin Kimmell</a>
 * @version
 */

public class Linker extends MultiVisitor {

  private SymbolTable baseTable; 

  public Linker (){
    toolName = "LINK";
    baseTable = null;
    logLevel = ERROR;
  }

  public Linker(int level) {
    toolName = "LINK";
    baseTable = null;
    logLevel = level;
  }

  
  /**
   * This <code>process</code> method resolves symbols for a
   * facet/package/domain.
   * The general strategy is:
   * 1. Create a new scope with the passed-in symboltable as its parent
   * 2. Traverse each child, adding it to the symboltable
   * 3. return the symboltable.
   *
   * @param f a <code>Facet</code> value
   * @param s a <code>SymbolTable</code> value
   * @return an <code>Object</code> value
   */
  public SymbolTable process(Facet f, SymbolTable s) {
    SymbolTable scope = new SymbolTable(s, this);
    log(MESSAGE, "Processing a facet " + ((Item)f.getParent()).getLabel().toString());    


    scope.setName(((Item)f.getParent()).getLabel().toString());
    // Import the domain's items to this scope
    ItemReference domain = (ItemReference) f.getDomain();
    if (domain != null) {
      // scope.importScope(domain.getReferenceName());
      log(MESSAGE, "Setting domain to be " + domain.getReferenceName());
      scope.setDomain(domain);
    } else {
      // log(MESSAGE, "Domain is null");
    } // end of else
    

    // Import the items of each use clause
    ExpressionList useClauses = f.getUseClauses();
    if (useClauses != null) {
      for (Iterator useIter = useClauses.iterator(); useIter.hasNext();){
	 ItemReference clause = (ItemReference) useIter.next();
	 scope.importScope(clause.getReferenceName());
      } // end of for (Iterator useIter = useClauses.iterator(); useIter.hasNext();)
      
    } // end of if (useClauses != null)
    
      

    BaseObject[] children = f.getAllChildObjects();
    for (int i = 0; i < children.length; i++) {
      traverse(children[i], scope);
    } // end of for (int i = 0; i < children.length; i++)



    // Modify the scope so that it only includes parameters and
    // exported symbols
    String symbol = null;
    SymbolTable newTable = new SymbolTable(s, this);
    newTable.setName(scope.getName());
    newTable.setDomain(scope.getDomain());
    newTable.setDomainName(scope.getDomainName());
    SymbolTableEntry entry;

    // This is a hack to allow lookups in scopes to see parent scopes
    // symbols
    //if (domain != null) {
    //  newTable.importScope(domain.getReferenceName());
    //} // end of if (domain != null)
    

    // Export the parameters
    ItemList parameters = f.getInterfaceItems();
    // Add the exports
    for (Iterator paramIter = parameters.iterator();
	 paramIter.hasNext();){
      
      
      Object next = paramIter.next();
      symbol = ((Item)next).getLabel().toString();
      
      //log(MESSAGE, "Exporting symbol " + symbol);
      entry = scope.getEntry(symbol);
      if (entry != null) {
	newTable.addEntry(symbol, entry);
      } // end of if (entry != null)
    }

    // Export the declaration items

    if (f.getExportAll()) {
      //log(MESSAGE, "Exporting all symbols");

	ItemList declarations = f.getDeclaredItems();
	for (Iterator declIter = declarations.iterator(); declIter.hasNext();){
	  Item next = (Item) declIter.next();
	  symbol = next.getLabel().toString();
	  entry = scope.getEntry(symbol);
	  if (entry != null) {
	    newTable.addEntry(symbol, entry);
	  }
	} // end of for (Iterator declIter = declarations.iterator(); declIter.hasNext();)
		
      } else {
	log(MESSAGE, "Not exporting all");
	

      
	// Add the exports
	ExpressionList exports = f.getExportItems();
	for (Iterator exportIter = exports.iterator();
	     exportIter.hasNext();){
	  
	  Object next = exportIter.next();
	  try {
	    symbol = ((ItemReference)next).getReferenceName();
	  } catch (ClassCastException e) {
	    log(ERROR, "Export " + next  + " could not be processed.");
	  } // end of try-catch
	  
	  
	  log(MESSAGE, "Exporting symbol " + symbol);
	  entry = scope.getEntry(symbol);
	  if (entry != null) {
	    newTable.addEntry(symbol, entry);
	  } // end of if (entry != null)
	} // end of for (Iterator exportIter = exports.iterator(); exportIter.hasNext();)
	
      } // end of else
    
    return newTable;
  }


  public Object process(Item i, SymbolTable s) {

    //log(MESSAGE, "Traversing " + i);
    SymbolTable valueTable = null; 
    Expression itemValue = i.getValue();
    Expression itemType = i.getType();

    if (itemType != null) {
      traverse(itemType, s);
    }
    
    
    if (itemValue != null) {
      try {
	valueTable = (SymbolTable) traverse(itemValue, s);
      } catch (ClassCastException e) {
	
      } // end of try-catch
    } 
    
    SymbolTableEntry entry = new SymbolTableEntry(i, valueTable);
    
    
    s.addEntry(i.getLabel().toString(), entry);
    return null;
  }


  public Object process(ItemReference i, SymbolTable s) {
    try {
      SymbolTableEntry entry = s.lookup(i.getReferenceName());
      Item item = entry.getValue();
      
      i.setItemReferred(item);
      // log(MESSAGE, "Setting " + i.getReferenceName() + " to " + item);
    } catch (NameNotFoundException e) {
      log(ERROR, "Symbol not found " + i.getReferenceName());
      log(ERROR, i.getSourceReference().toString());
      // loadLibrary(i.getReferenceName());
      // try to load the file from source
    } catch (VariableDerefException e) {
      log(ERROR,  "Cannot dereference " + i.getReferenceName());
    } finally {
      return null;
    } // end of finally
    
    
  }

  public Object process(Function fun, SymbolTable s) {
    SymbolTable scope = new SymbolTable(s, this);

    // scope.setName(((Item)fun.getParent()).getLabel().toString());
    BaseObject[] children = fun.getAllChildObjects();
    for (int i = 0; i < children.length; i++) {
      traverse(children[i], scope);
    } // end of for (int i = 0; i < children.length; i++)

    return scope;
  }

  public Object process(Application app, SymbolTable s) {
    // log(MESSAGE, "Application : " + app.toString());
    traverse(app.getFunction(), s);
    if (app.getParameters() == null) {
      log(DEBUG, "Application parameters is null");
      log(DEBUG, "Application parent: " + app.getParent().toString());
      //log(DEBUG, app.getSourceReference().toString());
    } // end of if
    
    
    for (Iterator paramIter = app.getParameters().iterator(); paramIter.hasNext();){
      traverse(paramIter.next(), s);
    } // end of for (Iterator paramIter = app.getParameters().iterator(); paramIter.hasNext();)
    
    return null;
  }


  public Object process(QuantifierReference ref, SymbolTable s) {
    log(MESSAGE,"Linking a quantifier reference");
    for (Iterator varIter = ref.getVariables().iterator(); varIter.hasNext();){
      traverse((Item) varIter.next(), s);
    } // end of for (Iterator varIter = ref.getVariables().iterator(); varIter.hasNext();)
    
    traverse(ref.getExpression(), s);
    return null;
  }

  public void loadLibrary(String packageName) {
    
    String rosettaPath = System.getProperty("ROSETTA_PATH");
    String sourceFile = rosettaPath + "/" + packageName + ".sld";
    log(MESSAGE, "Loading file: " + sourceFile);
    
    TestParser parser = new TestParser();
    Object root = parser.compileFile(sourceFile);
    traverse(root, baseTable);
  }
    
    

  public void init() {
    if (baseTable == null) {
      TestParser parser = new TestParser();
      Object lang = parser.compileFile("Rosetta/lang.sld");
      
      baseTable = new SymbolTable(null, this);
      traverse(lang, baseTable);
      baseTable.importScope("lang");
      loadPrimitives("Rosetta/primitives.txt", baseTable);
    }
  }

  public SymbolTable link(Object root) {
    init();
    return (SymbolTable) traverse(root, baseTable);
  }
    
    

  public void loadPrimitives(String filename, SymbolTable table) {
    
    String className;
    List primitives;

    log(MESSAGE, "Loading Primitives");

    try {
      FileReader file = new FileReader(filename);
      BufferedReader reader = new BufferedReader(file);
      while ((className = reader.readLine()) != null) {
	// Cut out comments
	className = className.trim();
	int commentBegin = className.indexOf("//");
	if ((commentBegin == -1) || (commentBegin > 0)) {
	  if (commentBegin != -1) {
	    className = className.substring(0,commentBegin);
	  } // end of if (commentBegin != -1)
	  className = className.trim();
	  // log(MESSAGE, "Classname " + className + " commentBegin: " + commentBegin);
	  // Load the primitive
	  try {
	    Class theClass = Class.forName(className);
	    PrimitiveProvider provider = (PrimitiveProvider)
	      theClass.newInstance();
	    primitives = provider.getPrimitives();

	    for (Iterator primIter = primitives.iterator(); primIter.hasNext();){
	      addPrimitive(table, (Primitive) primIter.next());
	    } // end of for (Iterator primIter =
	    // primitives.iterator();

	  } catch (ClassNotFoundException e) {
	    log(ERROR, "Cound not find Primitive class " + className);
	  } catch (InstantiationException e) {
	    log(ERROR, "Could not instantiate Primitive class " + className);
	  } catch (IllegalAccessException e) {
	    log(ERROR, "Could not call getPrimitives() on " + className);
	  } // end of catch
	  
	  
	} // end of if (commentBegin != -1)
	
      } // end of while ((className = reader.readLine()) != null)
      
     
    } catch (IOException  e) {
      log(ERROR, "Could not load primtives file " + filename);
    }
    
    
    
  }

  private void addPrimitive(SymbolTable table, Primitive prim) {
    // log(MESSAGE, "Loading primitive: " + prim.getName());
    try {
      SymbolTableEntry i = table.lookup(prim.getName());
      i.getValue().setValue((Expression) prim);
    } catch (NameNotFoundException e) {
      log(ERROR, "Could not lookup " + prim.getName());
    } catch (VariableDerefException e) {
      log(ERROR, "Variable deref exception " + prim.getName());
    } // end of catch
    
    
  }


  public SymbolTable getBaseTable() {
    return baseTable;
  }

  
  public static void main (String[] args) {
    
    if (args.length != 1) {
      System.out.println("usage: Linker filename");
      return;
    } // end of if (args.length != 1)
    
    Linker linker = new Linker();

    TestParser parser = new TestParser();
    BaseObject root = parser.compileFile(args[0]);
    linker.link(root);

    linker.getBaseTable().print("");

    
  } // end of main ()


  
  
}// Linker
