#!/usr/local/bin/perl

#-----------------------------------------------------------------------------
# Copyright (c) 2000 University of Kansas.
# All rights reserved.

# KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
# THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
# TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
# FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
# DERIVATIVES.

# By using or copying this Software, Licensee agrees to abide by the
# intellectual property laws, and all other applicable laws of the
# U.S., and the terms of this license.
#-----------------------------------------------------------------------------

#
# Author: Darryl Dieckman, ddieckma@ececs.uc.edu
# Updated:  Brandon Morel, morel@ittc.ku.edu
# Comment:  Updated for Rosetta and examples directory

#
#
# This script will run regression tests on the Rosetta parser.
# The script will run clean tests on all the test cases located in the 
# clean subdirectory, and dirty tests on all the test cases located in
# the dirty subdirectory.  As well as the example models.
#
# Clean Tests:
# Clean test cases must pass the following tests:
#    Parse of test case
#    * The parser must return a zero exit status after parsing the test case
#    * The output of the parser must match the expected output
#
#  Dirty Tests:
#    Parse of test case
#    * The parser must return a non-zero exit status when parsing a dirty
#      test case.
#    * The output of the parser must match the expected output
#    
#  Examples follow the same rules as the Clean Tests
#
#

use Cwd;
use File::Copy;

sub LogMsg {
    for($level = 0;$level < $logLevel; $level++){
	print $logfile "  ";
    }
    print $logfile @_[0];
}

sub PushLog {
    $logLevel++;
}

sub PopLog {
    $logLevel--;
}


#===================
#
# Test Results Printer
#
#===================
sub TestResults {
    open(testResults,">> $homedir/test.results");
    local($testLogCase) = shift @_;
    local($testLogExpl) = shift @_;
    local($testLogStatus) = shift @_;
    local($testLogResults) = shift @_;
    local($testLogFiles) = shift @_;
    printf testResults "%3d. %-30s %-30s %s\n"
			,$testLogCount,$testLogCase,$testLogExpl,$testLogStatus;
    $testLogCount++;
    if($testLogStatus eq $failed){
	print testResults "       Explanation:\n";
	print testResults "           $testLogResults\n";
	print testResults "       See Files:\n";
	print testResults "           $testLogFiles\n";
    }
    close(testResults);
}


#===================
#
# Summarize Results Header
#
#===================
sub SummarizeResults {
    
    local($cleanDir) = shift @_;
    open(testResults,">> $homedir/test.results");
    print testResults 
	"==========================================================================\n";
    print testResults 
	"                                  SUMMARY OF $cleanDir\n";
    print testResults 
	"==========================================================================\n";
    close(testResults);
}


#===================
#
# Compare two files
#
#===================
sub CompareFiles {
    
    # The two files
    local($expectFileName) = shift @_;
    local($resultFileName) = shift @_;
    local($status) = 0;
    local($lineNum) = 0;
    
    # Check that files exist
    if(! -e $expectFileName) {
	return -1;
    }
    
    if(! -e $resultFileName) {
	return -2;
    }
    
    # Open files
    open(resultFile,$resultFileName);
    open(expectFile,$expectFileName);

    do {
	$expectLine = <expectFile>;
	$resultLine = <resultFile>;
	$lineNum++;
	
	# Ignore stuff
	if($expectLine =~ /Rosetta Parser, Rudimentary Prototype(.*)/ &&
	   $resultLine =~ /Rosetta Parser, Rudimentary Prototype(.*)/) 
	{
	    # Ignore the version  number
	} elsif($expectLine =~ /Copyright\(C\)(.*)/ &&
		$resultLine =~ /Copyright\(C\)(.*)/){
	    # Ignore the copyright
        } elsif($expectLine =~ /All rights reserved(.*)/ &&
		$resultLine =~ /All rights reserved(.*)/) {
	    # Ignore the rights
	} 
	
        # Lines are different
        elsif($expectLine ne $resultLine){
	    print "Expected line:\n  $expectLine \n resultline \n $resultLine \n";
	    
	    if($status == 0){
		# Dump a header 
		LogMsg("\n");
		LogMsg("==> Differences between expected and actual results\n");
		LogMsg("==> expect file: $expectFile\n");
		LogMsg("==> result file: $resultFile\n");
	    }
	    
	    # Print the difference
	    LogMsg("  expect($lineNum): $expectLine");
	    LogMsg("  result($lineNum): $resultLine");
	    $status = 1;	    
	}
    } while(!eof(expectFile));
    
    # Files have different lengths
    if(eof(resultFile) == 0){
	LogMsg("==> Expected results file shorter than actusl\n");
	LogMsg("==> expect file: $expectFile\n");
	LogMsg("==> result file: $resultFile\n");
	while(!eof(resultFile)){
	    $resultLine = <resultFile>;
	    $lineNum++;
	    LogMsg("  result($lineNum): $resultLine");
	}
	return 1;
    }
    return $status;
}


#===================
#
# Difference Expected File
#
#===================
sub DiffExpected {

    # Get the result output and expected file
    local($resultFile) = shift @_;
    local($expectFile) = shift @_;
    
    # Compare the two files
    local($status) = CompareFiles($expectFile, $resultFile);

    # There are differences between the expected and result files
    if($status == 1){

	# Force the update
	if ($updateExpected eq "force"){
	    print "\n";
	    print "forcing the expected output file to copy data
from result file";
	    LogMsg("Updating expected file $expectFile from
$resultFile\n");
	    
	    # Copy to output to expected directory
	    copy ($resultFile, $expectFile);
	    return 1;
	}

	# Query the results
	elsif ($updateExpected eq "query"){
	    print "\n";
	    print "DO YOU WANT TO UPDATE THE EXPECT FILE $expectFile 
y/n ";
	    if (<> =~ "y"){
		print "coping $expectFile from $resultFile";
		LogMsg("Updating expected file $expectFile from $resultFile\n");
		copy ($resultFile, $expectFile);
		return 0;
	    }
	    LogMsg ("there are differences between the expected output file and the result file ");
	    return 2;
	}

	# The two files are different
	else {
	    LogMsg ("there are differences between the expected output file and the result file "); 
	    return 2;
	}
    } 

    # Error occured
    elsif ($status == -1){

	# Force update
	if ($updateExpected eq "force"){
	    print "\n";
	    print "forcing the expected output file to copy data
from result file";
	    LogMsg("Updating expected file $expectFile from
$resultFile\n");

	    # Copy output to expected directory
	    copy ($resultFile, $expectFile);
	    return 1;
	}

	# Copy expected
	elsif ($updateExpected eq "do"){
	    print "\n";
	    print "coping the $resultFile to the $expectFile";
	    LogMsg("Updating expected file $expectFile from
$resultFile\n since this test file is new");
	    copy ($resultFile, $expectFile);
	    return 3;
	}

        # Expected File not found
	else
	{
	    LogMsg ("The expected file does not exist");
	    return 4;
	}
    }

    # Error
    elsif ($status == -2){
	LogMsg("ERROR! The result '$resultFile' does not exist. Please investigate.");
	return 5;
    } 

    # Completed
    else {
	return 0;
    }
}


#===================
#
# Dirty Tests
#
#===================
sub DirtyTests {

    # Go to clean directory and put all the files into an array
    local($dirtyDir) = shift @_;
    opendir(DIR, $dirtyDir);
    my(@files) = readdir(DIR);
    closedir(DIR);
    
    # Go to test directory
    chdir "$dirtyDir" ||
	die "Can't locate dirty test directory $dirtyDir";
    
    # Check for a results directory, if none, then create one
    if(! (-d "results")){
	LogMsg("Creating results directory\n");
	mkdir("results",0777) ||
	    die "Unable to create results directory";
    }

    # Check for an expect directory, if none, then create one
    if(! (-d "expect")){
	LogMsg("Creating expect directory\n");
	mkdir("expect",0777) ||
	    die "Unable to create expect directory";
    }
    
    # For each .sld file, run ParserDirtyTest on it
    $dir = cwd();
    foreach $file (@files) {
	if($file =~ /\S+\.sld$/i){
	    LogMsg("Testing file: $file\n");
	    ParseDirtyTest($file);
	}
    }
}


#===================
#
# Remove the Results File
#
#===================
sub RemoveResultFile {

    # Get file name
    local($resultFile) = shift @_;
    
    # Unlink it
    if(-e $resultFile){
	unlink($resultFile) ||
	    die "Unable to remove result file $resultFile";
    }
}


#===================
#
# Parse Dirty Test
#
#===================
sub ParseDirtyTest {

    # Local Variables
    PushLog();
    local($fileBase) = shift @_;
    local($passTest) = 1;
    local($parseError) = 0;    
    local($testName) = "Dirty Parse test";
    LogMsg("$testName...");
    
    # Results of the tests variables
    local($resultFile) = "$resultDir$fileBase.parsetest";
    local($expectFile) = "$expectDir$fileBase.parsetest";

    # Remove old test results file
    RemoveResultFile($resultFile);
    
    # Run parser on file
    local($status) = system("$vsp_cmd $file 2> $resultFile 1>> /dev/null");
    
    # File parser thus it failed (dirty logic)
    if($status == 0){
	LogMsg("$failed\n");
	TestResults($fileBase, $testName, $failed, 
		    "The parser accepted a file that should contain and error",
		    $resultFile);
	$passTest = 0;
    } 
    
    # Did not parser thus it passed (dirty logic)
    else {
	LogMsg("$passed\n");
	TestResults($fileBase, $testName, $passed,"","");
    }
    
    # Test output against expected output
    $testName = "Expected parse output";
    LogMsg("$testName...");
    $status = DiffExpected($resultFile, $expectFile);

    # Passed
    if($status == 0){
	LogMsg("$passed\n");
	TestResults($fileBase, $testName, $passed, "", "");
      } 
    
    # Forced Update
    elsif ($status == 1){
	LogMsg("$passed \n");
	TestResults($fileBase, $testName, $passed, 
		    "forcefully coping the expected output file from the results",
		    $expectFile);
    } 

    # Failed
    elsif ($status == 2) {
	LogMsg("$failed\n");
	$passTest = 0;
	TestResults($fileBase, $testName, $failed, 
		    "There are differences between the expected parser output and the output of the test.",
		    $expectFile);
      } 

    # New File
    elsif ($status == 3){
	LogMsg("$passed\n");
	TestResults($fileBase, $testName, $passed,
		    "the test file is new and so the expected output file is copied from the result file",
		    $expectFile);
    }

    # Expected File not found
    elsif ($status == 4){
	LogMsg("$failed \n");
	$passTest =0;
	TestResults($fileBase, $testName, $failed,
		    "The expected file does not exist! please investigate",
		    $expectFile);	
    }
    
    # Results file not found
    elsif ($status == 5){
	LogMsg("$failed \n");
	$passTest =0;
	TestResults($fileBase, $testName, $failed,
		    "The result  file doesnot exist! please investigate",
		    $resultFile);
    }

    PopLog();
    return $passTest;
}


#===================
#
# Parse Test
#
#===================
sub ParseTest {

    # Local Variables
    PushLog();
    local($fileBase) = shift @_;
    local($passTest) = 1;
    local($parseError) = 0;
    local($testName) = "Parse test";
    LogMsg("$testName...");
    
    # Results of the tests variables
    local($resultFile) = "$resultDir$fileBase.parsetest";
    local($expectFile) = "$expectDir$fileBase.parsetest";

    # Remove old test results file
    RemoveResultFile($resultFile);

    # Run parser on file
    local($status) = system("$vsp_cmd $file 2> $resultFile 1>> /dev/null");
    
    # Passed
    if($status == 0){
	LogMsg("$passed\n");
	TestResults($fileBase, $testName, $passed, "","");
    } 

    # Failed
    else {
	LogMsg("$failed\n");
	TestResults($fileBase, $testName, $failed, 
		    "The parser returned an error when parsing this SLDL file",
		    $resultFile);
	$passTest = 0;
	$parseError = 1;
    }
    
    # Test output against expected output
    $testName = "Expected parse output";
    LogMsg("$testName...");
    $status = DiffExpected($resultFile, $expectFile);

    # Passed
    if($status == 0){
	LogMsg("$passed\n");
	TestResults($fileBase, $testName, $passed, "", "");
    }
    
    # Forced Update
    elsif ($status == 1){
	LogMsg("$passed \n");
	TestResults($fileBase, $testName, $passed, 
		    "forcefully coping the expected output file from the results",
		    $expectFile);
    }
    
    # Failed
    elsif ($status == 2) {
	LogMsg("$failed\n");
	$passTest = 0;
	TestResults($fileBase, $testName, $failed, 
		    "There are differences between the expected parser output and the output of the test",$expectFile);
      }

    # New File
    elsif ($status == 3){
	LogMsg("$passed\n");
	TestResults($fileBase, $testName, $passed,
		    "the test file is new and so the expected output file is copied from the result file",
		    $expectFile);
    }

    # Expected File not found
    elsif ($status == 4){
	LogMsg("$failed \n");
	$passTest =0;
	TestResults($fileBase, $testName, $failed,
		    "The expected file does not exist! please investigate",
		    $expectFile);	
    }
    
    # Result file not found
    elsif ($status == 5){
	LogMsg("$failed \n");
	$passTest =0;
	TestResults($fileBase, $testName, $failed,
		    "The result file does not exist! please investigate",
		    $resultFile);
    }
    
    PopLog();  
    return $parseError;
}


#===================
#
# Clean Test - General
#
#===================
sub CleanTests {

    # Go to clean directory and put all the files into an array
    local($cleanDir) = shift @_;
    opendir(DIR, $cleanDir);
    my(@files) = readdir(DIR);
    closedir(DIR);

    # Goto satcom's directory
    chdir "$cleanDir" ||
	die "Can't locate clean test directory $cleanDir";
    
    # Check for a results directory, if none, then create one
    if(! (-d "results")){
	LogMsg("Creating results directory\n");
	mkdir("results",0777) ||
	    die "Unable to create results directory";
    }

    # Check for an expect directory, if none, then create one
    if(! (-d "expect")){
	LogMsg("Creating expect directory\n");
	mkdir("expect",0777) ||
	    die "Unable to create expect directory";
    }

    # For each .sld file, run ParseTest on it
    foreach $file (@files) {
	if($file =~ /\S+\.sld$/i){
	    LogMsg("Testing file: $file\n");

	    if(ParseTest($file) == 0){
	    } else {
		# Could not parse, skip other tests
		LogMsg("Skipping other tests\n");
	    }
	}
    }
}

#===================
#
# Clean Test - SatComm
#
#===================
sub CleanTestsSatcom {

    # Local variables (satcom directory and file array
    local($cleanDir) = shift @_;
    local(@files);
    
    # Files must be arrange in array because order is critical
    $files[1] ="signalTypes.sld";
    $files[2] ="decoder.sld";
    $files[3] ="matlabpackage.sld";
    $files[4] ="testrequirements.sld";
    $files[5] ="costasLoop.sld";
    $files[6] ="carrierRecovery.sld";
    $files[7] ="components.sld";
    $files[8] ="accessorfacets.sld";
    $files[9] ="sigNoise_REQ.sld";

    # Goto satcom's directory
    chdir "$cleanDir" ||
	die "Can't locate clean test directory $cleanDir";
    
    # Check for a results directory, if none, then create one
    if(! (-d "results")){
	LogMsg("Creating results directory\n");
	mkdir("results",0777) ||
	    die "Unable to create results directory";
    }

    # Check for an expect directory, if none, then create one
    if(! (-d "expect")){
	LogMsg("Creating expect directory\n");
	mkdir("expect",0777) ||
	    die "Unable to create expect directory";
    }
    
    # For each .sld file, run ParseTest on it
    foreach $file (@files) {
	if($file =~ /\S+\.sld$/i){
	    LogMsg("Testing file: $file\n");

	    if(ParseTest($file) == 0){
	    } else {
		# Could not parse, skep other tests
		LogMsg("Skipping other tests\n");
	    }
	}
    }
}


#===================
#
# Main Program
#
#===================

# Get command options variable
$updateExpected = "false";
@ARGV = ('-') unless @ARGV;
while($ARGV = shift) {
    if($ARGV =~ /--help/ || $ARGV =~ /-help/ || $ARGV =~ /-h/){
	print "Usage:\n";
	print "  --help          print this message\n";
	print "  -force_update   Overwrite the expected results with the\n";
	print "                  generated results\n";
	print "  -query_update   Ask the use if the expected results should be\n";
	print "                  updated when they differ from the actual results\n";
	
    } elsif($ARGV =~ /-force_update/){
	$updateExpected = "force";
    } elsif($ARGV =~ /-query_update/){
	$updateExpected = "query";
    } elsif ($ARGV =~ /-do_update/){
	$updateExpected = "do";
    }
}

# Current directory variable (/test)
$curdir = cwd();

# Run command variable
$sldlhome = "/projects/sldg/sldl/unstable/Rosetta/tools";
$javahome = "/tools/java/i686/jdk1.2.2";
$javacchome = "/tools/javacc";
$ENV{"SLDL_HOME"} = $sldlhome;
$ENV{"JAVA_HOME"} = $javahome;
$ENV{"JAVACC_HOME"} = $javacchome;
$path=$ENV{"PATH"};
$ENV{"PATH"} = "$path:$javahome/bin";
$vsp_cmd = "$curdir/../bin/parseAntlr";
(-e $vsp_cmd) || 
    die "could not find sldl in $vsp_cmd.\n";

# Directory variables/run variables
$logfile = STDERR;
$logLevel = 0;
$resultDir = "results/";
$expectDir = "expect/";
$passed = "PASSED";
$failed = "FAILED";
$homedir = cwd();

# Remove old test results file
RemoveResultFile("$homedir/test.results");
chdir($homedir);

# Test SatCom
SummarizeResults("SATCOM EXAMPLE");
$testLogCount = 1;    
CleanTestsSatcom("$sldlhome/../publications/examples/satcom/");

# Test Actuator
SummarizeResults("ACTUATOR EXAMPLE");
$testLogCount = 1;
CleanTests("$sldlhome/../publications/examples/actuator");

# Test Alarm Clock
SummarizeResults("ALARM CLOCK EXAMPLE");
$testLogCount = 1;
CleanTests("$sldlhome/../publications/examples/alarm");

# Test Spring
SummarizeResults("SPRING EXAMPLE");
$testLogCount = 1;
CleanTests("$sldlhome/../publications/examples/spring");

# Test Clean
SummarizeResults("CLEAN DIRECTORY");
$testLogCount = 1;
chdir($homedir);
CleanTests("clean");

# Test Dirty
SummarizeResults("DIRTY DIRECTORY");
$testLogCount = 1;
chdir($homedir);
DirtyTests("dirty");





















