#!/usr/local/bin/perl

#-----------------------------------------------------------------------------
# Copyright (c) 2000 University of Kansas.
# All rights reserved.

# KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
# THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
# TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
# FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
# DERIVATIVES.

# By using or copying this Software, Licensee agrees to abide by the
# intellectual property laws, and all other applicable laws of the
# U.S., and the terms of this license.
#-----------------------------------------------------------------------------

#
# Author: Makarand Patil, makarand@ittc.ukans.edu

# script will run regression tests on the SLDL parser.
#
# Clean Tests:
# Clean test cases must pass the following tests:
#    Parse of test case
#    * The parser must return a zero exit status after parsing the test case
#
#    Serialization of SLDL Object Model (SOM)
#    * The parser must return a zero exit status when serializing SOM object module 
#    * The serialized object must be deserialzed without error
#    * After deserializaion the publishSOM method is invoked to generate SOM
#    * The SOM (before serialization and after deserialization) should show no differences
#


use Cwd;
use File::Copy;

sub LogMsg {
    for($level = 0;$level < $logLevel; $level++){
	print $logfile "  ";
    }
    print $logfile @_[0];
}
sub PushLog {
    $logLevel++;
}
sub PopLog {
    $logLevel--;
}
$testLogCount = 1;
@testLogCase;
@testLogExpl;
@testLogFiles;
@testLogStatus;
sub TestResults {
    # Test Case
    # Explanation of test
    # Status (PASS/FAIL)
    # Results - reason for failure
    # Relevant files
    $testLogCase[$testLogCount] = shift @_;
    $testLogExpl[$testLogCount] = shift @_;
    $testLogStatus[$testLogCount] = shift @_;
    $testLogResults[$testLogCount] = shift @_;
    $testLogFiles[$testLogCount] = shift @_;
    $testLogCount++;
}
sub SummarizeResults {
    open(testResults,">serializetest.results");
    print testResults 
	"==========================================================================\n";
    print testResults 
	"                                  SUMMARY\n";
    print testResults 
	"==========================================================================\n";
    
    for($i = 1; $i < $testLogCount; $i++){
	printf testResults "%3d. %-30s %-30s %s\n"
	    ,$i,$testLogCase[$i],$testLogExpl[$i], $testLogStatus[$i];
	
	if($testLogStatus[$i] eq $failed){
	    #
	    # Print explanation of error
	    #
	    print testResults "       Explanation:\n";
	    for $array_ref (@testLogResults[$i]){
		foreach $text (@$array_ref){
		    print testResults "           $text\n";
		}
	    }
	    #
	    # Print list of relevant files
	    #
	    print testResults "       See Files:\n";
	    for $array_ref (@testLogFiles[$i]){
		foreach $file (@$array_ref){
		    print testResults "           $file\n";
		}
	    }
	}
    }
    close(testResults);
    # system("less serializetest.results");
    LogMsg("Summary of test results stored in 'serializetest.results'\n");
}

# this routine needs serious clean up
sub CompareFiles {
    local($expectFileName) = shift @_;
    local($resultFileName) = shift @_;
    local($status) = 0;
    local($lineNum) = 0;
    
    # Check that files exist
    if(! -e $expectFileName){
	return -1;
    }
    if(! -e $resultFileName){
	return -2;
    }
    
    open(resultFile,$resultFileName);
    open(expectFile,$expectFileName);
    do {
	$expectLine = <expectFile>;
	$resultLine = <resultFile>;
	$lineNum++;
	if($expectLine =~ /Rosetta Parser, ver(.*)/ &&
	   $resultLine =~ /Rosetta Parser, ver(.*)/){
	    # Ignore the version  number
	} elsif($expectLine =~ /Copyright\(C\)(.*)/ &&
		$resultLine =~ /Copyright\(C\)(.*)/){
	    # Ignore the copyright
	} elsif($expectLine ne $resultLine){
	  print "Expected line:\n  $expectLine \n resultline \n $resultLine \n";
	  <>; 
	    if($status == 0){
		# Dump a header 
		LogMsg("\n");
		LogMsg("==> Differences between expected and actual results\n");
		LogMsg("==> expect file: $expectFile\n");
		LogMsg("==> result file: $resultFile\n");
	    }
	    # Print the difference
	    LogMsg("  expect($lineNum): $expectLine");
	    LogMsg("  result($lineNum): $resultLine");
	    $status = 1;
	    
	}
	
    } while(!eof(expectFile));
    
    if(eof(resultFile) == 0){
	LogMsg("==> Expected results file shorter than actusl\n");
	LogMsg("==> expect file: $expectFile\n");
	LogMsg("==> result file: $resultFile\n");
	while(!eof(resultFile)){
	    $resultLine = <resultFile>;
	    $lineNum++;
	    LogMsg("  result($lineNum): $resultLine");
	}
	return 1;
    }
    
    return $status;
}

sub DiffExpected {
    local($resultFile) = shift @_;
    local($expectFile) = shift @_;
    
    
    local($status) = CompareFiles($expectFile, $resultFile);
    
    
    if($status == 1){
	return 1;
    } elsif ($status == -1){
	if($updateExpected ne "false"){
	    print "\n";
	    print "The expected output file '$expectFile' does not exist.\n";
	    print "Using current results to create expected file.\n";
	    print "Copying $resultFile to $expectFile\n";
	    LogMsg("Creating expected file $expectFile from $resultFile\n");
	    copy($resultFile, $expectFile);
	} else {
	    LogMsg("ERROR! The expected file '$expectFile' does not exist! Please investigate.");
	}
	return -1;
    } elsif ($status == -2){
	LogMsg("ERROR! The result '$resultFile' does not exist. Please investigate.");
	return -1;
    } else {
	return 0;
    }
    
    
}


sub RemoveResultFile {
    local($resultFile) = shift @_;
    
    if(-e $resultFile){
	unlink($resultFile) ||
	    die "Unable to remove result file $resultFile";
    }
    
}

 
sub ParseTest {
    PushLog();
    # Process arguments
    local($fileBase) = shift @_;
    local($passTest) = 1;
    local($parseError) = 0;
    
    local($testName) = "Parse test";
    LogMsg("$testName...");
    
    
    local($resultFile) = "$resultDir$fileBase.parsetest";
    local($expectFile) = "$expectDir$fileBase.parsetest";
    RemoveResultFile($resultFile);
    local($status) = system("$som_cmd $file 2> /dev/null 1>> /dev/null");
    
    if($status == 0){
	LogMsg("$passed\n");
	TestResults($fileBase, $testName, $passed, [], []);
    } else {
	LogMsg("$failed\n");
	TestResults($fileBase, $testName, $failed, 
		    ["The parser returned an error when parsing this SLDL file"],
		    [$resultFile]);
	$passTest = 0;
	$parseError = 1;
    }
    
    PopLog();
    
    return $parseError;
}

# makarand 3/2/2000 -for testing Sldl object model Serialization 
sub SOMTest {
    PushLog();
    # Process arguments
    local($fileBase) = shift @_;
    local($passTest) = 0;
    local($objfile) = "ObjectModel.txt";
    # output file from DeserializeSOM is <filename>.obj
		local($Namestring) = substr($fileBase,0,-4);
		local($deserout) = "$Namestring.obj";
    local($testName) = "Parsing and SOM Serialization ";
    LogMsg("$testName...");
    local($resultFile) = "$resultDir$fileBase.sertest";
    local($expectFile) = "$expectDir$fileBase.sertest";
    RemoveResultFile($resultFile);
    RemoveResultFile($expectFile);
    # local($status) = system("$som_cmd -print-som $file 1> $resultFile 2> /dev/null");
    local($status) = system("$som_cmd $file 1> /dev/null 2> /dev/null");
    
    if($status == 0){
	LogMsg("$passed\n");
	TestResults($fileBase, $testName, $passed, [], []);
    } else {
	LogMsg("$failed\n");
	$passTest = 0;
	TestResults($fileBase, $testName, $failed, 
		    ["The parser returned an error when publishing SOM"],
		    [$resultFile]);
    }
		
    local($status) = system("mv $objfile $resultFile");

    $testName = "DeSerializing and publishing SOM";
    LogMsg("$testName...");
    local($status) = system("$des_cmd $file 2> /dev/null 1>> /dev/null");
    if($status == 0){
	LogMsg("$passed\n");
	TestResults($fileBase, $testName, $passed, [], []); 
    } else {
	$passTest = 0;
	LogMsg("$failed\n");
	TestResults($fileBase, $testName, $failed, 
		    ["The parser was unable to re-parse the generated SOM"],
		    [$resultFile]);
    }

    local($status) = system("mv $deserout $expectFile");
    $testName = "Comparing SOM after Deserialization";
    LogMsg("$testName...");
    $status = DiffExpected($resultFile, $expectFile);
    if($status == 0){
	LogMsg("$passed\n");
	TestResults($fileBase, $testName, $passed, [], []);
    } elsif($status == -1){
	$passTest = 0;
	LogMsg("$failed\n");
	TestResults($fileBase, $testName, $failed, 
		    ["Unable to locate the expected output file!"],
		    [$expectFile]);
    } else {
	$passTest = 0;
	LogMsg("$failed\n");
	TestResults($fileBase, $testName, $failed, 
		    ["There is a difference between the published SOM file",
		     "generated by the parser, and the reparsed new SOM file ",
		     "generated by the parser."],
		    [$resultFile, $expectFile]);
    }
    
    PopLog();
    
    return $passTest;
}

sub CleanTests {
    # Process arguments
    local($cleanDir) = shift @_;
    
    #
    # Get list of test files
    #
    opendir(DIR, $cleanDir);
    my(@files) = readdir(DIR);
    closedir(DIR);
    
    # Go to test director
    chdir "$cleanDir" ||
	die "Can't locate clean test directory $cleanDir";
    
    # Setup results directory
    if(! (-d "results")){
	LogMsg("Creating results directory\n");
	mkdir("results",0777) ||
	    die "Unable to create results directory";
    }
    
    # Setup resultsSOM and expectSOM directories
    if(! (-d "SERresults")){
	LogMsg("Creating resultsSOM directory\n");
	mkdir("SERresults",0777) ||
	    die "Unable to create resultsSOM directory";
    }
    if(! (-d "SERexpect")){
	LogMsg("Creating expectSOM directory\n");
	mkdir("SERexpect",0777) ||
	    die "Unable to create expectSOM directory";
    }

    # Process test files
    foreach $file (@files) {
	if($file =~ /\S+\.sld$/i){
	    LogMsg("Testing file: $file\n");
	    #
	    # Run parse tests
	    #
	    if(ParseTest($file) == 0){
		
	      # Run SOMTests
	      SOMTest($file);
	    } else {
		LogMsg("Skipping other tests\n");
	    }
	}
    }
}


#===================
#
# Main Program
#
#===================

#
# Global flags
#
$updateExpected = "false";

#
# Process Arguments
#
@ARGV = ('-') unless @ARGV;
while($ARGV = shift){
    if($ARGV =~ /--help/ || $ARGV =~ /-help/ || $ARGV =~ /-h/){
	print "Usage:\n";
	print "  --help     print this message\n";
	print "  -force_update   Overwrite the expected results with the\n";
	print "                  generated results\n";
	print "  -query_update   Ask the use if the expected results should be\n";
	print "                  updated when they differ from the actual results\n";
	
    } elsif($ARGV =~ /-force_update/){
	$updateExpected = "force";
    } elsif($ARGV =~ /-query_update/){
	$updateExpected = "query";
    }
    
}


#
# Setup environment variables
#
#  $uctoolsroot_env_var = "UCTOOLSROOT";
#  defined($uctoolsroot = $ENV{$uctoolsroot_env_var}) || 
#      die "Environment Variable UCTOOLSROOT Undefined.\n";

#
# Locate executables
#
$curdir = cwd();
#$som_cmd = "/users/makarand/projects/sldl-lrm/grammar/src/sldl";
$som_cmd = "$curdir/../src/sldl";
$des_cmd = "$curdir/../src/deserialize";

# $lsl_cmd = "/bin/lsl";

(-e $som_cmd) || 
    die "could not find sldl in $som_cmd.\nCheck your UCTOOLSROOT env variable\n"; 
#(-e $lsl_cmd) || 
#    die "could not find lsl in $lsl_cmd.\nCheck your UCTOOLSROOT env variable\n"; 

#
# Setup global variables
#
$logfile = STDERR;
$logLevel = 0;
$resultDir = "SERresults/";
$expectDir = "SERexpect/";
$passed = "PASSED";
$failed = "FAILED";

$homedir = cwd();

#
# Run SOMpublish Test
#
chdir($homedir);
local($status) = system("\\rm -f ../test/clean/SERresults/*test");
local($status) = system("\\rm -f ../test/clean/SERexpect/*test");

if(-d "clean"){
  CleanTests("clean");
}else {
  LogMsg("Unable to locate 'clean' test directory!\n");
}

#
# Output summary
#
chdir($homedir);
SummarizeResults();































